#!/usr/bin/env node
/******/ (() => { // webpackBootstrap
/******/ 	"use strict";
/******/ 	var __webpack_modules__ = ({

/***/ "./apps/interpreter/src/cli-util.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.extractAstNode = exports.extractDocument = exports.ExitCode = void 0;
const tslib_1 = __webpack_require__("tslib");
const fs = __webpack_require__("fs");
const path = __webpack_require__("path");
const vscode_languageserver_protocol_1 = __webpack_require__("vscode-languageserver-protocol");
const vscode_uri_1 = __webpack_require__("vscode-uri");
var ExitCode;
(function (ExitCode) {
    ExitCode[ExitCode["SUCCESS"] = 0] = "SUCCESS";
    ExitCode[ExitCode["FAILURE"] = 1] = "FAILURE";
})(ExitCode = exports.ExitCode || (exports.ExitCode = {}));
function extractDocument(fileName, services, logger) {
    var _a;
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const extensions = services.LanguageMetaData.fileExtensions;
        if (!extensions.includes(path.extname(fileName))) {
            const errorMessage = `Please choose a file with ${extensions.length === 1 ? 'this extension' : 'one of these extensions'}: ${extensions.map((extension) => `"${extension}"`).join(',')}`;
            logger.logErr(errorMessage);
            process.exit(ExitCode.FAILURE);
        }
        if (!fs.existsSync(fileName)) {
            logger.logErr(`File ${fileName} does not exist.`);
            process.exit(ExitCode.FAILURE);
        }
        const document = services.shared.workspace.LangiumDocuments.getOrCreateDocument(vscode_uri_1.URI.file(path.resolve(fileName)));
        yield services.shared.workspace.DocumentBuilder.build([document], {
            validationChecks: 'all',
        });
        const diagnostics = (_a = document.diagnostics) !== null && _a !== void 0 ? _a : [];
        const errDiagnostics = diagnostics.filter((diagnostic) => diagnostic.severity === vscode_languageserver_protocol_1.DiagnosticSeverity.Error);
        if (errDiagnostics.length > 0) {
            for (const errDiagnostic of errDiagnostics) {
                logger.logLanguageServerDiagnostic(errDiagnostic, document);
            }
            process.exit(ExitCode.FAILURE);
        }
        const nonErrDiagnostics = diagnostics.filter((diagnostic) => diagnostic.severity !== vscode_languageserver_protocol_1.DiagnosticSeverity.Error);
        for (const nonErrDiagnostic of nonErrDiagnostics) {
            logger.logLanguageServerDiagnostic(nonErrDiagnostic, document);
        }
        return document;
    });
}
exports.extractDocument = extractDocument;
function extractAstNode(fileName, services, logger) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        return (yield extractDocument(fileName, services, logger)).parseResult
            .value;
    });
}
exports.extractAstNode = extractAstNode;


/***/ }),

/***/ "./apps/interpreter/src/interpreter.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.logPipelineOverview = exports.useStdExtension = exports.runAction = void 0;
const tslib_1 = __webpack_require__("tslib");
const jayvee_execution_1 = __webpack_require__("./libs/execution/src/index.ts");
const R = __webpack_require__("./libs/execution/src/index.ts");
const exec_1 = __webpack_require__("./libs/extensions/std/exec/src/index.ts");
const lang_1 = __webpack_require__("./libs/extensions/std/lang/src/index.ts");
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
const chalk = __webpack_require__("chalk");
const node_1 = __webpack_require__("langium/node");
const cli_util_1 = __webpack_require__("./apps/interpreter/src/cli-util.ts");
const logger_factory_1 = __webpack_require__("./apps/interpreter/src/logging/logger-factory.ts");
const runtime_parameter_util_1 = __webpack_require__("./apps/interpreter/src/runtime-parameter-util.ts");
function runAction(fileName, options) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const loggerFactory = new logger_factory_1.LoggerFactory(options.debug);
        useStdExtension();
        (0, jayvee_execution_1.registerDefaultConstraintExecutors)();
        const services = (0, jayvee_language_server_1.createJayveeServices)(node_1.NodeFileSystem).Jayvee;
        const model = yield (0, cli_util_1.extractAstNode)(fileName, services, loggerFactory.createLogger());
        const requiredRuntimeParameters = (0, runtime_parameter_util_1.extractRequiredRuntimeParameters)(model);
        const parameterReadResult = (0, runtime_parameter_util_1.extractRuntimeParameters)(requiredRuntimeParameters, options.env, loggerFactory.createLogger());
        if (parameterReadResult === undefined) {
            process.exit(cli_util_1.ExitCode.FAILURE);
        }
        const interpretationExitCode = yield interpretJayveeModel(model, parameterReadResult, loggerFactory);
        process.exit(interpretationExitCode);
    });
}
exports.runAction = runAction;
function useStdExtension() {
    (0, jayvee_language_server_1.useExtension)(new lang_1.StdLangExtension());
    (0, jayvee_execution_1.useExtension)(new exec_1.StdExecExtension());
}
exports.useStdExtension = useStdExtension;
function interpretJayveeModel(model, runtimeParameters, loggerFactory) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const pipelineRuns = model.pipelines.map((pipeline) => {
            return runPipeline(pipeline, runtimeParameters, loggerFactory);
        });
        const exitCodes = yield Promise.all(pipelineRuns);
        if (exitCodes.includes(cli_util_1.ExitCode.FAILURE)) {
            return cli_util_1.ExitCode.FAILURE;
        }
        return cli_util_1.ExitCode.SUCCESS;
    });
}
function runPipeline(pipeline, runtimeParameters, loggerFactory) {
    var _a, _b;
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const executionContext = new jayvee_execution_1.ExecutionContext(pipeline, loggerFactory.createLogger(), runtimeParameters);
        logPipelineOverview(pipeline, runtimeParameters, executionContext.logger);
        const executionOrder = (0, jayvee_language_server_1.getBlocksInTopologicalSorting)(pipeline).map((block) => {
            return { block: block, value: jayvee_execution_1.NONE };
        });
        for (const blockData of executionOrder) {
            const block = blockData.block;
            executionContext.enterNode(block);
            const blockExecutor = (0, jayvee_execution_1.createBlockExecutor)(block);
            const parentData = (0, jayvee_language_server_1.collectParents)(block).map((parent) => executionOrder.find((blockData) => parent === blockData.block));
            const inputValue = ((_a = parentData[0]) === null || _a === void 0 ? void 0 : _a.value) === undefined ? jayvee_execution_1.NONE : (_b = parentData[0]) === null || _b === void 0 ? void 0 : _b.value;
            let result;
            // Check, if parent emitted a value
            if (inputValue != null) {
                try {
                    result = yield blockExecutor.execute(inputValue, executionContext);
                }
                catch (unexpectedError) {
                    executionContext.logger.logErrDiagnostic(`An unknown error occurred: ${unexpectedError instanceof Error
                        ? unexpectedError.message
                        : JSON.stringify(unexpectedError)}`, { node: blockData.block, property: 'name' });
                    return cli_util_1.ExitCode.FAILURE;
                }
                if (R.isErr(result)) {
                    executionContext.logger.logErrDiagnostic(result.left.message, result.left.diagnostic);
                    return cli_util_1.ExitCode.FAILURE;
                }
                blockData.value = result.right;
                // If parent emitted no value, skip all downstream blocks
            }
            else {
                blockData.value = null;
                executionContext.logger.logInfoDiagnostic(`Skipped execution because parent block ${parentData[0] ? parentData[0].block.name : 'NAME NOT FOUND'} emitted no value.`, { node: blockData.block, property: 'name' });
            }
            executionContext.exitNode(block);
        }
        return cli_util_1.ExitCode.SUCCESS;
    });
}
function logPipelineOverview(pipeline, runtimeParameters, logger) {
    const toString = (block, depth = 0) => {
        const blockString = `${'\t'.repeat(depth)} -> ${block.name} (${block.type.name})`;
        const childString = (0, jayvee_language_server_1.collectChildren)(block)
            .map((child) => toString(child, depth + 1))
            .join('\n');
        return blockString + '\n' + childString;
    };
    const linesBuffer = [];
    linesBuffer.push(chalk.underline('Overview:'));
    if (runtimeParameters.size > 0) {
        linesBuffer.push(`\tRuntime Parameters (${runtimeParameters.size}):`);
        for (const key of runtimeParameters.keys()) {
            linesBuffer.push(`\t\t${key}: ${runtimeParameters.has(key)
                ? // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                    runtimeParameters.get(key).toString()
                : 'undefined'}`);
        }
    }
    linesBuffer.push(`\tBlocks (${pipeline.blocks.length} blocks with ${pipeline.pipes.length} pipes):`);
    for (const block of (0, jayvee_language_server_1.collectStartingBlocks)(pipeline)) {
        linesBuffer.push(toString(block, 1));
    }
    logger.logInfo(linesBuffer.join('\n'));
}
exports.logPipelineOverview = logPipelineOverview;


/***/ }),

/***/ "./apps/interpreter/src/logging/default-logger.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.DefaultLogger = void 0;
const jayvee_execution_1 = __webpack_require__("./libs/execution/src/index.ts");
const chalk = __webpack_require__("chalk");
const errors_1 = __webpack_require__("langium/lib/utils/errors");
const vscode_languageserver_types_1 = __webpack_require__("vscode-languageserver-types");
class DefaultLogger extends jayvee_execution_1.Logger {
    constructor(enableDebugLogging, loggingContext) {
        super();
        this.enableDebugLogging = enableDebugLogging;
        this.loggingContext = loggingContext;
        this.TAB_TO_SPACES = 4;
    }
    logInfo(message) {
        console.log(`${chalk.bold(this.getContext())}${message}`);
    }
    logDebug(message) {
        if (this.enableDebugLogging) {
            console.log(`${chalk.bold(this.getContext())}${message}`);
        }
    }
    logErr(message) {
        console.error(`${chalk.bold(this.getContext())}${chalk.red(message)}`);
    }
    setLoggingContext(loggingContext) {
        this.loggingContext = loggingContext;
    }
    getContext() {
        return this.loggingContext !== undefined
            ? chalk.grey(`[${this.loggingContext}] `)
            : '';
    }
    logDiagnostic(severity, message, range, document) {
        const printFn = this.inferPrintFunction(severity);
        const colorFn = this.inferChalkColor(severity);
        this.logDiagnosticMessage(severity, message, printFn, colorFn);
        this.logDiagnosticInfo(range, document, printFn, colorFn);
        printFn('');
    }
    logDiagnosticMessage(severityName, message, printFn, colorFn) {
        printFn(`${chalk.bold(colorFn(severityName))}: ${message}`);
    }
    logDiagnosticInfo(range, document, printFn, colorFn) {
        const startLineNumber = range.start.line + 1;
        const endLineNumber = range.end.line + 1;
        const fullRange = {
            start: {
                line: range.start.line,
                character: 0,
            },
            end: {
                line: range.end.line,
                character: vscode_languageserver_types_1.uinteger.MAX_VALUE,
            },
        };
        const text = document.textDocument.getText(fullRange).trimEnd();
        const lines = text.split('\n');
        const lineNumberLength = Math.floor(Math.log10(endLineNumber)) + 1;
        printFn(`In ${document.uri.path}:${startLineNumber}:${range.start.character + 1}`);
        lines.forEach((line, i) => {
            const lineNumber = startLineNumber + i;
            const paddedLineNumber = String(lineNumber).padStart(lineNumberLength, ' ');
            printFn(`${chalk.grey(`${paddedLineNumber} |`)} ${line.replace(/\t/g, ' '.repeat(this.TAB_TO_SPACES))}`);
            let underlineFrom = 0;
            let underlineTo = line.length;
            if (lineNumber === startLineNumber) {
                underlineFrom = range.start.character;
            }
            if (lineNumber === endLineNumber) {
                underlineTo = range.end.character;
            }
            const underlineIndent = this.repeatCharAccordingToString(' ', line.substring(0, underlineFrom), this.TAB_TO_SPACES);
            const underline = this.repeatCharAccordingToString('^', line.substring(underlineFrom, underlineTo), this.TAB_TO_SPACES);
            printFn(`${chalk.grey(`${' '.repeat(lineNumberLength)} |`)} ${underlineIndent}${colorFn(underline)}`);
        });
    }
    /**
     * Repeats {@link charToRepeat} as many times as {@link accordingTo} is long.
     * For each occurrence of \t in {@link accordingTo},
     * {@link charToRepeat} is repeated {@link tabRepeats} times instead of once.
     */
    repeatCharAccordingToString(charToRepeat, accordingTo, tabRepeats) {
        return Array.from(accordingTo).reduce((prev, cur) => {
            const repeatedChar = cur === '\t' ? charToRepeat.repeat(tabRepeats) : charToRepeat;
            return `${prev}${repeatedChar}`;
        }, '');
    }
    inferPrintFunction(severity) {
        switch (severity) {
            case jayvee_execution_1.DiagnosticSeverity.ERROR:
                return console.error;
            case jayvee_execution_1.DiagnosticSeverity.WARNING:
                return console.warn;
            case jayvee_execution_1.DiagnosticSeverity.INFO:
            case jayvee_execution_1.DiagnosticSeverity.HINT:
                return console.info;
            default:
                (0, errors_1.assertUnreachable)(severity);
        }
    }
    inferChalkColor(severity) {
        switch (severity) {
            case jayvee_execution_1.DiagnosticSeverity.ERROR:
                return chalk.red;
            case jayvee_execution_1.DiagnosticSeverity.WARNING:
                return chalk.yellow;
            case jayvee_execution_1.DiagnosticSeverity.INFO:
                return chalk.green;
            case jayvee_execution_1.DiagnosticSeverity.HINT:
                return chalk.blue;
            default:
                (0, errors_1.assertUnreachable)(severity);
        }
    }
}
exports.DefaultLogger = DefaultLogger;


/***/ }),

/***/ "./apps/interpreter/src/logging/logger-factory.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.LoggerFactory = void 0;
const default_logger_1 = __webpack_require__("./apps/interpreter/src/logging/default-logger.ts");
class LoggerFactory {
    constructor(enableDebugLogging) {
        this.enableDebugLogging = enableDebugLogging;
    }
    createLogger(loggingContext) {
        return new default_logger_1.DefaultLogger(this.enableDebugLogging, loggingContext);
    }
}
exports.LoggerFactory = LoggerFactory;


/***/ }),

/***/ "./apps/interpreter/src/runtime-parameter-util.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.parseParameterAsMatchingType = exports.extractRuntimeParameters = exports.extractRequiredRuntimeParameters = void 0;
const assert_1 = __webpack_require__("assert");
const R = __webpack_require__("./libs/execution/src/index.ts");
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
const langium_1 = __webpack_require__("langium");
/**
 * Extracts all required runtime parameter ast nodes.
 * @param model The @see JayveeModel ast node
 * @returns a list of @see RuntimeParameterLiteral
 */
function extractRequiredRuntimeParameters(model) {
    const runtimeParameters = [];
    (0, langium_1.streamAst)(model).forEach((node) => {
        if ((0, jayvee_language_server_1.isRuntimeParameterLiteral)(node)) {
            runtimeParameters.push(node);
        }
    });
    return runtimeParameters;
}
exports.extractRequiredRuntimeParameters = extractRequiredRuntimeParameters;
/**
 * Creates a map with all the runtime parameter values.
 * @param requiredParameters A list of all required runtime parameters, e.g. by @see extractRequiredRuntimeParameters
 * @param env The environment variable map
 * @param logger the logger that shall be used for logging
 * @returns all runtime parameters stored as a map if all required ones are present
 */
function extractRuntimeParameters(requiredParameters, env, logger) {
    let errorCount = 0;
    const parameters = new Map();
    for (const requiredParameter of requiredParameters) {
        const parameterValue = env.get(requiredParameter.name);
        if (parameterValue === undefined) {
            logger.logErrDiagnostic(`Runtime parameter ${requiredParameter.name} is missing. Please provide a value by adding "-e ${requiredParameter.name}=<value>" to your command.`, { node: requiredParameter });
            ++errorCount;
            continue;
        }
        const type = requiredParameter.$container.$container.$container.type;
        const metaInf = (0, jayvee_language_server_1.getOrFailMetaInformation)(type);
        const propertyName = requiredParameter.$container.name;
        const propertySpec = metaInf.getPropertySpecification(propertyName);
        (0, assert_1.strict)(propertySpec !== undefined, `Property with name "${propertyName}" is not allowed in ${type.name}`);
        const parseResult = parseParameterAsMatchingType(parameterValue, propertySpec.type, requiredParameter);
        if (R.isErr(parseResult)) {
            logger.logErrDiagnostic(parseResult.left.message, parseResult.left.diagnostic);
            continue;
        }
        parameters.set(requiredParameter.name, parseResult.right);
    }
    if (errorCount > 0) {
        return undefined;
    }
    return parameters;
}
exports.extractRuntimeParameters = extractRuntimeParameters;
const TRUE = 'true';
const FALSE = 'false';
/**
 * Parses a runtime parameter value to the required type.
 * @param value The string value to be parsed.
 * @param type The type according to which the value is parsed.
 * @param astNode The ast node representing the parameter. Used for the diagnostics in case of errors.
 * @returns the parsed parameter value if parseable, error details if not.
 */
function parseParameterAsMatchingType(value, type, astNode) {
    (0, assert_1.strict)((0, jayvee_language_server_1.runtimeParameterAllowedForType)(type), `Runtime parameters of type ${type} are not allowed`);
    switch (type) {
        case jayvee_language_server_1.PropertyValuetype.TEXT:
            return R.ok(value);
        case jayvee_language_server_1.PropertyValuetype.INTEGER:
            if (!/^[+-]?[1-9][0-9]*$/.test(value)) {
                return R.err({
                    message: `Runtime parameter ${astNode.name} has value "${value}" which is not of type integer.`,
                    diagnostic: { node: astNode },
                });
            }
            return R.ok(Number.parseInt(value, 10));
        case jayvee_language_server_1.PropertyValuetype.DECIMAL:
            if (!/^[+-]?[0-9]+(\.[0-9]+)?$/.test(value)) {
                return R.err({
                    message: `Runtime parameter ${astNode.name} has value "${value}" which is not of type decimal.`,
                    diagnostic: { node: astNode },
                });
            }
            return R.ok(Number.parseFloat(value));
        case jayvee_language_server_1.PropertyValuetype.BOOLEAN:
            if (value === TRUE) {
                return R.ok(true);
            }
            if (value === FALSE) {
                return R.ok(false);
            }
            return R.err({
                message: `Runtime parameter ${astNode.name} has value "${value}" which is not of type boolean. Expected "${TRUE}" or "${FALSE}".`,
                diagnostic: { node: astNode },
            });
        default:
            throw new Error(`Unable to parse runtime parameters of type ${type}, please provide an implementation.`);
    }
}
exports.parseParameterAsMatchingType = parseParameterAsMatchingType;


/***/ }),

/***/ "./libs/execution/src/index.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
const tslib_1 = __webpack_require__("tslib");
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/index.ts"), exports);


/***/ }),

/***/ "./libs/execution/src/lib/blocks/block-executor-class.ts":
/***/ ((__unused_webpack_module, exports) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));


/***/ }),

/***/ "./libs/execution/src/lib/blocks/block-executor-registry.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.createBlockExecutor = exports.getRegisteredBlockExecutors = exports.registerBlockExecutor = void 0;
const assert_1 = __webpack_require__("assert");
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
const blockExecutorRegistry = new jayvee_language_server_1.Registry();
function registerBlockExecutor(executorClass) {
    blockExecutorRegistry.register(executorClass.type, executorClass);
}
exports.registerBlockExecutor = registerBlockExecutor;
function getRegisteredBlockExecutors() {
    return [...blockExecutorRegistry.getAll()];
}
exports.getRegisteredBlockExecutors = getRegisteredBlockExecutors;
function createBlockExecutor(block) {
    const blockType = block.type.name;
    const blockExecutor = blockExecutorRegistry.get(blockType);
    (0, assert_1.strict)(blockExecutor !== undefined, `No executor was registered for block type ${blockType}`);
    return new blockExecutor();
}
exports.createBlockExecutor = createBlockExecutor;


/***/ }),

/***/ "./libs/execution/src/lib/blocks/block-executor.ts":
/***/ ((__unused_webpack_module, exports) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));


/***/ }),

/***/ "./libs/execution/src/lib/blocks/execution-result.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.okData = exports.isErr = exports.isOk = exports.err = exports.ok = void 0;
const E = __webpack_require__("fp-ts/lib/Either");
/**
 * Creates an @Ok object from a data object typed T.
 * @param data the data object
 * @returns the created @Ok object
 */
function ok(data) {
    return E.right(data);
}
exports.ok = ok;
/**
 * Creates an @Err object from a @ExecutionErrorDetails object.
 * @param details the @ExecutionErrorDetails object
 * @returns the created @Err object
 */
function err(details) {
    return E.left(details);
}
exports.err = err;
/**
 * Type guard for @see Ok
 */
function isOk(result) {
    return E.isRight(result);
}
exports.isOk = isOk;
/**
 * Type guard for @see Err
 */
function isErr(result) {
    return E.isLeft(result);
}
exports.isErr = isErr;
/**
 * Convenience method to get wrapped data of an @see Ok object.
 */
function okData(ok) {
    return ok.right;
}
exports.okData = okData;


/***/ }),

/***/ "./libs/execution/src/lib/blocks/index.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
const tslib_1 = __webpack_require__("tslib");
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/blocks/block-executor.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/blocks/block-executor-class.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/blocks/block-executor-registry.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/blocks/execution-result.ts"), exports);


/***/ }),

/***/ "./libs/execution/src/lib/constraints/constraint-executor-registry.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.createConstraintExecutor = exports.getRegisteredConstraintExecutors = exports.registerConstraintExecutor = exports.registerDefaultConstraintExecutors = void 0;
const assert_1 = __webpack_require__("assert");
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
const allowlist_constraint_executor_1 = __webpack_require__("./libs/execution/src/lib/constraints/executors/allowlist-constraint-executor.ts");
const denylist_constraint_executor_1 = __webpack_require__("./libs/execution/src/lib/constraints/executors/denylist-constraint-executor.ts");
const length_constraint_executor_1 = __webpack_require__("./libs/execution/src/lib/constraints/executors/length-constraint-executor.ts");
const range_constraint_executor_1 = __webpack_require__("./libs/execution/src/lib/constraints/executors/range-constraint-executor.ts");
const regex_constraint_executor_1 = __webpack_require__("./libs/execution/src/lib/constraints/executors/regex-constraint-executor.ts");
const constraintExecutorRegistry = new jayvee_language_server_1.Registry();
function registerDefaultConstraintExecutors() {
    registerConstraintExecutor(allowlist_constraint_executor_1.AllowlistConstraintExecutor);
    registerConstraintExecutor(denylist_constraint_executor_1.DenylistConstraintExecutor);
    registerConstraintExecutor(regex_constraint_executor_1.RegexConstraintExecutor);
    registerConstraintExecutor(length_constraint_executor_1.LengthConstraintExecutor);
    registerConstraintExecutor(range_constraint_executor_1.RangeConstraintExecutor);
}
exports.registerDefaultConstraintExecutors = registerDefaultConstraintExecutors;
function registerConstraintExecutor(executorClass) {
    constraintExecutorRegistry.register(executorClass.type, executorClass);
}
exports.registerConstraintExecutor = registerConstraintExecutor;
function getRegisteredConstraintExecutors() {
    return constraintExecutorRegistry.getAll();
}
exports.getRegisteredConstraintExecutors = getRegisteredConstraintExecutors;
function createConstraintExecutor(constraint) {
    const constraintType = constraint.type.name;
    const constraintExecutor = constraintExecutorRegistry.get(constraintType);
    (0, assert_1.strict)(constraintExecutor !== undefined, `No executor was registered for constraint type ${constraintType}`);
    return new constraintExecutor();
}
exports.createConstraintExecutor = createConstraintExecutor;


/***/ }),

/***/ "./libs/execution/src/lib/constraints/executors/allowlist-constraint-executor.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.AllowlistConstraintExecutor = void 0;
const tslib_1 = __webpack_require__("tslib");
const implements_static_decorator_1 = __webpack_require__("./libs/execution/src/lib/util/implements-static-decorator.ts");
let AllowlistConstraintExecutor = class AllowlistConstraintExecutor {
    isValid(value, context) {
        if (typeof value !== 'string') {
            return false;
        }
        const allowlist = context.getTextCollectionPropertyValue('allowlist');
        const allowlistValues = allowlist.map((e) => e.value);
        return allowlistValues.includes(value);
    }
};
AllowlistConstraintExecutor.type = 'AllowlistConstraint';
AllowlistConstraintExecutor = tslib_1.__decorate([
    (0, implements_static_decorator_1.implementsStatic)()
], AllowlistConstraintExecutor);
exports.AllowlistConstraintExecutor = AllowlistConstraintExecutor;


/***/ }),

/***/ "./libs/execution/src/lib/constraints/executors/denylist-constraint-executor.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.DenylistConstraintExecutor = void 0;
const tslib_1 = __webpack_require__("tslib");
const implements_static_decorator_1 = __webpack_require__("./libs/execution/src/lib/util/implements-static-decorator.ts");
let DenylistConstraintExecutor = class DenylistConstraintExecutor {
    isValid(value, context) {
        if (typeof value !== 'string') {
            return false;
        }
        const denylist = context.getTextCollectionPropertyValue('denylist');
        const denylistValues = denylist.map((e) => e.value);
        return !denylistValues.includes(value);
    }
};
DenylistConstraintExecutor.type = 'DenylistConstraint';
DenylistConstraintExecutor = tslib_1.__decorate([
    (0, implements_static_decorator_1.implementsStatic)()
], DenylistConstraintExecutor);
exports.DenylistConstraintExecutor = DenylistConstraintExecutor;


/***/ }),

/***/ "./libs/execution/src/lib/constraints/executors/length-constraint-executor.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.LengthConstraintExecutor = void 0;
const tslib_1 = __webpack_require__("tslib");
const implements_static_decorator_1 = __webpack_require__("./libs/execution/src/lib/util/implements-static-decorator.ts");
let LengthConstraintExecutor = class LengthConstraintExecutor {
    isValid(value, context) {
        if (typeof value !== 'string') {
            return false;
        }
        const minLength = context.getNumericPropertyValue('minLength');
        const maxLength = context.getNumericPropertyValue('maxLength');
        return minLength <= value.length && value.length <= maxLength;
    }
};
LengthConstraintExecutor.type = 'LengthConstraint';
LengthConstraintExecutor = tslib_1.__decorate([
    (0, implements_static_decorator_1.implementsStatic)()
], LengthConstraintExecutor);
exports.LengthConstraintExecutor = LengthConstraintExecutor;


/***/ }),

/***/ "./libs/execution/src/lib/constraints/executors/range-constraint-executor.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.RangeConstraintExecutor = void 0;
const tslib_1 = __webpack_require__("tslib");
const implements_static_decorator_1 = __webpack_require__("./libs/execution/src/lib/util/implements-static-decorator.ts");
let RangeConstraintExecutor = class RangeConstraintExecutor {
    isValid(value, context) {
        let numericValue;
        if (typeof value === 'string') {
            numericValue = Number.parseFloat(value);
        }
        else if (typeof value === 'number') {
            numericValue = value;
        }
        else {
            return false;
        }
        const lowerBound = context.getNumericPropertyValue('lowerBound');
        const lowerBoundInclusive = context.getBooleanPropertyValue('lowerBoundInclusive');
        const upperBound = context.getNumericPropertyValue('upperBound');
        const upperBoundInclusive = context.getBooleanPropertyValue('upperBoundInclusive');
        const lowerBoundFulfilled = lowerBoundInclusive
            ? lowerBound <= numericValue
            : lowerBound < numericValue;
        const upperBoundFulfilled = upperBoundInclusive
            ? numericValue <= upperBound
            : numericValue < upperBound;
        return lowerBoundFulfilled && upperBoundFulfilled;
    }
};
RangeConstraintExecutor.type = 'RangeConstraint';
RangeConstraintExecutor = tslib_1.__decorate([
    (0, implements_static_decorator_1.implementsStatic)()
], RangeConstraintExecutor);
exports.RangeConstraintExecutor = RangeConstraintExecutor;


/***/ }),

/***/ "./libs/execution/src/lib/constraints/executors/regex-constraint-executor.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.RegexConstraintExecutor = void 0;
const tslib_1 = __webpack_require__("tslib");
const implements_static_decorator_1 = __webpack_require__("./libs/execution/src/lib/util/implements-static-decorator.ts");
let RegexConstraintExecutor = class RegexConstraintExecutor {
    isValid(value, context) {
        if (typeof value !== 'string') {
            return false;
        }
        const regex = context.getRegexPropertyValue('regex');
        return regex.test(value);
    }
};
RegexConstraintExecutor.type = 'RegexConstraint';
RegexConstraintExecutor = tslib_1.__decorate([
    (0, implements_static_decorator_1.implementsStatic)()
], RegexConstraintExecutor);
exports.RegexConstraintExecutor = RegexConstraintExecutor;


/***/ }),

/***/ "./libs/execution/src/lib/constraints/index.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
const tslib_1 = __webpack_require__("tslib");
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/constraints/constraint-executor-registry.ts"), exports);


/***/ }),

/***/ "./libs/execution/src/lib/execution-context.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.ExecutionContext = void 0;
const assert_1 = __webpack_require__("assert");
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
const langium_1 = __webpack_require__("langium");
class ExecutionContext {
    constructor(pipeline, logger, runtimeParameters) {
        this.pipeline = pipeline;
        this.logger = logger;
        this.runtimeParameters = runtimeParameters;
        this.stack = [];
        logger.setLoggingContext(pipeline.name);
    }
    enterNode(node) {
        this.stack.push(node);
        this.updateLoggingContext();
    }
    exitNode(node) {
        const poppedNode = this.stack.pop();
        (0, assert_1.strict)(poppedNode === node);
        this.updateLoggingContext();
    }
    getCurrentNode() {
        const currentNode = this.stack[this.stack.length - 1];
        if (currentNode === undefined) {
            return this.pipeline;
        }
        return currentNode;
    }
    updateLoggingContext() {
        this.logger.setLoggingContext(this.getCurrentNode().name);
    }
    getTextPropertyValue(propertyName) {
        const propertyValue = this.getPropertyValue(propertyName);
        (0, assert_1.strict)(typeof propertyValue === 'string');
        return propertyValue;
    }
    getNumericPropertyValue(propertyName) {
        const propertyValue = this.getPropertyValue(propertyName);
        (0, assert_1.strict)(typeof propertyValue === 'number');
        return propertyValue;
    }
    getBooleanPropertyValue(propertyName) {
        const propertyValue = this.getPropertyValue(propertyName);
        (0, assert_1.strict)(typeof propertyValue === 'boolean');
        return propertyValue;
    }
    getRegexPropertyValue(propertyName) {
        const propertyValue = this.getPropertyValue(propertyName);
        (0, assert_1.strict)(typeof propertyValue === 'string');
        return new RegExp(propertyValue);
    }
    getCellRangePropertyValue(propertyName) {
        const propertyValue = this.getPropertyValue(propertyName);
        (0, assert_1.strict)((0, jayvee_language_server_1.isCellRangeLiteral)(propertyValue));
        return new jayvee_language_server_1.CellRangeWrapper(propertyValue);
    }
    getTextCollectionPropertyValue(propertyName) {
        const propertyValue = this.getPropertyValue(propertyName);
        (0, assert_1.strict)(Array.isArray(propertyValue));
        (0, assert_1.strict)(propertyValue.every(jayvee_language_server_1.isTextLiteral));
        return propertyValue;
    }
    getNumericCollectionPropertyValue(propertyName) {
        const propertyValue = this.getPropertyValue(propertyName);
        (0, assert_1.strict)(Array.isArray(propertyValue));
        (0, assert_1.strict)(propertyValue.every(jayvee_language_server_1.isNumericLiteral));
        return propertyValue;
    }
    getCellRangeCollectionPropertyValue(propertyName) {
        const propertyValue = this.getPropertyValue(propertyName);
        (0, assert_1.strict)(Array.isArray(propertyValue));
        (0, assert_1.strict)(propertyValue.every(jayvee_language_server_1.isCellRangeLiteral));
        return propertyValue.map((cellRange) => new jayvee_language_server_1.CellRangeWrapper(cellRange));
    }
    getValuetypeAssignmentCollectionPropertyValue(propertyName) {
        const propertyValue = this.getPropertyValue(propertyName);
        (0, assert_1.strict)(Array.isArray(propertyValue));
        (0, assert_1.strict)(propertyValue.every(jayvee_language_server_1.isValuetypeAssignmentLiteral));
        return propertyValue.map((assignment) => assignment.value);
    }
    getProperty(propertyName) {
        const currentNode = this.getCurrentNode();
        if ((0, jayvee_language_server_1.isPipelineDefinition)(currentNode)) {
            return undefined;
        }
        return currentNode.body.properties.find((property) => property.name === propertyName);
    }
    getOrFailProperty(propertyName) {
        const property = this.getProperty(propertyName);
        (0, assert_1.strict)(property !== undefined);
        return property;
    }
    getPropertyValue(propertyName) {
        const property = this.getProperty(propertyName);
        if (property === undefined) {
            return this.getDefaultPropertyValue(propertyName);
        }
        const propertyValue = property.value;
        if ((0, jayvee_language_server_1.isRuntimeParameterLiteral)(propertyValue)) {
            return this.runtimeParameters.get(propertyValue.name);
        }
        if ((0, jayvee_language_server_1.isCollectionLiteral)(propertyValue)) {
            return propertyValue.values;
        }
        if ((0, jayvee_language_server_1.isCellRangeLiteral)(propertyValue)) {
            return propertyValue;
        }
        const value = propertyValue.value;
        if ((0, langium_1.isReference)(value)) {
            const reference = value.ref;
            (0, assert_1.strict)(reference !== undefined);
            return reference;
        }
        return value;
    }
    getDefaultPropertyValue(propertyName) {
        const currentNode = this.getCurrentNode();
        (0, assert_1.strict)(!(0, jayvee_language_server_1.isPipelineDefinition)(currentNode));
        const metaInf = (0, jayvee_language_server_1.getOrFailMetaInformation)(currentNode.type);
        const propertySpec = metaInf.getPropertySpecification(propertyName);
        (0, assert_1.strict)(propertySpec !== undefined);
        const defaultValue = propertySpec.defaultValue;
        (0, assert_1.strict)(defaultValue !== undefined);
        return defaultValue;
    }
}
exports.ExecutionContext = ExecutionContext;


/***/ }),

/***/ "./libs/execution/src/lib/extension.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.useExtension = void 0;
const block_executor_registry_1 = __webpack_require__("./libs/execution/src/lib/blocks/block-executor-registry.ts");
function useExtension(extension) {
    extension.getBlockExecutors().forEach(block_executor_registry_1.registerBlockExecutor);
}
exports.useExtension = useExtension;


/***/ }),

/***/ "./libs/execution/src/lib/index.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
const tslib_1 = __webpack_require__("tslib");
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/blocks/index.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/constraints/index.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/types/index.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/util/index.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/visitors/index.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/execution-context.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/extension.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/logger.ts"), exports);


/***/ }),

/***/ "./libs/execution/src/lib/logger.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.Logger = exports.DiagnosticSeverity = void 0;
const assert_1 = __webpack_require__("assert");
const langium_1 = __webpack_require__("langium");
const ls = __webpack_require__("vscode-languageserver");
var DiagnosticSeverity;
(function (DiagnosticSeverity) {
    DiagnosticSeverity["ERROR"] = "error";
    DiagnosticSeverity["WARNING"] = "warning";
    DiagnosticSeverity["INFO"] = "info";
    DiagnosticSeverity["HINT"] = "hint";
})(DiagnosticSeverity = exports.DiagnosticSeverity || (exports.DiagnosticSeverity = {}));
class Logger {
    logLangiumDiagnostic(severity, message, diagnostic) {
        this.logDiagnostic(severity, message, (0, langium_1.getDiagnosticRange)(diagnostic), (0, langium_1.getDocument)(diagnostic.node));
    }
    logErrDiagnostic(message, diagnostic) {
        this.logLangiumDiagnostic(DiagnosticSeverity.ERROR, message, diagnostic);
    }
    logWarnDiagnostic(message, diagnostic) {
        this.logLangiumDiagnostic(DiagnosticSeverity.WARNING, message, diagnostic);
    }
    logInfoDiagnostic(message, diagnostic) {
        this.logLangiumDiagnostic(DiagnosticSeverity.INFO, message, diagnostic);
    }
    logHintDiagnostic(message, diagnostic) {
        this.logLangiumDiagnostic(DiagnosticSeverity.HINT, message, diagnostic);
    }
    logLanguageServerDiagnostic(lsDiagnostic, document) {
        (0, assert_1.strict)(lsDiagnostic.severity !== undefined, 'The diagnostic severity is assumed to be present');
        this.logDiagnostic(this.toDiagnosticSeverity(lsDiagnostic.severity), lsDiagnostic.message, lsDiagnostic.range, document);
    }
    toDiagnosticSeverity(severity) {
        switch (severity) {
            case ls.DiagnosticSeverity.Error:
                return DiagnosticSeverity.ERROR;
            case ls.DiagnosticSeverity.Warning:
                return DiagnosticSeverity.WARNING;
            case ls.DiagnosticSeverity.Information:
                return DiagnosticSeverity.INFO;
            case ls.DiagnosticSeverity.Hint:
                return DiagnosticSeverity.HINT;
        }
    }
}
exports.Logger = Logger;


/***/ }),

/***/ "./libs/execution/src/lib/types/index.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
const tslib_1 = __webpack_require__("tslib");
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/types/io-types/index.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/types/valuetypes/index.ts"), exports);


/***/ }),

/***/ "./libs/execution/src/lib/types/io-types/binary-file.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.BinaryFile = void 0;
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
const file_1 = __webpack_require__("./libs/execution/src/lib/types/io-types/file.ts");
class BinaryFile extends file_1.File {
    constructor() {
        super(...arguments);
        this.ioType = jayvee_language_server_1.IOType.FILE;
    }
}
exports.BinaryFile = BinaryFile;


/***/ }),

/***/ "./libs/execution/src/lib/types/io-types/file.ts":
/***/ ((__unused_webpack_module, exports) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.MimeType = exports.FileExtension = exports.File = void 0;
/**
 * Represents a file with its name, extension, file type, and content.
 * @class File
 */
class File {
    constructor(
    /**
     * The name of the file, without the extension.
     * @property {string} name
     */
    name, 
    /**
     * The file extension in lower case, NONE / empty string for unknown or missing file extensions.
     * @property {FileExtension} extension
     */
    extension, 
    /**
     * The MIME type of the file taken from the Content-Type header (for HTTP requests only),
     * Otherwise inferred from the file extension, default application/octet-stream for unknown or missing file extensions.
     * @property {MimeType} mimeType
     */
    mimeType, content) {
        this.name = name;
        this.extension = extension;
        this.mimeType = mimeType;
        this.content = content;
    }
}
exports.File = File;
/**
 * An enumeration of common file extensions. New extensions for Files need to be registered here.
 *
 * @enum {string}
 */
var FileExtension;
(function (FileExtension) {
    FileExtension["ZIP"] = "zip";
    FileExtension["TXT"] = "txt";
    FileExtension["CSV"] = "csv";
    FileExtension["NONE"] = "";
})(FileExtension = exports.FileExtension || (exports.FileExtension = {}));
/**
 * An enumeration of common MIME types.
 *
 * @enum {string}
 */
var MimeType;
(function (MimeType) {
    MimeType["APPLICATION_ZIP"] = "application/zip";
    MimeType["APPLICATION_OCTET_STREAM"] = "application/octet-stream";
    MimeType["TEXT_CSV"] = "text/csv";
    MimeType["TEXT_PLAIN"] = "text/plain";
})(MimeType = exports.MimeType || (exports.MimeType = {}));


/***/ }),

/***/ "./libs/execution/src/lib/types/io-types/filesystem-inmemory.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.InMemoryFileSystem = void 0;
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
class InMemoryFileSystem {
    constructor() {
        this.ioType = jayvee_language_server_1.IOType.FILE_SYSTEM;
        // Hierachical file system
        this.fileSystemIndex = new Map();
    }
    processPath(path) {
        const parts = path
            .toLowerCase()
            .split(InMemoryFileSystem.PATH_SEPARATOR)
            .filter((p) => p !== ''); // Process paths like "folder1//folder1" to "folder1/folder2"
        const processedParts = [];
        for (const part of parts) {
            if (part === InMemoryFileSystem.CURRENT_DIR) {
                continue; // Skip current dirs in path
            }
            if (part === InMemoryFileSystem.PARENT_DIR) {
                processedParts.pop(); // Go level up in folder hierarchy if ..
            }
            else {
                processedParts.push(part);
            }
        }
        return processedParts;
    }
    getFile(filePath) {
        const processedParts = this.processPath(filePath);
        // If we have a minimum valid path
        if (processedParts.length > 0) {
            let currentFileSystemIndex = this.fileSystemIndex;
            // Get the name of the requested file
            const fileName = processedParts[processedParts.length - 1];
            // If we have a directory, we traverse it, if not (processedParts == 1) skip the travesal
            for (let i = 0; i < processedParts.length - 1; i++) {
                const childFileSystem = currentFileSystemIndex.get(processedParts[i]);
                // If we dont find current path-part, stop methodcall
                if (!childFileSystem) {
                    return null;
                }
                currentFileSystemIndex = childFileSystem.fileSystemIndex;
            }
            // If we are in the correct directory
            const file = currentFileSystemIndex.get(fileName);
            return file === undefined ? null : file;
        }
        return null;
    }
    putFile(filePath, file) {
        const processedParts = this.processPath(filePath);
        let currentFileSystemIndex = this.fileSystemIndex;
        // If we need to traverse
        for (let i = 0; i < processedParts.length - 1; i++) {
            const part = processedParts[i];
            // Check, if directory already exists, if not create it
            let childFileSystemIndex = currentFileSystemIndex.get(part);
            if (!childFileSystemIndex) {
                childFileSystemIndex = new InMemoryFileSystem();
                currentFileSystemIndex.set(part, childFileSystemIndex);
            }
            currentFileSystemIndex = childFileSystemIndex.fileSystemIndex;
        }
        // Safe the actual file
        currentFileSystemIndex.set(processedParts[processedParts.length - 1], file);
        return this;
    }
}
exports.InMemoryFileSystem = InMemoryFileSystem;
InMemoryFileSystem.PATH_SEPARATOR = '/';
InMemoryFileSystem.CURRENT_DIR = '.';
InMemoryFileSystem.PARENT_DIR = '..';


/***/ }),

/***/ "./libs/execution/src/lib/types/io-types/filesystem.ts":
/***/ ((__unused_webpack_module, exports) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));


/***/ }),

/***/ "./libs/execution/src/lib/types/io-types/index.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
const tslib_1 = __webpack_require__("tslib");
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/types/io-types/binary-file.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/types/io-types/file.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/types/io-types/filesystem.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/types/io-types/filesystem-inmemory.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/types/io-types/io-type-implementation.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/types/io-types/none.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/types/io-types/sheet.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/types/io-types/table.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/types/io-types/text-file.ts"), exports);


/***/ }),

/***/ "./libs/execution/src/lib/types/io-types/io-type-implementation.ts":
/***/ ((__unused_webpack_module, exports) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));


/***/ }),

/***/ "./libs/execution/src/lib/types/io-types/none.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.NONE = exports.None = void 0;
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
class None {
    // eslint-disable-next-line @typescript-eslint/no-empty-function
    constructor() {
        this.ioType = jayvee_language_server_1.IOType.NONE;
    }
    static get singletonInstance() {
        if (this._instance === undefined) {
            this._instance = new this();
        }
        return this._instance;
    }
}
exports.None = None;
exports.NONE = None.singletonInstance;


/***/ }),

/***/ "./libs/execution/src/lib/types/io-types/sheet.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.Sheet = void 0;
const assert_1 = __webpack_require__("assert");
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
class Sheet {
    constructor(data) {
        this.data = data;
        this.ioType = jayvee_language_server_1.IOType.SHEET;
        this.numberOfRows = data.length;
        this.numberOfColumns = data.reduce((prev, curr) => {
            return curr.length > prev ? curr.length : prev;
        }, 0);
    }
    getNumberOfRows() {
        return this.numberOfRows;
    }
    getNumberOfColumns() {
        return this.numberOfColumns;
    }
    getHeaderRow() {
        (0, assert_1.strict)(this.getNumberOfRows() > 0, 'The sheet is expected to be non-empty and have a header row');
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        return this.data[0];
    }
    iterateRows(callbackFn) {
        this.data.forEach(callbackFn);
    }
    clone() {
        return new Sheet(structuredClone(this.data));
    }
    deleteRow(row) {
        (0, assert_1.strict)(this.isInBounds(row));
        row = this.resolveRelativeIndexes(row);
        const rowIndex = (0, jayvee_language_server_1.getRowIndex)(row);
        this.data.splice(rowIndex, 1);
        this.numberOfRows--;
    }
    deleteColumn(column) {
        (0, assert_1.strict)(this.isInBounds(column));
        column = this.resolveRelativeIndexes(column);
        const columnIndex = (0, jayvee_language_server_1.getColumnIndex)(column);
        this.data.forEach((row) => {
            row.splice(columnIndex, 1);
        });
        this.numberOfColumns--;
    }
    writeCell(cell, content) {
        (0, assert_1.strict)(this.isInBounds(cell));
        cell = this.resolveRelativeIndexes(cell);
        const cellIndex = (0, jayvee_language_server_1.getCellIndex)(cell);
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        this.data[cellIndex.rowIndex][cellIndex.columnIndex] = content;
    }
    selectRange(range) {
        (0, assert_1.strict)(this.isInBounds(range));
        range = this.resolveRelativeIndexes(range);
        this.data = this.data.reduce((previous, row, rowIndex) => {
            if (rowIndex < range.from.rowIndex || range.to.rowIndex < rowIndex) {
                return previous;
            }
            return [
                ...previous,
                row.slice(range.from.columnIndex, range.to.columnIndex + 1),
            ];
        }, []);
        this.numberOfColumns = range.to.columnIndex - range.from.columnIndex + 1;
        this.numberOfRows = range.to.rowIndex - range.from.rowIndex + 1;
    }
    isInBounds(range) {
        const bounds = this.getBounds();
        return range.isInBounds(bounds);
    }
    resolveRelativeIndexes(range) {
        if (range.hasRelativeIndexes()) {
            const bounds = this.getBounds();
            return range.resolveRelativeIndexes(bounds);
        }
        return range;
    }
    getBounds() {
        return {
            lastColumnIndex: this.numberOfColumns - 1,
            lastRowIndex: this.numberOfRows - 1,
        };
    }
}
exports.Sheet = Sheet;


/***/ }),

/***/ "./libs/execution/src/lib/types/io-types/table.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.Table = void 0;
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
const sql_column_type_visitor_1 = __webpack_require__("./libs/execution/src/lib/visitors/sql-column-type-visitor.ts");
const sql_value_representation_visitor_1 = __webpack_require__("./libs/execution/src/lib/visitors/sql-value-representation-visitor.ts");
class Table {
    constructor(columnInformation, data) {
        this.columnInformation = columnInformation;
        this.data = data;
        this.ioType = jayvee_language_server_1.IOType.TABLE;
        this.numberOfRows = data.length;
        this.numberOfColumns = columnInformation.length;
    }
    getNumberOfRows() {
        return this.numberOfRows;
    }
    getNumberOfColumns() {
        return this.numberOfColumns;
    }
    static generateDropTableStatement(tableName) {
        return `DROP TABLE IF EXISTS "${tableName}";`;
    }
    generateInsertValuesStatement(tableName) {
        const valueRepresentationVisitor = new sql_value_representation_visitor_1.SQLValueRepresentationVisitor();
        const valueRepresentationFormatters = this.columnInformation.map((columnInformation) => columnInformation.type.acceptVisitor(valueRepresentationVisitor));
        const valuesStatement = this.data
            .map((row) => {
            return `(${row
                .map((value, index) => { var _a; return (_a = valueRepresentationFormatters[index]) === null || _a === void 0 ? void 0 : _a.call(valueRepresentationFormatters, value); })
                .join(',')})`;
        })
            .join(',');
        return `INSERT INTO "${tableName}" (${this.columnInformation
            .map((columnInformation) => `"${columnInformation.name}"`)
            .join(',')}) VALUES ${valuesStatement}`;
    }
    generateCreateTableStatement(tableName) {
        const columnTypeVisitor = new sql_column_type_visitor_1.SQLColumnTypeVisitor();
        const columnStatements = this.columnInformation.map((columnInformation) => {
            return `"${columnInformation.name}" ${columnInformation.type.acceptVisitor(columnTypeVisitor)}`;
        });
        return `CREATE TABLE IF NOT EXISTS "${tableName}" (${columnStatements.join(',')});`;
    }
}
exports.Table = Table;


/***/ }),

/***/ "./libs/execution/src/lib/types/io-types/text-file.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.TextFile = void 0;
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
const file_1 = __webpack_require__("./libs/execution/src/lib/types/io-types/file.ts");
class TextFile extends file_1.File {
    constructor() {
        super(...arguments);
        this.ioType = jayvee_language_server_1.IOType.TEXT_FILE;
    }
}
exports.TextFile = TextFile;


/***/ }),

/***/ "./libs/execution/src/lib/types/valuetypes/atomic-valuetype.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.AtomicValuetype = void 0;
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
const constraint_executor_registry_1 = __webpack_require__("./libs/execution/src/lib/constraints/constraint-executor-registry.ts");
class AtomicValuetype {
    constructor(astNode, primitiveValuetype) {
        this.astNode = astNode;
        this.primitiveValuetype = primitiveValuetype;
    }
    acceptVisitor(visitor) {
        return this.primitiveValuetype.acceptVisitor(visitor);
    }
    isValid(value, context) {
        if (!this.primitiveValuetype.isValid(value, context)) {
            return false;
        }
        const constraints = this.getConstraints();
        for (const constraint of constraints) {
            const constraintExecutor = (0, constraint_executor_registry_1.createConstraintExecutor)(constraint);
            context.enterNode(constraint);
            const valueFulfilledConstraint = constraintExecutor.isValid(value, context);
            context.exitNode(constraint);
            if (!valueFulfilledConstraint) {
                return false;
            }
        }
        return true;
    }
    getStandardRepresentation(value) {
        return this.primitiveValuetype.getStandardRepresentation(value);
    }
    getConstraints() {
        const constraintCollection = this.astNode.constraints;
        const constraintReferences = (0, jayvee_language_server_1.validateTypedCollection)(constraintCollection, jayvee_language_server_1.isConstraintReferenceLiteral).validItems;
        const constraints = constraintReferences.map(
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        (constraintReference) => constraintReference.value.ref);
        return constraints;
    }
}
exports.AtomicValuetype = AtomicValuetype;


/***/ }),

/***/ "./libs/execution/src/lib/types/valuetypes/index.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
const tslib_1 = __webpack_require__("tslib");
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/types/valuetypes/primitive/index.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/types/valuetypes/visitors/index.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/types/valuetypes/atomic-valuetype.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/types/valuetypes/valuetype.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/types/valuetypes/valuetype-util.ts"), exports);


/***/ }),

/***/ "./libs/execution/src/lib/types/valuetypes/primitive/boolean-valuetype.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.BooleanValuetype = void 0;
const assert_1 = __webpack_require__("assert");
class BooleanValuetype {
    constructor(astNode) {
        this.astNode = astNode;
        this.BOOLEAN_STRING_REPRESENTATIONS = [
            'true',
            'True',
            'false',
            'False',
        ];
        (0, assert_1.strict)(astNode.keyword === 'boolean');
    }
    acceptVisitor(visitor) {
        return visitor.visitBoolean(this);
    }
    isValid(value) {
        if (typeof value === 'boolean') {
            return true;
        }
        if (typeof value === 'string') {
            return this.BOOLEAN_STRING_REPRESENTATIONS.includes(value);
        }
        return false;
    }
    getStandardRepresentation(value) {
        if (typeof value === 'boolean') {
            return value;
        }
        if (typeof value === 'string') {
            switch (value.toLowerCase()) {
                case 'true':
                    return true;
                default:
                    return false;
            }
        }
        throw new Error(
        // eslint-disable-next-line @typescript-eslint/restrict-template-expressions
        `Invalid value: ${value} for type ${this.astNode.keyword}`);
    }
}
exports.BooleanValuetype = BooleanValuetype;


/***/ }),

/***/ "./libs/execution/src/lib/types/valuetypes/primitive/decimal-valuetype.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.DecimalValuetype = void 0;
const assert_1 = __webpack_require__("assert");
class DecimalValuetype {
    constructor(astNode) {
        this.astNode = astNode;
        this.DOT_SEPARATOR_REGEX = /^[+-]?([0-9]*[.])?[0-9]+$/;
        this.COMMA_SEPARATOR_REGEX = /^[+-]?([0-9]*[,])?[0-9]+$/;
        (0, assert_1.strict)(astNode.keyword === 'decimal');
    }
    isValid(value) {
        if (typeof value === 'string') {
            return (this.DOT_SEPARATOR_REGEX.test(value) ||
                this.COMMA_SEPARATOR_REGEX.test(value));
        }
        return !Number.isNaN(value);
    }
    acceptVisitor(visitor) {
        return visitor.visitDecimal(this);
    }
    getStandardRepresentation(value) {
        if (typeof value === 'number') {
            return value;
        }
        if (typeof value === 'string') {
            let stringValue = value;
            if (this.COMMA_SEPARATOR_REGEX.test(stringValue)) {
                stringValue = stringValue.replace(',', '.');
            }
            return Number.parseFloat(stringValue);
        }
        throw new Error(
        // eslint-disable-next-line @typescript-eslint/restrict-template-expressions
        `Invalid value: ${value} for type ${this.astNode.keyword}`);
    }
}
exports.DecimalValuetype = DecimalValuetype;


/***/ }),

/***/ "./libs/execution/src/lib/types/valuetypes/primitive/index.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
const tslib_1 = __webpack_require__("tslib");
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/types/valuetypes/primitive/boolean-valuetype.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/types/valuetypes/primitive/decimal-valuetype.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/types/valuetypes/primitive/integer-valuetype.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/types/valuetypes/primitive/primitive-valuetype.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/types/valuetypes/primitive/text-valuetype.ts"), exports);


/***/ }),

/***/ "./libs/execution/src/lib/types/valuetypes/primitive/integer-valuetype.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.IntegerValuetype = void 0;
const assert_1 = __webpack_require__("assert");
class IntegerValuetype {
    constructor(astNode) {
        this.astNode = astNode;
        (0, assert_1.strict)(astNode.keyword === 'integer');
    }
    isValid(value) {
        if (typeof value === 'string') {
            return /^[+-]?[0-9]+$/.test(value);
        }
        return Number.isInteger(value);
    }
    acceptVisitor(visitor) {
        return visitor.visitInteger(this);
    }
    getStandardRepresentation(value) {
        if (typeof value === 'number') {
            return value;
        }
        if (typeof value === 'string') {
            return Number.parseInt(value, 10);
        }
        throw new Error(
        // eslint-disable-next-line @typescript-eslint/restrict-template-expressions
        `Invalid value: ${value} for type ${this.astNode.keyword}`);
    }
}
exports.IntegerValuetype = IntegerValuetype;


/***/ }),

/***/ "./libs/execution/src/lib/types/valuetypes/primitive/primitive-valuetype.ts":
/***/ ((__unused_webpack_module, exports) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));


/***/ }),

/***/ "./libs/execution/src/lib/types/valuetypes/primitive/text-valuetype.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.TextValuetype = void 0;
const assert_1 = __webpack_require__("assert");
class TextValuetype {
    constructor(astNode) {
        this.astNode = astNode;
        (0, assert_1.strict)(astNode.keyword === 'text');
    }
    isValid(value) {
        return typeof value === 'string';
    }
    acceptVisitor(visitor) {
        return visitor.visitText(this);
    }
    getStandardRepresentation(value) {
        if (typeof value === 'number') {
            return value.toString();
        }
        if (typeof value === 'string') {
            return value;
        }
        throw new Error(
        // eslint-disable-next-line @typescript-eslint/restrict-template-expressions
        `Invalid value: ${value} for type ${this.astNode.keyword}`);
    }
}
exports.TextValuetype = TextValuetype;


/***/ }),

/***/ "./libs/execution/src/lib/types/valuetypes/valuetype-util.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.createValuetype = void 0;
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
const errors_1 = __webpack_require__("langium/lib/utils/errors");
const atomic_valuetype_1 = __webpack_require__("./libs/execution/src/lib/types/valuetypes/atomic-valuetype.ts");
const boolean_valuetype_1 = __webpack_require__("./libs/execution/src/lib/types/valuetypes/primitive/boolean-valuetype.ts");
const decimal_valuetype_1 = __webpack_require__("./libs/execution/src/lib/types/valuetypes/primitive/decimal-valuetype.ts");
const integer_valuetype_1 = __webpack_require__("./libs/execution/src/lib/types/valuetypes/primitive/integer-valuetype.ts");
const text_valuetype_1 = __webpack_require__("./libs/execution/src/lib/types/valuetypes/primitive/text-valuetype.ts");
function createValuetype(valuetype) {
    if ((0, jayvee_language_server_1.isValuetypeDefinitionReference)(valuetype)) {
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        const valuetypeDefinition = valuetype.reference.ref;
        const primitiveValuetypeKeyword = valuetypeDefinition.type;
        const primitiveValuetype = createPrimitiveValuetype(primitiveValuetypeKeyword);
        return new atomic_valuetype_1.AtomicValuetype(valuetypeDefinition, primitiveValuetype);
    }
    return createPrimitiveValuetype(valuetype);
}
exports.createValuetype = createValuetype;
function createPrimitiveValuetype(keywordLiteral) {
    switch (keywordLiteral.keyword) {
        case 'text':
            return new text_valuetype_1.TextValuetype(keywordLiteral);
        case 'decimal':
            return new decimal_valuetype_1.DecimalValuetype(keywordLiteral);
        case 'integer':
            return new integer_valuetype_1.IntegerValuetype(keywordLiteral);
        case 'boolean':
            return new boolean_valuetype_1.BooleanValuetype(keywordLiteral);
        default:
            (0, errors_1.assertUnreachable)(keywordLiteral.keyword);
    }
}


/***/ }),

/***/ "./libs/execution/src/lib/types/valuetypes/valuetype.ts":
/***/ ((__unused_webpack_module, exports) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));


/***/ }),

/***/ "./libs/execution/src/lib/types/valuetypes/visitors/index.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
const tslib_1 = __webpack_require__("tslib");
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/types/valuetypes/visitors/valuetype-visitor.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/types/valuetypes/visitors/visitable-valuetype.ts"), exports);


/***/ }),

/***/ "./libs/execution/src/lib/types/valuetypes/visitors/valuetype-visitor.ts":
/***/ ((__unused_webpack_module, exports) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.ValuetypeVisitor = void 0;
class ValuetypeVisitor {
}
exports.ValuetypeVisitor = ValuetypeVisitor;


/***/ }),

/***/ "./libs/execution/src/lib/types/valuetypes/visitors/visitable-valuetype.ts":
/***/ ((__unused_webpack_module, exports) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));


/***/ }),

/***/ "./libs/execution/src/lib/util/implements-static-decorator.ts":
/***/ ((__unused_webpack_module, exports) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.implementsStatic = void 0;
/**
 * Decorator for classes, so they need to implement a given interface using static members / functions.
 *
 * @example
 * interface SampleInterface {
 *     a: string
 *     foo(): void;
 * }
 * @implementsStatic<SampleInterface>()
 * class SampleClass {
 *     static a: string = 'A';
 *     static foo(): void {
 *         return;
 *     }
 * }
 */
function implementsStatic() {
    return (constructor) => {
        constructor;
    };
}
exports.implementsStatic = implementsStatic;


/***/ }),

/***/ "./libs/execution/src/lib/util/index.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
const tslib_1 = __webpack_require__("tslib");
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/util/implements-static-decorator.ts"), exports);


/***/ }),

/***/ "./libs/execution/src/lib/visitors/index.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
const tslib_1 = __webpack_require__("tslib");
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/visitors/sql-column-type-visitor.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/visitors/sql-value-representation-visitor.ts"), exports);


/***/ }),

/***/ "./libs/execution/src/lib/visitors/sql-column-type-visitor.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.SQLColumnTypeVisitor = void 0;
const valuetype_visitor_1 = __webpack_require__("./libs/execution/src/lib/types/valuetypes/visitors/valuetype-visitor.ts");
class SQLColumnTypeVisitor extends valuetype_visitor_1.ValuetypeVisitor {
    visitBoolean() {
        return 'boolean';
    }
    visitDecimal() {
        return 'real';
    }
    visitInteger() {
        return 'integer';
    }
    visitText() {
        return 'text';
    }
}
exports.SQLColumnTypeVisitor = SQLColumnTypeVisitor;


/***/ }),

/***/ "./libs/execution/src/lib/visitors/sql-value-representation-visitor.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.SQLValueRepresentationVisitor = void 0;
const valuetype_visitor_1 = __webpack_require__("./libs/execution/src/lib/types/valuetypes/visitors/valuetype-visitor.ts");
class SQLValueRepresentationVisitor extends valuetype_visitor_1.ValuetypeVisitor {
    visitBoolean(valuetype) {
        return (value) => {
            return valuetype.getStandardRepresentation(value)
                ? String.raw `'true'`
                : String.raw `'false'`;
        };
    }
    visitDecimal(valuetype) {
        return (value) => {
            return valuetype.getStandardRepresentation(value).toString();
        };
    }
    visitInteger(valuetype) {
        return (value) => {
            return valuetype.getStandardRepresentation(value).toString();
        };
    }
    visitText(valuetype) {
        return (value) => {
            const standardValueRepresentation = valuetype.getStandardRepresentation(value);
            const escapedValueRepresentation = escapeSingleQuotes(standardValueRepresentation);
            return `'${escapedValueRepresentation}'`;
        };
    }
}
exports.SQLValueRepresentationVisitor = SQLValueRepresentationVisitor;
function escapeSingleQuotes(value) {
    return value.replace(/'/g, `''`);
}


/***/ }),

/***/ "./libs/extensions/rdbms/exec/src/extension.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.RdbmsExecExtension = void 0;
const lib_1 = __webpack_require__("./libs/extensions/rdbms/exec/src/lib/index.ts");
class RdbmsExecExtension {
    getBlockExecutors() {
        return [lib_1.PostgresLoaderExecutor, lib_1.SQLiteLoaderExecutor];
    }
}
exports.RdbmsExecExtension = RdbmsExecExtension;


/***/ }),

/***/ "./libs/extensions/rdbms/exec/src/index.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
const tslib_1 = __webpack_require__("tslib");
tslib_1.__exportStar(__webpack_require__("./libs/extensions/rdbms/exec/src/extension.ts"), exports);


/***/ }),

/***/ "./libs/extensions/rdbms/exec/src/lib/index.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
const tslib_1 = __webpack_require__("tslib");
tslib_1.__exportStar(__webpack_require__("./libs/extensions/rdbms/exec/src/lib/postgres-loader-executor.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/extensions/rdbms/exec/src/lib/sqlite-loader-executor.ts"), exports);


/***/ }),

/***/ "./libs/extensions/rdbms/exec/src/lib/postgres-loader-executor.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.PostgresLoaderExecutor = void 0;
const tslib_1 = __webpack_require__("tslib");
const R = __webpack_require__("./libs/execution/src/index.ts");
const jayvee_execution_1 = __webpack_require__("./libs/execution/src/index.ts");
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
const pg_1 = __webpack_require__("pg");
let PostgresLoaderExecutor = class PostgresLoaderExecutor {
    constructor() {
        this.inputType = jayvee_language_server_1.IOType.TABLE;
        this.outputType = jayvee_language_server_1.IOType.NONE;
    }
    execute(input, context) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const host = context.getTextPropertyValue('host');
            const port = context.getNumericPropertyValue('port');
            const user = context.getTextPropertyValue('username');
            const password = context.getTextPropertyValue('password');
            const database = context.getTextPropertyValue('database');
            const table = context.getTextPropertyValue('table');
            const client = new pg_1.Client({
                host,
                port,
                user,
                password,
                database,
            });
            try {
                context.logger.logDebug(`Connecting to database`);
                yield client.connect();
                context.logger.logDebug(`Dropping previous table "${table}" if it exists`);
                yield client.query(jayvee_execution_1.Table.generateDropTableStatement(table));
                context.logger.logDebug(`Creating table "${table}"`);
                yield client.query(input.generateCreateTableStatement(table));
                context.logger.logDebug(`Inserting ${input.getNumberOfRows()} row(s) into table "${table}"`);
                yield client.query(input.generateInsertValuesStatement(table));
                context.logger.logDebug(`The data was successfully loaded into the database`);
                return R.ok(jayvee_execution_1.NONE);
            }
            catch (err) {
                return R.err({
                    message: `Could not write to postgres database: ${err instanceof Error ? err.message : JSON.stringify(err)}`,
                    diagnostic: { node: context.getCurrentNode(), property: 'name' },
                });
            }
            finally {
                yield client.end();
            }
        });
    }
};
PostgresLoaderExecutor.type = 'PostgresLoader';
PostgresLoaderExecutor = tslib_1.__decorate([
    (0, jayvee_execution_1.implementsStatic)()
], PostgresLoaderExecutor);
exports.PostgresLoaderExecutor = PostgresLoaderExecutor;


/***/ }),

/***/ "./libs/extensions/rdbms/exec/src/lib/sqlite-loader-executor.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.SQLiteLoaderExecutor = void 0;
const tslib_1 = __webpack_require__("tslib");
const R = __webpack_require__("./libs/execution/src/index.ts");
const jayvee_execution_1 = __webpack_require__("./libs/execution/src/index.ts");
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
const sqlite3 = __webpack_require__("sqlite3");
let SQLiteLoaderExecutor = class SQLiteLoaderExecutor {
    constructor() {
        this.inputType = jayvee_language_server_1.IOType.TABLE;
        this.outputType = jayvee_language_server_1.IOType.NONE;
    }
    execute(input, context) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const file = context.getTextPropertyValue('file');
            const table = context.getTextPropertyValue('table');
            let db;
            try {
                context.logger.logDebug(`Opening database file ${file}`);
                db = new sqlite3.Database(file);
                context.logger.logDebug(`Dropping previous table "${table}" if it exists`);
                yield this.runQuery(db, jayvee_execution_1.Table.generateDropTableStatement(table));
                context.logger.logDebug(`Creating table "${table}"`);
                yield this.runQuery(db, input.generateCreateTableStatement(table));
                context.logger.logDebug(`Inserting ${input.getNumberOfRows()} row(s) into table "${table}"`);
                yield this.runQuery(db, input.generateInsertValuesStatement(table));
                context.logger.logDebug(`The data was successfully loaded into the database`);
                return R.ok(jayvee_execution_1.NONE);
            }
            catch (err) {
                return R.err({
                    message: `Could not write to sqlite database: ${err instanceof Error ? err.message : JSON.stringify(err)}`,
                    diagnostic: { node: context.getCurrentNode(), property: 'name' },
                });
            }
            finally {
                db === null || db === void 0 ? void 0 : db.close();
            }
        });
    }
    runQuery(db, query) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => {
                db.run(query, (result, error) => error ? reject(error) : resolve(result));
            });
        });
    }
};
SQLiteLoaderExecutor.type = 'SQLiteLoader';
SQLiteLoaderExecutor = tslib_1.__decorate([
    (0, jayvee_execution_1.implementsStatic)()
], SQLiteLoaderExecutor);
exports.SQLiteLoaderExecutor = SQLiteLoaderExecutor;


/***/ }),

/***/ "./libs/extensions/rdbms/lang/src/extension.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.RdbmsLangExtension = void 0;
const lib_1 = __webpack_require__("./libs/extensions/rdbms/lang/src/lib/index.ts");
class RdbmsLangExtension {
    getBlockMetaInf() {
        return [lib_1.PostgresLoaderMetaInformation, lib_1.SQLiteLoaderMetaInformation];
    }
}
exports.RdbmsLangExtension = RdbmsLangExtension;


/***/ }),

/***/ "./libs/extensions/rdbms/lang/src/index.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
const tslib_1 = __webpack_require__("tslib");
tslib_1.__exportStar(__webpack_require__("./libs/extensions/rdbms/lang/src/extension.ts"), exports);


/***/ }),

/***/ "./libs/extensions/rdbms/lang/src/lib/index.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
const tslib_1 = __webpack_require__("tslib");
tslib_1.__exportStar(__webpack_require__("./libs/extensions/rdbms/lang/src/lib/postgres-loader-meta-information.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/extensions/rdbms/lang/src/lib/sqlite-loader-meta-information.ts"), exports);


/***/ }),

/***/ "./libs/extensions/rdbms/lang/src/lib/postgres-loader-meta-information.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.PostgresLoaderMetaInformation = void 0;
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
class PostgresLoaderMetaInformation extends jayvee_language_server_1.BlockMetaInformation {
    constructor() {
        super('PostgresLoader', {
            host: {
                type: jayvee_language_server_1.PropertyValuetype.TEXT,
                docs: {
                    description: 'The hostname or IP address of the Postgres database.',
                },
            },
            port: {
                type: jayvee_language_server_1.PropertyValuetype.INTEGER,
                docs: {
                    description: 'The port of the Postgres database.',
                },
            },
            username: {
                type: jayvee_language_server_1.PropertyValuetype.TEXT,
                docs: {
                    description: 'The username to login to the Postgres database.',
                },
            },
            password: {
                type: jayvee_language_server_1.PropertyValuetype.TEXT,
                docs: {
                    description: 'The password to login to the Postgres database.',
                },
            },
            database: {
                type: jayvee_language_server_1.PropertyValuetype.TEXT,
                docs: {
                    description: 'The database to use.',
                },
            },
            table: {
                type: jayvee_language_server_1.PropertyValuetype.TEXT,
                docs: {
                    description: 'The name of the table to write into.',
                },
            },
        }, jayvee_language_server_1.IOType.TABLE, jayvee_language_server_1.IOType.NONE);
        this.docs.description = 'Loads a `Table` into a PostgreSQL database sink.';
        this.docs.examples = [
            {
                code: blockExampleUsage,
                description: 'A local Postgres instance is filled with table data about cars.',
            },
        ];
    }
}
exports.PostgresLoaderMetaInformation = PostgresLoaderMetaInformation;
const blockExampleUsage = `block CarsLoader oftype PostgresLoader {
  host: "localhost";
  port: "5432";
  username: "postgres";
  password: "postgres";
  database: "CarsDB";
  table: "Cars";
}`;


/***/ }),

/***/ "./libs/extensions/rdbms/lang/src/lib/sqlite-loader-meta-information.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.SQLiteLoaderMetaInformation = void 0;
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
class SQLiteLoaderMetaInformation extends jayvee_language_server_1.BlockMetaInformation {
    constructor() {
        super('SQLiteLoader', {
            table: {
                type: jayvee_language_server_1.PropertyValuetype.TEXT,
                docs: {
                    description: 'The name of the table to write into.',
                },
            },
            file: {
                type: jayvee_language_server_1.PropertyValuetype.TEXT,
                docs: {
                    description: 'The path to the SQLite file that will be created. Usual file extensions are `.sqlite` and `.db`.',
                },
            },
        }, jayvee_language_server_1.IOType.TABLE, jayvee_language_server_1.IOType.NONE);
        this.docs.description = 'Loads a `Table` into a SQLite database sink.';
        this.docs.examples = [
            {
                code: blockExampleUsage,
                description: 'A local SQLite file is created at the given path and filled with table data about cars.',
            },
        ];
    }
}
exports.SQLiteLoaderMetaInformation = SQLiteLoaderMetaInformation;
const blockExampleUsage = `block CarsLoader oftype SQLiteLoader {
  table: "Cars";
  file: "./cars.db";
}`;


/***/ }),

/***/ "./libs/extensions/std/exec/src/archive-interpreter-executor.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.ArchiveInterpreterExecutor = void 0;
const tslib_1 = __webpack_require__("tslib");
const path = __webpack_require__("path");
const R = __webpack_require__("./libs/execution/src/index.ts");
const jayvee_execution_1 = __webpack_require__("./libs/execution/src/index.ts");
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
const JSZip = __webpack_require__("jszip");
const file_util_1 = __webpack_require__("./libs/extensions/std/exec/src/file-util.ts");
let ArchiveInterpreterExecutor = class ArchiveInterpreterExecutor {
    constructor() {
        this.inputType = jayvee_language_server_1.IOType.FILE;
        this.outputType = jayvee_language_server_1.IOType.FILE_SYSTEM;
    }
    execute(archiveFile, context) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (context.getTextPropertyValue('archiveType') === 'zip') {
                const fs = yield this.loadZipFileToInMemoryFileSystem(archiveFile, context);
                if (R.isErr(fs)) {
                    return fs;
                }
                return R.ok(fs.right);
            }
            return R.err({
                message: `Archive is not a zip-archive`,
                diagnostic: { node: context.getCurrentNode(), property: 'name' },
            });
        });
    }
    loadZipFileToInMemoryFileSystem(archiveFile, context) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            context.logger.logDebug(`Loading zip file from binary content`);
            try {
                const jszip = JSZip();
                const root = new jayvee_execution_1.InMemoryFileSystem();
                const archivedObjects = yield jszip.loadAsync(archiveFile.content);
                for (const [relPath, archivedObject] of Object.entries(archivedObjects.files)) {
                    if (!archivedObject.dir) {
                        const content = yield archivedObject.async('arraybuffer');
                        // Ext incl. leading dot
                        const extName = path.extname(archivedObject.name);
                        // Filename without ext and dot
                        const fileName = path.basename(archivedObject.name, extName);
                        const mimeType = (0, file_util_1.inferMimeTypeFromContentTypeString)(extName) ||
                            jayvee_execution_1.MimeType.APPLICATION_OCTET_STREAM;
                        const fileExtension = (0, file_util_1.inferFileExtensionFromFileExtensionString)(extName) ||
                            jayvee_execution_1.FileExtension.NONE;
                        const file = new jayvee_execution_1.BinaryFile(fileName, fileExtension, mimeType, content);
                        root.putFile(relPath, file);
                    }
                }
                return R.ok(root);
            }
            catch (error) {
                return R.err({
                    message: `Unexpected Error ${error instanceof Error ? error.message : JSON.stringify(jayvee_execution_1.err)} occured during processing`,
                    diagnostic: { node: context.getCurrentNode(), property: 'name' },
                });
            }
        });
    }
};
ArchiveInterpreterExecutor.type = 'ArchiveInterpreter';
ArchiveInterpreterExecutor = tslib_1.__decorate([
    (0, jayvee_execution_1.implementsStatic)()
], ArchiveInterpreterExecutor);
exports.ArchiveInterpreterExecutor = ArchiveInterpreterExecutor;


/***/ }),

/***/ "./libs/extensions/std/exec/src/extension.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.StdExecExtension = void 0;
const exec_1 = __webpack_require__("./libs/extensions/rdbms/exec/src/index.ts");
const exec_2 = __webpack_require__("./libs/extensions/tabular/exec/src/index.ts");
const archive_interpreter_executor_1 = __webpack_require__("./libs/extensions/std/exec/src/archive-interpreter-executor.ts");
const file_picker_executor_1 = __webpack_require__("./libs/extensions/std/exec/src/file-picker-executor.ts");
const http_extractor_executor_1 = __webpack_require__("./libs/extensions/std/exec/src/http-extractor-executor.ts");
const text_file_interpreter_executor_1 = __webpack_require__("./libs/extensions/std/exec/src/text-file-interpreter-executor.ts");
const text_line_deleter_executor_1 = __webpack_require__("./libs/extensions/std/exec/src/text-line-deleter-executor.ts");
const text_range_selector_executor_1 = __webpack_require__("./libs/extensions/std/exec/src/text-range-selector-executor.ts");
class StdExecExtension {
    constructor() {
        this.wrappedExtensions = [
            new exec_2.TabularExecExtension(),
            new exec_1.RdbmsExecExtension(),
        ];
    }
    getBlockExecutors() {
        return [
            ...this.wrappedExtensions.map((x) => x.getBlockExecutors()).flat(),
            http_extractor_executor_1.HttpExtractorExecutor,
            text_file_interpreter_executor_1.TextFileInterpreterExecutor,
            text_range_selector_executor_1.TextRangeSelectorExecutor,
            text_line_deleter_executor_1.TextLineDeleterExecutor,
            archive_interpreter_executor_1.ArchiveInterpreterExecutor,
            file_picker_executor_1.FilePickerExecutor,
        ];
    }
}
exports.StdExecExtension = StdExecExtension;


/***/ }),

/***/ "./libs/extensions/std/exec/src/file-picker-executor.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.FilePickerExecutor = void 0;
const tslib_1 = __webpack_require__("tslib");
const R = __webpack_require__("./libs/execution/src/index.ts");
const jayvee_execution_1 = __webpack_require__("./libs/execution/src/index.ts");
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
let FilePickerExecutor = class FilePickerExecutor {
    constructor() {
        this.inputType = jayvee_language_server_1.IOType.FILE_SYSTEM;
        this.outputType = jayvee_language_server_1.IOType.FILE;
    }
    // eslint-disable-next-line @typescript-eslint/require-await
    execute(fileSystem, context) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const file = fileSystem.getFile(context.getTextPropertyValue('path'));
            return R.ok(file);
        });
    }
};
FilePickerExecutor.type = 'FilePicker';
FilePickerExecutor = tslib_1.__decorate([
    (0, jayvee_execution_1.implementsStatic)()
], FilePickerExecutor);
exports.FilePickerExecutor = FilePickerExecutor;


/***/ }),

/***/ "./libs/extensions/std/exec/src/file-util.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.inferFileExtensionFromContentTypeString = exports.inferFileExtensionFromFileExtensionString = exports.inferMimeTypeFromContentTypeString = void 0;
const jayvee_execution_1 = __webpack_require__("./libs/execution/src/index.ts");
const mime = __webpack_require__("mime-types");
function inferMimeTypeFromContentTypeString(fileExtension) {
    if (fileExtension !== undefined) {
        const inferredMimeType = mime.lookup(fileExtension);
        if (Object.values(jayvee_execution_1.MimeType).includes(inferredMimeType)) {
            return inferredMimeType;
        }
    }
    return undefined;
}
exports.inferMimeTypeFromContentTypeString = inferMimeTypeFromContentTypeString;
function inferFileExtensionFromFileExtensionString(fileExtension) {
    if (fileExtension !== undefined) {
        const inferredFileExtension = fileExtension.replace('.', '');
        if (Object.values(jayvee_execution_1.FileExtension).includes(inferredFileExtension)) {
            return inferredFileExtension;
        }
    }
    return undefined;
}
exports.inferFileExtensionFromFileExtensionString = inferFileExtensionFromFileExtensionString;
function inferFileExtensionFromContentTypeString(contentType) {
    if (contentType !== undefined) {
        const inferredFileExtension = mime.extension(contentType);
        if (inferredFileExtension !== false) {
            if (Object.values(jayvee_execution_1.FileExtension).includes(inferredFileExtension)) {
                return inferredFileExtension;
            }
        }
    }
    return undefined;
}
exports.inferFileExtensionFromContentTypeString = inferFileExtensionFromContentTypeString;


/***/ }),

/***/ "./libs/extensions/std/exec/src/http-extractor-executor.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.HttpExtractorExecutor = void 0;
const tslib_1 = __webpack_require__("tslib");
const https = __webpack_require__("https");
const path = __webpack_require__("path");
const R = __webpack_require__("./libs/execution/src/index.ts");
const jayvee_execution_1 = __webpack_require__("./libs/execution/src/index.ts");
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
const file_util_1 = __webpack_require__("./libs/extensions/std/exec/src/file-util.ts");
let HttpExtractorExecutor = class HttpExtractorExecutor {
    constructor() {
        this.inputType = jayvee_language_server_1.IOType.NONE;
        this.outputType = jayvee_language_server_1.IOType.FILE;
    }
    execute(input, context) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const url = context.getTextPropertyValue('url');
            const file = yield this.fetchRawDataAsFile(url, context);
            if (R.isErr(file)) {
                return file;
            }
            return R.ok(file.right);
        });
    }
    fetchRawDataAsFile(url, context) {
        context.logger.logDebug(`Fetching raw data from ${url}`);
        return new Promise((resolve) => {
            https.get(url, (response) => {
                const responseCode = response.statusCode;
                // Catch errors
                if (responseCode === undefined || responseCode >= 400) {
                    resolve(R.err({
                        message: `HTTP fetch failed with code ${responseCode !== null && responseCode !== void 0 ? responseCode : 'undefined'}. Please check your connection.`,
                        diagnostic: { node: context.getOrFailProperty('url') },
                    }));
                }
                // Get chunked data and store to ArrayBuffer
                let rawData = new Uint8Array(0);
                response.on('data', (chunk) => {
                    const tmp = new Uint8Array(rawData.length + chunk.length);
                    tmp.set(rawData, 0);
                    tmp.set(chunk, rawData.length);
                    rawData = tmp;
                });
                // When all data is downloaded, create file
                response.on('end', () => {
                    context.logger.logDebug(`Successfully fetched raw data`);
                    response.headers;
                    // Infer Mimetype from HTTP-Header, if not inferrable, then default to application/octet-stream
                    const mimeType = (0, file_util_1.inferMimeTypeFromContentTypeString)(response.headers['content-type']) || jayvee_execution_1.MimeType.APPLICATION_OCTET_STREAM;
                    // Infer FileName and FileExtension from url, if not inferrable, then default to None
                    // Get last element of URL assuming this is a filename
                    const url = new URL(context.getTextPropertyValue('url'));
                    let fileName = url.pathname.split('/').pop();
                    if (fileName === undefined) {
                        fileName = url.pathname.replace('/', '-');
                    }
                    const extName = path.extname(fileName);
                    let fileExtension = (0, file_util_1.inferFileExtensionFromFileExtensionString)(extName) ||
                        jayvee_execution_1.FileExtension.NONE;
                    // If FileExtension is not in url, try to infer extension from content-type, if not inferrable, then default to None
                    if (fileExtension === jayvee_execution_1.FileExtension.NONE) {
                        fileExtension =
                            (0, file_util_1.inferFileExtensionFromContentTypeString)(response.headers['content-type']) || jayvee_execution_1.FileExtension.NONE;
                    }
                    // Create file and return file
                    const file = new jayvee_execution_1.BinaryFile(fileName, fileExtension, mimeType, rawData.buffer);
                    resolve(R.ok(file));
                });
                response.on('error', (errorObj) => {
                    resolve(R.err({
                        message: errorObj.message,
                        diagnostic: { node: context.getCurrentNode(), property: 'name' },
                    }));
                });
            });
        });
    }
};
HttpExtractorExecutor.type = 'HttpExtractor';
HttpExtractorExecutor = tslib_1.__decorate([
    (0, jayvee_execution_1.implementsStatic)()
], HttpExtractorExecutor);
exports.HttpExtractorExecutor = HttpExtractorExecutor;


/***/ }),

/***/ "./libs/extensions/std/exec/src/index.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
const tslib_1 = __webpack_require__("tslib");
tslib_1.__exportStar(__webpack_require__("./libs/extensions/std/exec/src/extension.ts"), exports);


/***/ }),

/***/ "./libs/extensions/std/exec/src/text-file-interpreter-executor.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.TextFileInterpreterExecutor = void 0;
const tslib_1 = __webpack_require__("tslib");
const util_1 = __webpack_require__("util");
const R = __webpack_require__("./libs/execution/src/index.ts");
const jayvee_execution_1 = __webpack_require__("./libs/execution/src/index.ts");
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
let TextFileInterpreterExecutor = class TextFileInterpreterExecutor {
    constructor() {
        this.inputType = jayvee_language_server_1.IOType.FILE;
        this.outputType = jayvee_language_server_1.IOType.TEXT_FILE;
    }
    // eslint-disable-next-line @typescript-eslint/require-await
    execute(file, context) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const encoding = context.getTextPropertyValue('encoding');
            const lineBreak = context.getRegexPropertyValue('lineBreak');
            const decoder = new util_1.TextDecoder(encoding);
            context.logger.logDebug(`Decoding file content using encoding "${encoding}"`);
            const textContent = decoder.decode(file.content);
            context.logger.logDebug(`Splitting lines using line break /${lineBreak.source}/`);
            const lines = splitLines(textContent, lineBreak);
            context.logger.logDebug(`Lines were split successfully, the resulting text file has ${lines.length} lines`);
            return R.ok(new jayvee_execution_1.TextFile(file.name, file.extension, file.mimeType, lines));
        });
    }
};
TextFileInterpreterExecutor.type = 'TextFileInterpreter';
TextFileInterpreterExecutor = tslib_1.__decorate([
    (0, jayvee_execution_1.implementsStatic)()
], TextFileInterpreterExecutor);
exports.TextFileInterpreterExecutor = TextFileInterpreterExecutor;
function splitLines(textContent, lineBreak) {
    const lines = textContent.split(lineBreak);
    // There may be an additional empty line due to the previous splitting
    if (lines[lines.length - 1] === '') {
        lines.splice(lines.length - 1, 1);
    }
    return lines;
}


/***/ }),

/***/ "./libs/extensions/std/exec/src/text-line-deleter-executor.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.TextLineDeleterExecutor = void 0;
const tslib_1 = __webpack_require__("tslib");
const R = __webpack_require__("./libs/execution/src/index.ts");
const jayvee_execution_1 = __webpack_require__("./libs/execution/src/index.ts");
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
let TextLineDeleterExecutor = class TextLineDeleterExecutor {
    constructor() {
        this.inputType = jayvee_language_server_1.IOType.TEXT_FILE;
        this.outputType = jayvee_language_server_1.IOType.TEXT_FILE;
    }
    // eslint-disable-next-line @typescript-eslint/require-await
    execute(file, context) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const lines = context.getNumericCollectionPropertyValue('lines');
            const numberOfLines = file.content.length;
            for (const line of lines) {
                const lineNumber = line.value;
                if (lineNumber > numberOfLines) {
                    return R.err({
                        message: `Line ${lineNumber} does not exist in the text file, only ${file.content.length} line(s) are present`,
                        diagnostic: { node: line },
                    });
                }
            }
            const distinctLines = new Set(lines.map((l) => l.value));
            const sortedLines = [...distinctLines].sort((a, b) => a - b);
            context.logger.logDebug(`Deleting line(s) ${sortedLines.join(', ')}`);
            const reversedLines = sortedLines.reverse();
            const newContent = [...file.content];
            for (const lineToDelete of reversedLines) {
                newContent.splice(lineToDelete - 1, 1);
            }
            return R.ok(new jayvee_execution_1.TextFile(file.name, file.extension, file.mimeType, newContent));
        });
    }
};
TextLineDeleterExecutor.type = 'TextLineDeleter';
TextLineDeleterExecutor = tslib_1.__decorate([
    (0, jayvee_execution_1.implementsStatic)()
], TextLineDeleterExecutor);
exports.TextLineDeleterExecutor = TextLineDeleterExecutor;


/***/ }),

/***/ "./libs/extensions/std/exec/src/text-range-selector-executor.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.TextRangeSelectorExecutor = void 0;
const tslib_1 = __webpack_require__("tslib");
const R = __webpack_require__("./libs/execution/src/index.ts");
const jayvee_execution_1 = __webpack_require__("./libs/execution/src/index.ts");
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
let TextRangeSelectorExecutor = class TextRangeSelectorExecutor {
    constructor() {
        this.inputType = jayvee_language_server_1.IOType.TEXT_FILE;
        this.outputType = jayvee_language_server_1.IOType.TEXT_FILE;
    }
    // eslint-disable-next-line @typescript-eslint/require-await
    execute(file, context) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const lineFrom = context.getNumericPropertyValue('lineFrom');
            const lineTo = context.getNumericPropertyValue('lineTo');
            const numberOfLines = file.content.length;
            context.logger.logDebug(`Selecting lines from ${lineFrom} to ${lineTo === Number.POSITIVE_INFINITY || lineTo >= numberOfLines
                ? 'the end'
                : `${lineTo}`}`);
            const selectedLines = file.content.slice(lineFrom - 1, lineTo);
            return R.ok(new jayvee_execution_1.TextFile(file.name, file.extension, file.mimeType, selectedLines));
        });
    }
};
TextRangeSelectorExecutor.type = 'TextRangeSelector';
TextRangeSelectorExecutor = tslib_1.__decorate([
    (0, jayvee_execution_1.implementsStatic)()
], TextRangeSelectorExecutor);
exports.TextRangeSelectorExecutor = TextRangeSelectorExecutor;


/***/ }),

/***/ "./libs/extensions/std/lang/src/archive-interpreter-meta-inf.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.ArchiveInterpreterMetaInformation = void 0;
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
class ArchiveInterpreterMetaInformation extends jayvee_language_server_1.BlockMetaInformation {
    constructor() {
        super(
        // How the block type should be called:
        'ArchiveInterpreter', 
        // Property definitions:
        {
            archiveType: {
                type: jayvee_language_server_1.PropertyValuetype.TEXT,
                docs: {
                    description: 'The archive type to be interpreted, e.g., `"zip"`.',
                },
            },
        }, 
        // Input type:
        jayvee_language_server_1.IOType.FILE, 
        // Output type:
        jayvee_language_server_1.IOType.FILE_SYSTEM);
        this.docs.description =
            'Interprets an archive file of type `File` as a `FileSystem`.';
    }
}
exports.ArchiveInterpreterMetaInformation = ArchiveInterpreterMetaInformation;


/***/ }),

/***/ "./libs/extensions/std/lang/src/extension.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.StdLangExtension = void 0;
const lang_1 = __webpack_require__("./libs/extensions/rdbms/lang/src/index.ts");
const lang_2 = __webpack_require__("./libs/extensions/tabular/lang/src/index.ts");
const archive_interpreter_meta_inf_1 = __webpack_require__("./libs/extensions/std/lang/src/archive-interpreter-meta-inf.ts");
const file_picker_meta_inf_1 = __webpack_require__("./libs/extensions/std/lang/src/file-picker-meta-inf.ts");
const http_extractor_meta_inf_1 = __webpack_require__("./libs/extensions/std/lang/src/http-extractor-meta-inf.ts");
const text_file_interpreter_meta_inf_1 = __webpack_require__("./libs/extensions/std/lang/src/text-file-interpreter-meta-inf.ts");
const text_line_deleter_meta_inf_1 = __webpack_require__("./libs/extensions/std/lang/src/text-line-deleter-meta-inf.ts");
const text_range_selector_meta_inf_1 = __webpack_require__("./libs/extensions/std/lang/src/text-range-selector-meta-inf.ts");
class StdLangExtension {
    constructor() {
        this.wrappedExtensions = [
            new lang_2.TabularLangExtension(),
            new lang_1.RdbmsLangExtension(),
        ];
    }
    getBlockMetaInf() {
        return [
            ...this.wrappedExtensions.map((x) => x.getBlockMetaInf()).flat(),
            http_extractor_meta_inf_1.HttpExtractorMetaInformation,
            text_file_interpreter_meta_inf_1.TextFileInterpreterMetaInformation,
            text_range_selector_meta_inf_1.TextRangeSelectorMetaInformation,
            text_line_deleter_meta_inf_1.TextLineDeleterMetaInformation,
            archive_interpreter_meta_inf_1.ArchiveInterpreterMetaInformation,
            file_picker_meta_inf_1.FilePickerMetaInformation,
        ];
    }
}
exports.StdLangExtension = StdLangExtension;


/***/ }),

/***/ "./libs/extensions/std/lang/src/file-picker-meta-inf.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.FilePickerMetaInformation = void 0;
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
class FilePickerMetaInformation extends jayvee_language_server_1.BlockMetaInformation {
    constructor() {
        super(
        // How the block type should be called:
        'FilePicker', 
        // Property definitions:
        {
            path: {
                type: jayvee_language_server_1.PropertyValuetype.TEXT,
                docs: {
                    description: 'The path of the file to get',
                },
            },
        }, 
        // Input type:
        jayvee_language_server_1.IOType.FILE_SYSTEM, 
        // Output type:
        jayvee_language_server_1.IOType.FILE);
        this.docs.description = 'Picks an File out of an FileSystem and outputs it';
        this.docs.examples = [
            {
                code: blockExampleUsage,
                description: 'The block receives a Filesystem and gets the file specified in path-property',
            },
        ];
    }
}
exports.FilePickerMetaInformation = FilePickerMetaInformation;
const blockExampleUsage = `block AgencyFilePicker oftype FilePicker {
  path: "./Agencies.txt";
}`;


/***/ }),

/***/ "./libs/extensions/std/lang/src/http-extractor-meta-inf.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.HttpExtractorMetaInformation = void 0;
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
class HttpExtractorMetaInformation extends jayvee_language_server_1.BlockMetaInformation {
    constructor() {
        super(
        // How the block type should be called:
        'HttpExtractor', 
        // Property definitions:
        {
            url: {
                type: jayvee_language_server_1.PropertyValuetype.TEXT,
                docs: {
                    description: 'The URL to the file in the web to extract.',
                    examples: [
                        {
                            code: 'url: "tinyurl.com/4ub9spwz"',
                            description: 'Specifies the URL to fetch the data from.',
                        },
                    ],
                },
            },
        }, 
        // Input type:
        jayvee_language_server_1.IOType.NONE, 
        // Output type:
        jayvee_language_server_1.IOType.FILE);
        this.docs.description = 'Extracts a `File` from the web.';
        this.docs.examples = [
            {
                code: blockExampleUsage,
                description: 'Fetches a file from the given URL.',
            },
        ];
    }
}
exports.HttpExtractorMetaInformation = HttpExtractorMetaInformation;
const blockExampleUsage = `block CarsFileExtractor oftype HttpExtractor {
  url: "tinyurl.com/4ub9spwz";
}`;


/***/ }),

/***/ "./libs/extensions/std/lang/src/index.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
const tslib_1 = __webpack_require__("tslib");
tslib_1.__exportStar(__webpack_require__("./libs/extensions/std/lang/src/extension.ts"), exports);


/***/ }),

/***/ "./libs/extensions/std/lang/src/text-file-interpreter-meta-inf.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.TextFileInterpreterMetaInformation = void 0;
const assert_1 = __webpack_require__("assert");
const util_1 = __webpack_require__("util");
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
class TextFileInterpreterMetaInformation extends jayvee_language_server_1.BlockMetaInformation {
    constructor() {
        super('TextFileInterpreter', {
            encoding: {
                type: jayvee_language_server_1.PropertyValuetype.TEXT,
                defaultValue: 'utf-8',
                docs: {
                    description: 'The encoding used for decoding the file contents.',
                },
                validation: (property, accept) => {
                    const propertyValue = property.value;
                    (0, assert_1.strict)((0, jayvee_language_server_1.isTextLiteral)(propertyValue));
                    try {
                        new util_1.TextDecoder(propertyValue.value);
                    }
                    catch (error) {
                        accept('error', `Unknown encoding "${propertyValue.value}"`, {
                            node: propertyValue,
                        });
                    }
                },
            },
            lineBreak: {
                type: jayvee_language_server_1.PropertyValuetype.REGEX,
                defaultValue: '\r?\n',
                docs: {
                    description: 'The regex for identifying line breaks.',
                },
            },
        }, 
        // Input type:
        jayvee_language_server_1.IOType.FILE, 
        // Output type:
        jayvee_language_server_1.IOType.TEXT_FILE);
        this.docs.description = 'Interprets a `File` as a `TextFile`.';
    }
}
exports.TextFileInterpreterMetaInformation = TextFileInterpreterMetaInformation;


/***/ }),

/***/ "./libs/extensions/std/lang/src/text-line-deleter-meta-inf.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.TextLineDeleterMetaInformation = void 0;
const assert_1 = __webpack_require__("assert");
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
class TextLineDeleterMetaInformation extends jayvee_language_server_1.BlockMetaInformation {
    constructor() {
        super('TextLineDeleter', {
            lines: {
                type: jayvee_language_server_1.PropertyValuetype.COLLECTION,
                validation: (property, accept) => {
                    const propertyValue = property.value;
                    (0, assert_1.strict)((0, jayvee_language_server_1.isCollectionLiteral)(propertyValue));
                    const { validItems, invalidItems } = (0, jayvee_language_server_1.validateTypedCollection)(propertyValue, jayvee_language_server_1.isNumericLiteral);
                    const invalidTypeErrorMessage = 'Only integers are allowed in this collection';
                    invalidItems.forEach((invalidValue) => accept('error', invalidTypeErrorMessage, {
                        node: invalidValue,
                    }));
                    for (const numericLiteral of validItems) {
                        if (!Number.isInteger(numericLiteral.value)) {
                            accept('error', invalidTypeErrorMessage, {
                                node: numericLiteral,
                            });
                        }
                        if (numericLiteral.value <= 0) {
                            accept('error', `Line numbers need to be greater than zero`, {
                                node: numericLiteral,
                            });
                        }
                    }
                },
                docs: {
                    description: 'The line numbers to delete.',
                },
            },
        }, 
        // Input type:
        jayvee_language_server_1.IOType.TEXT_FILE, 
        // Output type:
        jayvee_language_server_1.IOType.TEXT_FILE);
        this.docs.description = 'Deletes individual lines from a `TextFile`.';
    }
}
exports.TextLineDeleterMetaInformation = TextLineDeleterMetaInformation;


/***/ }),

/***/ "./libs/extensions/std/lang/src/text-range-selector-meta-inf.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.TextRangeSelectorMetaInformation = void 0;
const assert_1 = __webpack_require__("assert");
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
class TextRangeSelectorMetaInformation extends jayvee_language_server_1.BlockMetaInformation {
    constructor() {
        super('TextRangeSelector', {
            lineFrom: {
                type: jayvee_language_server_1.PropertyValuetype.INTEGER,
                defaultValue: 1,
                validation: greaterThanZeroValidation,
            },
            lineTo: {
                type: jayvee_language_server_1.PropertyValuetype.INTEGER,
                defaultValue: Number.POSITIVE_INFINITY,
                validation: greaterThanZeroValidation,
            },
        }, 
        // Input type:
        jayvee_language_server_1.IOType.TEXT_FILE, 
        // Output type:
        jayvee_language_server_1.IOType.TEXT_FILE, (propertyBody, accept) => {
            const lineFromProperty = propertyBody.properties.find((p) => p.name === 'lineFrom');
            const lineToProperty = propertyBody.properties.find((p) => p.name === 'lineTo');
            if (lineFromProperty === undefined || lineToProperty === undefined) {
                return;
            }
            (0, assert_1.strict)((0, jayvee_language_server_1.isNumericLiteral)(lineFromProperty.value));
            (0, assert_1.strict)((0, jayvee_language_server_1.isNumericLiteral)(lineToProperty.value));
            const lineFrom = lineFromProperty.value.value;
            const lineTo = lineToProperty.value.value;
            if (lineFrom > lineTo) {
                [lineFromProperty, lineToProperty].forEach((property) => {
                    accept('error', 'The lower line number needs to be smaller or equal to the upper line number', { node: property.value });
                });
            }
        });
        this.docs.description = 'Selects a range of lines from a `TextFile`.';
    }
}
exports.TextRangeSelectorMetaInformation = TextRangeSelectorMetaInformation;
function greaterThanZeroValidation(property, accept) {
    const propertyValue = property.value;
    (0, assert_1.strict)((0, jayvee_language_server_1.isNumericLiteral)(propertyValue));
    if (propertyValue.value <= 0) {
        accept('error', `Line numbers need to be greater than zero`, {
            node: propertyValue,
        });
    }
}


/***/ }),

/***/ "./libs/extensions/tabular/exec/src/extension.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.TabularExecExtension = void 0;
const cell_range_selector_executor_1 = __webpack_require__("./libs/extensions/tabular/exec/src/lib/cell-range-selector-executor.ts");
const cell_writer_executor_1 = __webpack_require__("./libs/extensions/tabular/exec/src/lib/cell-writer-executor.ts");
const column_deleter_executor_1 = __webpack_require__("./libs/extensions/tabular/exec/src/lib/column-deleter-executor.ts");
const csv_interpreter_executor_1 = __webpack_require__("./libs/extensions/tabular/exec/src/lib/csv-interpreter-executor.ts");
const row_deleter_executor_1 = __webpack_require__("./libs/extensions/tabular/exec/src/lib/row-deleter-executor.ts");
const table_interpreter_executor_1 = __webpack_require__("./libs/extensions/tabular/exec/src/lib/table-interpreter-executor.ts");
class TabularExecExtension {
    getBlockExecutors() {
        return [
            cell_writer_executor_1.CellWriterExecutor,
            column_deleter_executor_1.ColumnDeleterExecutor,
            row_deleter_executor_1.RowDeleterExecutor,
            cell_range_selector_executor_1.CellRangeSelectorExecutor,
            table_interpreter_executor_1.TableInterpreterExecutor,
            csv_interpreter_executor_1.CSVInterpreterExecutor,
        ];
    }
}
exports.TabularExecExtension = TabularExecExtension;


/***/ }),

/***/ "./libs/extensions/tabular/exec/src/index.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
const tslib_1 = __webpack_require__("tslib");
tslib_1.__exportStar(__webpack_require__("./libs/extensions/tabular/exec/src/extension.ts"), exports);


/***/ }),

/***/ "./libs/extensions/tabular/exec/src/lib/cell-range-selector-executor.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.CellRangeSelectorExecutor = void 0;
const tslib_1 = __webpack_require__("tslib");
const R = __webpack_require__("./libs/execution/src/index.ts");
const jayvee_execution_1 = __webpack_require__("./libs/execution/src/index.ts");
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
let CellRangeSelectorExecutor = class CellRangeSelectorExecutor {
    constructor() {
        this.inputType = jayvee_language_server_1.IOType.SHEET;
        this.outputType = jayvee_language_server_1.IOType.SHEET;
    }
    // eslint-disable-next-line @typescript-eslint/require-await
    execute(inputSheet, context) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const relativeRange = context.getCellRangePropertyValue('select');
            const absoluteRange = inputSheet.resolveRelativeIndexes(relativeRange);
            if (!inputSheet.isInBounds(absoluteRange)) {
                return R.err({
                    message: 'The specified cell range does not fit the sheet',
                    diagnostic: { node: absoluteRange.astNode },
                });
            }
            context.logger.logDebug(`Selecting cell range ${absoluteRange.toString()}`);
            const resultingSheet = inputSheet.clone();
            resultingSheet.selectRange(absoluteRange);
            return R.ok(resultingSheet);
        });
    }
};
CellRangeSelectorExecutor.type = 'CellRangeSelector';
CellRangeSelectorExecutor = tslib_1.__decorate([
    (0, jayvee_execution_1.implementsStatic)()
], CellRangeSelectorExecutor);
exports.CellRangeSelectorExecutor = CellRangeSelectorExecutor;


/***/ }),

/***/ "./libs/extensions/tabular/exec/src/lib/cell-writer-executor.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.CellWriterExecutor = void 0;
const tslib_1 = __webpack_require__("tslib");
const assert_1 = __webpack_require__("assert");
const R = __webpack_require__("./libs/execution/src/index.ts");
const jayvee_execution_1 = __webpack_require__("./libs/execution/src/index.ts");
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
let CellWriterExecutor = class CellWriterExecutor {
    constructor() {
        this.inputType = jayvee_language_server_1.IOType.SHEET;
        this.outputType = jayvee_language_server_1.IOType.SHEET;
    }
    // eslint-disable-next-line @typescript-eslint/require-await
    execute(inputSheet, context) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const relativeCell = context.getCellRangePropertyValue('at');
            const content = context.getTextPropertyValue('write');
            (0, assert_1.strict)((0, jayvee_language_server_1.isCellWrapper)(relativeCell));
            const absoluteCell = inputSheet.resolveRelativeIndexes(relativeCell);
            if (!inputSheet.isInBounds(absoluteCell)) {
                return R.err({
                    message: 'The specified cell does not exist in the sheet',
                    diagnostic: { node: absoluteCell.astNode },
                });
            }
            context.logger.logDebug(`Writing "${content}" at cell ${(0, jayvee_language_server_1.getCellIndex)(absoluteCell).toString()}`);
            const resultingSheet = inputSheet.clone();
            resultingSheet.writeCell(absoluteCell, content);
            return R.ok(resultingSheet);
        });
    }
};
CellWriterExecutor.type = 'CellWriter';
CellWriterExecutor = tslib_1.__decorate([
    (0, jayvee_execution_1.implementsStatic)()
], CellWriterExecutor);
exports.CellWriterExecutor = CellWriterExecutor;


/***/ }),

/***/ "./libs/extensions/tabular/exec/src/lib/column-deleter-executor.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.ColumnDeleterExecutor = void 0;
const tslib_1 = __webpack_require__("tslib");
const assert_1 = __webpack_require__("assert");
const R = __webpack_require__("./libs/execution/src/index.ts");
const jayvee_execution_1 = __webpack_require__("./libs/execution/src/index.ts");
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
let ColumnDeleterExecutor = class ColumnDeleterExecutor {
    constructor() {
        this.inputType = jayvee_language_server_1.IOType.SHEET;
        this.outputType = jayvee_language_server_1.IOType.SHEET;
    }
    // eslint-disable-next-line @typescript-eslint/require-await
    execute(inputSheet, context) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const relativeColumns = context.getCellRangeCollectionPropertyValue('delete');
            (0, assert_1.strict)(relativeColumns.every(jayvee_language_server_1.isColumnWrapper));
            let absoluteColumns = relativeColumns.map((column) => inputSheet.resolveRelativeIndexes(column));
            for (const column of absoluteColumns) {
                if (!inputSheet.isInBounds(column)) {
                    const columnIndex = (0, jayvee_language_server_1.getColumnIndex)(column);
                    return R.err({
                        message: `The specified column ${(0, jayvee_language_server_1.columnIndexToString)(columnIndex)} does not exist in the sheet`,
                        diagnostic: { node: column.astNode },
                    });
                }
            }
            // Required for removing duplicates in the next step
            this.sortByColumnIndex(absoluteColumns);
            // That way, the upcoming deletion is only called once per individual column
            absoluteColumns = this.removeDuplicateColumns(absoluteColumns);
            context.logger.logDebug(`Deleting column(s) ${absoluteColumns
                .map(jayvee_language_server_1.getColumnIndex)
                .map(jayvee_language_server_1.columnIndexToString)
                .join(', ')}`);
            // By reversing the order, the column indexes stay stable during deletion
            absoluteColumns.reverse();
            const resultingSheet = inputSheet.clone();
            absoluteColumns.forEach((column) => {
                resultingSheet.deleteColumn(column);
            });
            return R.ok(resultingSheet);
        });
    }
    sortByColumnIndex(columns) {
        columns.sort((columnA, columnB) => (0, jayvee_language_server_1.getColumnIndex)(columnA) - (0, jayvee_language_server_1.getColumnIndex)(columnB));
    }
    removeDuplicateColumns(columns) {
        return columns.reduce((previous, column, index) => {
            const previousColumn = previous[index - 1];
            if (previousColumn !== undefined) {
                if ((0, jayvee_language_server_1.getColumnIndex)(previousColumn) === (0, jayvee_language_server_1.getColumnIndex)(column)) {
                    // The current column is a duplicate because it has the same index as the previous column
                    return previous;
                }
            }
            return [...previous, column];
        }, []);
    }
};
ColumnDeleterExecutor.type = 'ColumnDeleter';
ColumnDeleterExecutor = tslib_1.__decorate([
    (0, jayvee_execution_1.implementsStatic)()
], ColumnDeleterExecutor);
exports.ColumnDeleterExecutor = ColumnDeleterExecutor;


/***/ }),

/***/ "./libs/extensions/tabular/exec/src/lib/csv-interpreter-executor.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.CSVInterpreterExecutor = void 0;
const tslib_1 = __webpack_require__("tslib");
const parse_1 = __webpack_require__("@fast-csv/parse");
const R = __webpack_require__("./libs/execution/src/index.ts");
const jayvee_execution_1 = __webpack_require__("./libs/execution/src/index.ts");
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
const E = __webpack_require__("fp-ts/lib/Either");
const Either_1 = __webpack_require__("fp-ts/lib/Either");
let CSVInterpreterExecutor = class CSVInterpreterExecutor {
    constructor() {
        this.inputType = jayvee_language_server_1.IOType.TEXT_FILE;
        this.outputType = jayvee_language_server_1.IOType.SHEET;
    }
    execute(file, context) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const delimiter = context.getTextPropertyValue('delimiter');
            const enclosing = context.getTextPropertyValue('enclosing');
            const enclosingEscape = context.getTextPropertyValue('enclosingEscape');
            context.logger.logDebug(`Parsing raw data as CSV using delimiter "${delimiter}"`);
            const parseOptions = {
                delimiter,
                quote: enclosing,
                escape: enclosingEscape,
            };
            const csvData = yield parseAsCsv(file.content, parseOptions);
            if ((0, Either_1.isLeft)(csvData)) {
                return Promise.resolve(R.err({
                    message: `CSV parse failed in line ${csvData.left.lineNumber}: ${csvData.left.error.message}`,
                    diagnostic: { node: context.getCurrentNode(), property: 'name' },
                }));
            }
            const sheet = new jayvee_execution_1.Sheet(csvData.right);
            context.logger.logDebug(`Parsing raw data as CSV-sheet successful`);
            return Promise.resolve(R.ok(sheet));
        });
    }
};
CSVInterpreterExecutor.type = 'CSVInterpreter';
CSVInterpreterExecutor = tslib_1.__decorate([
    (0, jayvee_execution_1.implementsStatic)()
], CSVInterpreterExecutor);
exports.CSVInterpreterExecutor = CSVInterpreterExecutor;
function parseAsCsv(lines, parseOptions) {
    var lines_1, lines_1_1;
    var e_1, _a;
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        let lineNumber = 1;
        const rows = [];
        try {
            for (lines_1 = tslib_1.__asyncValues(lines); lines_1_1 = yield lines_1.next(), !lines_1_1.done;) {
                const line = lines_1_1.value;
                const rowParseResult = yield parseLineAsRow(line, parseOptions);
                if ((0, Either_1.isLeft)(rowParseResult)) {
                    return E.left({ error: rowParseResult.left, lineNumber });
                }
                rows.push(rowParseResult.right);
                ++lineNumber;
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (lines_1_1 && !lines_1_1.done && (_a = lines_1.return)) yield _a.call(lines_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
        return E.right(rows);
    });
}
function parseLineAsRow(line, parseOptions) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        return new Promise((resolve) => {
            let row;
            (0, parse_1.parseString)(line, parseOptions)
                .on('data', (data) => {
                row = data;
            })
                .on('error', (error) => {
                resolve(E.left(error));
            })
                .on('end', () => {
                resolve(E.right(row));
            });
        });
    });
}


/***/ }),

/***/ "./libs/extensions/tabular/exec/src/lib/row-deleter-executor.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.RowDeleterExecutor = void 0;
const tslib_1 = __webpack_require__("tslib");
const assert_1 = __webpack_require__("assert");
const R = __webpack_require__("./libs/execution/src/index.ts");
const jayvee_execution_1 = __webpack_require__("./libs/execution/src/index.ts");
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
let RowDeleterExecutor = class RowDeleterExecutor {
    constructor() {
        this.inputType = jayvee_language_server_1.IOType.SHEET;
        this.outputType = jayvee_language_server_1.IOType.SHEET;
    }
    // eslint-disable-next-line @typescript-eslint/require-await
    execute(inputSheet, context) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const relativeRows = context.getCellRangeCollectionPropertyValue('delete');
            (0, assert_1.strict)(relativeRows.every(jayvee_language_server_1.isRowWrapper));
            let absoluteRows = relativeRows.map((row) => inputSheet.resolveRelativeIndexes(row));
            for (const row of absoluteRows) {
                if (!inputSheet.isInBounds(row)) {
                    const rowIndex = (0, jayvee_language_server_1.getRowIndex)(row);
                    return R.err({
                        message: `The specified row ${(0, jayvee_language_server_1.rowIndexToString)(rowIndex)} does not exist in the sheet`,
                        diagnostic: { node: row.astNode },
                    });
                }
            }
            // Required for removing duplicates in the next step
            this.sortByRowIndex(absoluteRows);
            // That way, the upcoming deletion is only called once per individual row
            absoluteRows = this.removeDuplicateRows(absoluteRows);
            context.logger.logDebug(`Deleting row(s) ${absoluteRows
                .map(jayvee_language_server_1.getRowIndex)
                .map(jayvee_language_server_1.rowIndexToString)
                .join(', ')}`);
            // By reversing the order, the row indexes stay stable during deletion
            absoluteRows.reverse();
            const resultingSheet = inputSheet.clone();
            absoluteRows.forEach((row) => {
                resultingSheet.deleteRow(row);
            });
            return R.ok(resultingSheet);
        });
    }
    sortByRowIndex(rows) {
        rows.sort((firstRow, secondRow) => (0, jayvee_language_server_1.getRowIndex)(firstRow) - (0, jayvee_language_server_1.getRowIndex)(secondRow));
    }
    removeDuplicateRows(rows) {
        return rows.reduce((previous, row, index) => {
            const previousRow = previous[index - 1];
            if (previousRow !== undefined) {
                if ((0, jayvee_language_server_1.getRowIndex)(previousRow) === (0, jayvee_language_server_1.getRowIndex)(row)) {
                    // The current row is a duplicate because it has the same index as the previous row
                    return previous;
                }
            }
            return [...previous, row];
        }, []);
    }
};
RowDeleterExecutor.type = 'RowDeleter';
RowDeleterExecutor = tslib_1.__decorate([
    (0, jayvee_execution_1.implementsStatic)()
], RowDeleterExecutor);
exports.RowDeleterExecutor = RowDeleterExecutor;


/***/ }),

/***/ "./libs/extensions/tabular/exec/src/lib/table-interpreter-executor.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.TableInterpreterExecutor = void 0;
const tslib_1 = __webpack_require__("tslib");
const assert_1 = __webpack_require__("assert");
const R = __webpack_require__("./libs/execution/src/index.ts");
const jayvee_execution_1 = __webpack_require__("./libs/execution/src/index.ts");
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
let TableInterpreterExecutor = class TableInterpreterExecutor {
    constructor() {
        this.inputType = jayvee_language_server_1.IOType.SHEET;
        this.outputType = jayvee_language_server_1.IOType.TABLE;
    }
    // eslint-disable-next-line @typescript-eslint/require-await
    execute(inputSheet, context) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const header = context.getBooleanPropertyValue('header');
            const columnDefinitions = context.getValuetypeAssignmentCollectionPropertyValue('columns');
            let columnEntries;
            if (header) {
                if (inputSheet.getNumberOfRows() < 1) {
                    return R.err({
                        message: 'The input sheet is empty and thus has no header',
                        diagnostic: {
                            node: context.getOrFailProperty('header'),
                        },
                    });
                }
                const headerRow = inputSheet.getHeaderRow();
                columnEntries = this.deriveColumnDefinitionEntriesFromHeader(columnDefinitions, headerRow, context);
            }
            else {
                if (inputSheet.getNumberOfColumns() < columnDefinitions.length) {
                    return R.err({
                        message: `There are ${columnDefinitions.length} column definitions but the input sheet only has ${inputSheet.getNumberOfColumns()} columns`,
                        diagnostic: {
                            node: context.getOrFailProperty('columns'),
                        },
                    });
                }
                columnEntries =
                    this.deriveColumnDefinitionEntriesWithoutHeader(columnDefinitions);
            }
            const numberOfTableRows = header
                ? inputSheet.getNumberOfRows() - 1
                : inputSheet.getNumberOfRows();
            context.logger.logDebug(`Validating ${numberOfTableRows} row(s) according to the column types`);
            const tableData = this.constructAndValidateTableData(inputSheet, header, columnEntries, context);
            context.logger.logDebug(`Validation completed, the resulting table has ${tableData.length} row(s) and ${columnEntries.length} column(s)`);
            const columnInformation = columnEntries.map((columnEntry) => ({
                name: columnEntry.columnName,
                type: columnEntry.valuetype,
            }));
            const resultingTable = new jayvee_execution_1.Table(columnInformation, tableData);
            return R.ok(resultingTable);
        });
    }
    constructAndValidateTableData(sheet, header, columnEntries, context) {
        const tableData = [];
        sheet.iterateRows((sheetRow, sheetRowIndex) => {
            if (header && sheetRowIndex === 0) {
                return;
            }
            const tableRow = this.constructAndValidateTableRow(sheetRow, sheetRowIndex, columnEntries, context);
            if (tableRow === undefined) {
                context.logger.logDebug(`Omitting row ${(0, jayvee_language_server_1.rowIndexToString)(sheetRowIndex)}`);
            }
            else {
                tableData.push(tableRow);
            }
        });
        return tableData;
    }
    constructAndValidateTableRow(sheetRow, sheetRowIndex, columnEntries, context) {
        let invalidRow = false;
        const tableRow = [];
        columnEntries.forEach((columnEntry, tableColumnIndex) => {
            const sheetColumnIndex = columnEntry.sheetColumnIndex;
            // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
            const value = sheetRow[sheetColumnIndex];
            if (!columnEntry.valuetype.isValid(value, context)) {
                const cellIndex = new jayvee_language_server_1.CellIndex(sheetColumnIndex, sheetRowIndex);
                context.logger.logDebug(`The value at cell ${cellIndex.toString()} does not match the type ${(0, jayvee_language_server_1.getValuetypeName)(columnEntry.astNode.type)}`);
                invalidRow = true;
                return;
            }
            tableRow[tableColumnIndex] = value;
        });
        // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
        if (invalidRow) {
            return undefined;
        }
        (0, assert_1.strict)(tableRow.length === columnEntries.length);
        return tableRow;
    }
    deriveColumnDefinitionEntriesWithoutHeader(columnDefinitions) {
        return columnDefinitions.map((columnDefinition, columnDefinitionIndex) => ({
            sheetColumnIndex: columnDefinitionIndex,
            columnName: columnDefinition.name,
            valuetype: (0, jayvee_execution_1.createValuetype)(columnDefinition.type),
            astNode: columnDefinition,
        }));
    }
    deriveColumnDefinitionEntriesFromHeader(columnDefinitions, headerRow, context) {
        context.logger.logDebug(`Matching header with provided column names`);
        const columnEntries = [];
        for (const columnDefinition of columnDefinitions) {
            const indexOfMatchingHeader = headerRow.findIndex((headerColumnName) => headerColumnName === columnDefinition.name);
            if (indexOfMatchingHeader === -1) {
                context.logger.logDebug(`Omitting column "${columnDefinition.name}" as the name was not found in the header`);
                continue;
            }
            columnEntries.push({
                sheetColumnIndex: indexOfMatchingHeader,
                columnName: columnDefinition.name,
                valuetype: (0, jayvee_execution_1.createValuetype)(columnDefinition.type),
                astNode: columnDefinition,
            });
        }
        return columnEntries;
    }
};
TableInterpreterExecutor.type = 'TableInterpreter';
TableInterpreterExecutor = tslib_1.__decorate([
    (0, jayvee_execution_1.implementsStatic)()
], TableInterpreterExecutor);
exports.TableInterpreterExecutor = TableInterpreterExecutor;


/***/ }),

/***/ "./libs/extensions/tabular/lang/src/extension.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.TabularLangExtension = void 0;
const cell_range_selector_meta_inf_1 = __webpack_require__("./libs/extensions/tabular/lang/src/lib/cell-range-selector-meta-inf.ts");
const cell_writer_meta_inf_1 = __webpack_require__("./libs/extensions/tabular/lang/src/lib/cell-writer-meta-inf.ts");
const column_deleter_meta_inf_1 = __webpack_require__("./libs/extensions/tabular/lang/src/lib/column-deleter-meta-inf.ts");
const csv_interpreter_meta_inf_1 = __webpack_require__("./libs/extensions/tabular/lang/src/lib/csv-interpreter-meta-inf.ts");
const row_deleter_meta_inf_1 = __webpack_require__("./libs/extensions/tabular/lang/src/lib/row-deleter-meta-inf.ts");
const table_interpreter_meta_inf_1 = __webpack_require__("./libs/extensions/tabular/lang/src/lib/table-interpreter-meta-inf.ts");
class TabularLangExtension {
    getBlockMetaInf() {
        return [
            column_deleter_meta_inf_1.ColumnDeleterMetaInformation,
            row_deleter_meta_inf_1.RowDeleterMetaInformation,
            cell_range_selector_meta_inf_1.CellRangeSelectorMetaInformation,
            cell_writer_meta_inf_1.CellWriterMetaInformation,
            table_interpreter_meta_inf_1.TableInterpreterMetaInformation,
            csv_interpreter_meta_inf_1.CSVInterpreterMetaInformation,
        ];
    }
}
exports.TabularLangExtension = TabularLangExtension;


/***/ }),

/***/ "./libs/extensions/tabular/lang/src/index.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
const tslib_1 = __webpack_require__("tslib");
tslib_1.__exportStar(__webpack_require__("./libs/extensions/tabular/lang/src/extension.ts"), exports);


/***/ }),

/***/ "./libs/extensions/tabular/lang/src/lib/cell-range-selector-meta-inf.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.CellRangeSelectorMetaInformation = void 0;
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
class CellRangeSelectorMetaInformation extends jayvee_language_server_1.BlockMetaInformation {
    constructor() {
        super('CellRangeSelector', {
            select: {
                type: jayvee_language_server_1.PropertyValuetype.CELL_RANGE,
                docs: {
                    description: 'The cell range to select.',
                    examples: [
                        {
                            code: 'select: range A1:E*',
                            description: 'Select cells from `A1` to the last cell of column `E`.',
                        },
                    ],
                },
            },
        }, jayvee_language_server_1.IOType.SHEET, jayvee_language_server_1.IOType.SHEET);
        this.docs.description =
            'Selects a subset of a `Sheet` to produce a new `Sheet`.';
        this.docs.examples = [
            {
                code: blockExample,
                description: 'Selects the cells in the given range and produces a new `Sheet` containing only the selected cells.',
            },
        ];
    }
}
exports.CellRangeSelectorMetaInformation = CellRangeSelectorMetaInformation;
const blockExample = `block CarsCoreDataSelector oftype CellRangeSelector {
  select: range A1:E*;
}`;


/***/ }),

/***/ "./libs/extensions/tabular/lang/src/lib/cell-writer-meta-inf.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.CellWriterMetaInformation = void 0;
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
class CellWriterMetaInformation extends jayvee_language_server_1.BlockMetaInformation {
    constructor() {
        super('CellWriter', {
            write: {
                type: jayvee_language_server_1.PropertyValuetype.TEXT,
                docs: {
                    description: 'The value to write.',
                    examples: [
                        {
                            code: 'write: "Name"',
                            description: 'Write the value "Name" into the cell',
                        },
                    ],
                },
            },
            at: {
                type: jayvee_language_server_1.PropertyValuetype.CELL_RANGE,
                validation: (property, accept) => {
                    const propertyValue = property.value;
                    if (!(0, jayvee_language_server_1.isCellRangeLiteral)(propertyValue)) {
                        return;
                    }
                    if (!jayvee_language_server_1.CellRangeWrapper.canBeWrapped(propertyValue)) {
                        return;
                    }
                    const semanticCellRange = new jayvee_language_server_1.CellRangeWrapper(propertyValue);
                    if (!(0, jayvee_language_server_1.isCellWrapper)(semanticCellRange)) {
                        accept('error', 'A single cell needs to be selected', {
                            node: semanticCellRange.astNode,
                        });
                    }
                },
                docs: {
                    description: 'The cell to write into.',
                    examples: [{ code: 'at: A1', description: 'Write into cell A1' }],
                    validation: 'You need to specify exactly one cell.',
                },
            },
        }, jayvee_language_server_1.IOType.SHEET, jayvee_language_server_1.IOType.SHEET);
        this.docs.description = 'Writes a textual value into a cell of a `Sheet`.';
        this.docs.examples = [
            {
                code: blockExample,
                description: 'Write the value "Name" into cell `A1`.',
            },
        ];
    }
}
exports.CellWriterMetaInformation = CellWriterMetaInformation;
const blockExample = `block NameHeaderWriter oftype CellWriter {
  at: cell A1;
  write: "Name";
}`;


/***/ }),

/***/ "./libs/extensions/tabular/lang/src/lib/column-deleter-meta-inf.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.ColumnDeleterMetaInformation = void 0;
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
class ColumnDeleterMetaInformation extends jayvee_language_server_1.BlockMetaInformation {
    constructor() {
        super('ColumnDeleter', {
            delete: {
                type: jayvee_language_server_1.PropertyValuetype.COLLECTION,
                validation: (property, accept) => {
                    const propertyValue = property.value;
                    if (!(0, jayvee_language_server_1.isCollectionLiteral)(propertyValue)) {
                        return;
                    }
                    const { validItems, invalidItems } = (0, jayvee_language_server_1.validateTypedCollection)(propertyValue, jayvee_language_server_1.isCellRangeLiteral);
                    invalidItems.forEach((invalidValue) => accept('error', 'Only cell ranges are allowed in this collection', {
                        node: invalidValue,
                    }));
                    for (const collectionValue of validItems) {
                        if (!jayvee_language_server_1.CellRangeWrapper.canBeWrapped(collectionValue)) {
                            continue;
                        }
                        const semanticCellRange = new jayvee_language_server_1.CellRangeWrapper(collectionValue);
                        if (!(0, jayvee_language_server_1.isColumnWrapper)(semanticCellRange)) {
                            accept('error', 'An entire column needs to be selected', {
                                node: semanticCellRange.astNode,
                            });
                        }
                    }
                },
                docs: {
                    description: 'The columns to delete.',
                    examples: [
                        {
                            code: 'delete: [column B]',
                            description: 'Delete column B.',
                        },
                        {
                            code: 'delete: [column B, column C]',
                            description: 'Delete column B and column C.',
                        },
                    ],
                    validation: 'You need to specify at least one column.',
                },
            },
        }, jayvee_language_server_1.IOType.SHEET, jayvee_language_server_1.IOType.SHEET);
        this.docs.description =
            'Deletes columns from a `Sheet`. Column IDs of subsequent columns will be shifted accordingly, so there will be no gaps.';
        this.docs.examples = [
            {
                code: blockExample,
                description: 'Deletes column B (i.e. the second column).',
            },
        ];
    }
}
exports.ColumnDeleterMetaInformation = ColumnDeleterMetaInformation;
const blockExample = `block MpgColumnDeleter oftype ColumnDeleter {
  delete: [column B];
}`;


/***/ }),

/***/ "./libs/extensions/tabular/lang/src/lib/csv-interpreter-meta-inf.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.CSVInterpreterMetaInformation = void 0;
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
class CSVInterpreterMetaInformation extends jayvee_language_server_1.BlockMetaInformation {
    constructor() {
        super(
        // How the block type should be called:
        'CSVInterpreter', 
        // Property definitions:
        {
            delimiter: {
                type: jayvee_language_server_1.PropertyValuetype.TEXT,
                defaultValue: ',',
                docs: {
                    description: 'The delimiter for values in the CSV file.',
                    examples: [
                        {
                            code: 'delimiter: ","',
                            description: 'Commas are used to separate values in the CSV file.',
                        },
                    ],
                },
            },
            enclosing: {
                type: jayvee_language_server_1.PropertyValuetype.TEXT,
                defaultValue: '',
                docs: {
                    description: 'The enclosing character that may be used for values in the CSV file.',
                },
            },
            enclosingEscape: {
                type: jayvee_language_server_1.PropertyValuetype.TEXT,
                defaultValue: '',
                docs: {
                    description: 'The character to escape enclosing characters in values.',
                },
            },
        }, 
        // Input type:
        jayvee_language_server_1.IOType.TEXT_FILE, 
        // Output type:
        jayvee_language_server_1.IOType.SHEET);
        this.docs.description =
            'Interprets an input file as a csv-file containing string-values delimited by `delimiter` and outputs a `Sheet`.';
        this.docs.examples = [
            {
                code: blockExample,
                description: 'Interprets an input file as a csv-file containing string-values delimited by `;` and outputs `Sheet`.',
            },
        ];
    }
}
exports.CSVInterpreterMetaInformation = CSVInterpreterMetaInformation;
const blockExample = `block AgencyCSVInterpreter oftype CSVInterpreter {  
    delimiter: ";"
  }`;


/***/ }),

/***/ "./libs/extensions/tabular/lang/src/lib/row-deleter-meta-inf.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.RowDeleterMetaInformation = void 0;
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
class RowDeleterMetaInformation extends jayvee_language_server_1.BlockMetaInformation {
    constructor() {
        super('RowDeleter', {
            delete: {
                type: jayvee_language_server_1.PropertyValuetype.COLLECTION,
                validation: (property, accept) => {
                    const propertyValue = property.value;
                    if (!(0, jayvee_language_server_1.isCollectionLiteral)(propertyValue)) {
                        return;
                    }
                    const { validItems, invalidItems } = (0, jayvee_language_server_1.validateTypedCollection)(propertyValue, jayvee_language_server_1.isCellRangeLiteral);
                    invalidItems.forEach((invalidValue) => accept('error', 'Only cell ranges are allowed in this collection', {
                        node: invalidValue,
                    }));
                    for (const collectionValue of validItems) {
                        if (!jayvee_language_server_1.CellRangeWrapper.canBeWrapped(collectionValue)) {
                            continue;
                        }
                        const semanticCellRange = new jayvee_language_server_1.CellRangeWrapper(collectionValue);
                        if (!(0, jayvee_language_server_1.isRowWrapper)(semanticCellRange)) {
                            accept('error', 'An entire row needs to be selected', {
                                node: semanticCellRange.astNode,
                            });
                        }
                    }
                },
                docs: {
                    description: 'The rows to delete.',
                    examples: [
                        {
                            code: 'delete: [row 2]',
                            description: 'Delete row 2.',
                        },
                        {
                            code: 'delete: [row 2, row 3]',
                            description: 'Delete row 2 and row 3.',
                        },
                    ],
                    validation: 'You need to specify at least one row.',
                },
            },
        }, jayvee_language_server_1.IOType.SHEET, jayvee_language_server_1.IOType.SHEET);
        this.docs.description =
            'Deletes one or more rows from a `Sheet`. Row IDs of subsequent rows will be shifted accordingly, so there will be no gaps.';
        this.docs.examples = [
            {
                code: blockExample,
                description: 'Deletes row 2 (i.e. the second row).',
            },
        ];
    }
}
exports.RowDeleterMetaInformation = RowDeleterMetaInformation;
const blockExample = `block SecondRowDeleter oftype ColumnDeleter {
  delete: [row 2];
}`;


/***/ }),

/***/ "./libs/extensions/tabular/lang/src/lib/table-interpreter-meta-inf.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.TableInterpreterMetaInformation = void 0;
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
class TableInterpreterMetaInformation extends jayvee_language_server_1.BlockMetaInformation {
    constructor() {
        super('TableInterpreter', {
            header: {
                type: jayvee_language_server_1.PropertyValuetype.BOOLEAN,
                docs: {
                    description: 'Whether the first row should be interpreted as header row.',
                    examples: [
                        {
                            code: 'header: true',
                            description: 'The first row is interpreted as table header. The values in the header row will become the column names of the table.',
                        },
                        {
                            code: 'header: false',
                            description: 'The first row is NOT interpreted as table header and columns of the sheet are directly mapped to table columns. The column names are taken form the provided names in the `columns` property.',
                        },
                    ],
                },
            },
            columns: {
                type: jayvee_language_server_1.PropertyValuetype.COLLECTION,
                validation: (property, accept) => {
                    const propertyValue = property.value;
                    if (!(0, jayvee_language_server_1.isCollectionLiteral)(propertyValue)) {
                        return;
                    }
                    const { validItems, invalidItems } = (0, jayvee_language_server_1.validateTypedCollection)(propertyValue, jayvee_language_server_1.isValuetypeAssignmentLiteral);
                    invalidItems.forEach((invalidValue) => accept('error', 'Only type assignments are allowed in this collection', {
                        node: invalidValue,
                    }));
                    const valuetypeAssignments = validItems.map((assignment) => assignment.value);
                    (0, jayvee_language_server_1.getNodesWithNonUniqueNames)(valuetypeAssignments).forEach((valuetypeAssignment) => {
                        accept('error', `The column name "${valuetypeAssignment.name}" needs to be unique.`, {
                            node: valuetypeAssignment,
                            property: 'name',
                        });
                    });
                },
                docs: {
                    description: 'Collection of valuetype assignments. Uses column names (potentially matched with the header or by sequence depending on the `header` property) to assign a primitive valuetype to each column.',
                    examples: [
                        {
                            code: 'columns: [ "name" typed text ]',
                            description: 'There is one column with the header "name". All values in this colum are typed as text.',
                        },
                    ],
                    validation: 'Needs to be a collection of valuetype assignments. Each column needs to have a unique name.',
                },
            },
        }, jayvee_language_server_1.IOType.SHEET, jayvee_language_server_1.IOType.TABLE);
        this.docs.description =
            'Interprets a `Sheet` as a `Table`. In case a header row is present in the sheet, its names can be matched with the provided column names. Otherwise, the provided column names are assigned in order.';
        this.docs.examples = [
            {
                code: blockExampleWithHeader,
                description: 'Interprets a `Sheet` about cars with a topmost header row and interprets it as a `Table` by assigning a primitive valuetype to each column. The column names are matched to the header, so the order of the type assignments does not matter.',
            },
            {
                code: blockExampleWithoutHeader,
                description: 'Interprets a `Sheet` about cars without a topmost header row and interprets it as a `Table` by sequentially assigning a name and a primitive valuetype to each column of the sheet. Note that the order of columns matters here. The first column (column `A`) will be named "name", the second column (column `B`) will be named "mpg" etc.',
            },
        ];
    }
}
exports.TableInterpreterMetaInformation = TableInterpreterMetaInformation;
const blockExampleWithHeader = `block CarsTableInterpreter oftype TableInterpreter {
  header: true;
  columns: [
    "name" typed text,
    "mpg" typed decimal,
    "cyl" typed integer,
  ];
}`;
const blockExampleWithoutHeader = `block CarsTableInterpreter oftype TableInterpreter {
  header: false;
  columns: [
    "name" typed text,
    "mpg" typed decimal,
    "cyl" typed integer,
  ];
}`;


/***/ }),

/***/ "./libs/language-server/src/index.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
const tslib_1 = __webpack_require__("tslib");
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/index.ts"), exports);


/***/ }),

/***/ "./libs/language-server/src/lib/ast/collection-util.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.validateTypedCollection = exports.isTypedCollection = void 0;
const ast_1 = __webpack_require__("./libs/language-server/src/lib/ast/generated/ast.ts");
function isTypedCollection(collection, collectionItemTypeGuard) {
    if (!(0, ast_1.isCollectionLiteral)(collection)) {
        return false;
    }
    return (validateTypedCollection(collection, collectionItemTypeGuard).invalidItems
        .length === 0);
}
exports.isTypedCollection = isTypedCollection;
function validateTypedCollection(collection, collectionItemTypeGuard) {
    const validItems = collection.values.filter(collectionItemTypeGuard);
    const invalidItems = collection.values.filter((item) => !collectionItemTypeGuard(item));
    return {
        validItems: validItems,
        invalidItems: invalidItems,
    };
}
exports.validateTypedCollection = validateTypedCollection;


/***/ }),

/***/ "./libs/language-server/src/lib/ast/generated/ast.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


/******************************************************************************
 * This file was generated by langium-cli 1.0.0.
 * DO NOT EDIT MANUALLY!
 ******************************************************************************/
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.isSinglePipeDefinition = exports.SinglePipeDefinition = exports.isRuntimeParameterLiteral = exports.RuntimeParameterLiteral = exports.isRowLiteral = exports.RowLiteral = exports.isRegexLiteral = exports.RegexLiteral = exports.isRangeLiteral = exports.RangeLiteral = exports.isPropertyBody = exports.PropertyBody = exports.isPropertyAssignment = exports.PropertyAssignment = exports.isPrimitiveValuetypeKeywordLiteral = exports.PrimitiveValuetypeKeywordLiteral = exports.isPipelineDefinition = exports.PipelineDefinition = exports.isNumericLiteral = exports.NumericLiteral = exports.isJayveeModel = exports.JayveeModel = exports.isConstraintTypeLiteral = exports.ConstraintTypeLiteral = exports.isConstraintReferenceLiteral = exports.ConstraintReferenceLiteral = exports.isConstraintDefinition = exports.ConstraintDefinition = exports.isColumnLiteral = exports.ColumnLiteral = exports.isCollectionLiteral = exports.CollectionLiteral = exports.isChainedPipeDefinition = exports.ChainedPipeDefinition = exports.isCellLiteral = exports.CellLiteral = exports.isBooleanLiteral = exports.BooleanLiteral = exports.isBlockTypeLiteral = exports.BlockTypeLiteral = exports.isBlockDefinition = exports.BlockDefinition = exports.isPropertyValueLiteral = exports.PropertyValueLiteral = exports.isPipeDefinition = exports.PipeDefinition = exports.isCellRangeLiteral = exports.CellRangeLiteral = exports.isAtomicLiteral = exports.AtomicLiteral = void 0;
exports.reflection = exports.JayveeAstReflection = exports.isValuetypeDefinitionReference = exports.ValuetypeDefinitionReference = exports.isValuetypeDefinition = exports.ValuetypeDefinition = exports.isValuetypeAssignmentLiteral = exports.ValuetypeAssignmentLiteral = exports.isValuetypeAssignment = exports.ValuetypeAssignment = exports.isTextLiteral = exports.TextLiteral = void 0;
/* eslint-disable */
const langium_1 = __webpack_require__("langium");
exports.AtomicLiteral = 'AtomicLiteral';
function isAtomicLiteral(item) {
    return exports.reflection.isInstance(item, exports.AtomicLiteral);
}
exports.isAtomicLiteral = isAtomicLiteral;
exports.CellRangeLiteral = 'CellRangeLiteral';
function isCellRangeLiteral(item) {
    return exports.reflection.isInstance(item, exports.CellRangeLiteral);
}
exports.isCellRangeLiteral = isCellRangeLiteral;
exports.PipeDefinition = 'PipeDefinition';
function isPipeDefinition(item) {
    return exports.reflection.isInstance(item, exports.PipeDefinition);
}
exports.isPipeDefinition = isPipeDefinition;
exports.PropertyValueLiteral = 'PropertyValueLiteral';
function isPropertyValueLiteral(item) {
    return exports.reflection.isInstance(item, exports.PropertyValueLiteral);
}
exports.isPropertyValueLiteral = isPropertyValueLiteral;
exports.BlockDefinition = 'BlockDefinition';
function isBlockDefinition(item) {
    return exports.reflection.isInstance(item, exports.BlockDefinition);
}
exports.isBlockDefinition = isBlockDefinition;
exports.BlockTypeLiteral = 'BlockTypeLiteral';
function isBlockTypeLiteral(item) {
    return exports.reflection.isInstance(item, exports.BlockTypeLiteral);
}
exports.isBlockTypeLiteral = isBlockTypeLiteral;
exports.BooleanLiteral = 'BooleanLiteral';
function isBooleanLiteral(item) {
    return exports.reflection.isInstance(item, exports.BooleanLiteral);
}
exports.isBooleanLiteral = isBooleanLiteral;
exports.CellLiteral = 'CellLiteral';
function isCellLiteral(item) {
    return exports.reflection.isInstance(item, exports.CellLiteral);
}
exports.isCellLiteral = isCellLiteral;
exports.ChainedPipeDefinition = 'ChainedPipeDefinition';
function isChainedPipeDefinition(item) {
    return exports.reflection.isInstance(item, exports.ChainedPipeDefinition);
}
exports.isChainedPipeDefinition = isChainedPipeDefinition;
exports.CollectionLiteral = 'CollectionLiteral';
function isCollectionLiteral(item) {
    return exports.reflection.isInstance(item, exports.CollectionLiteral);
}
exports.isCollectionLiteral = isCollectionLiteral;
exports.ColumnLiteral = 'ColumnLiteral';
function isColumnLiteral(item) {
    return exports.reflection.isInstance(item, exports.ColumnLiteral);
}
exports.isColumnLiteral = isColumnLiteral;
exports.ConstraintDefinition = 'ConstraintDefinition';
function isConstraintDefinition(item) {
    return exports.reflection.isInstance(item, exports.ConstraintDefinition);
}
exports.isConstraintDefinition = isConstraintDefinition;
exports.ConstraintReferenceLiteral = 'ConstraintReferenceLiteral';
function isConstraintReferenceLiteral(item) {
    return exports.reflection.isInstance(item, exports.ConstraintReferenceLiteral);
}
exports.isConstraintReferenceLiteral = isConstraintReferenceLiteral;
exports.ConstraintTypeLiteral = 'ConstraintTypeLiteral';
function isConstraintTypeLiteral(item) {
    return exports.reflection.isInstance(item, exports.ConstraintTypeLiteral);
}
exports.isConstraintTypeLiteral = isConstraintTypeLiteral;
exports.JayveeModel = 'JayveeModel';
function isJayveeModel(item) {
    return exports.reflection.isInstance(item, exports.JayveeModel);
}
exports.isJayveeModel = isJayveeModel;
exports.NumericLiteral = 'NumericLiteral';
function isNumericLiteral(item) {
    return exports.reflection.isInstance(item, exports.NumericLiteral);
}
exports.isNumericLiteral = isNumericLiteral;
exports.PipelineDefinition = 'PipelineDefinition';
function isPipelineDefinition(item) {
    return exports.reflection.isInstance(item, exports.PipelineDefinition);
}
exports.isPipelineDefinition = isPipelineDefinition;
exports.PrimitiveValuetypeKeywordLiteral = 'PrimitiveValuetypeKeywordLiteral';
function isPrimitiveValuetypeKeywordLiteral(item) {
    return exports.reflection.isInstance(item, exports.PrimitiveValuetypeKeywordLiteral);
}
exports.isPrimitiveValuetypeKeywordLiteral = isPrimitiveValuetypeKeywordLiteral;
exports.PropertyAssignment = 'PropertyAssignment';
function isPropertyAssignment(item) {
    return exports.reflection.isInstance(item, exports.PropertyAssignment);
}
exports.isPropertyAssignment = isPropertyAssignment;
exports.PropertyBody = 'PropertyBody';
function isPropertyBody(item) {
    return exports.reflection.isInstance(item, exports.PropertyBody);
}
exports.isPropertyBody = isPropertyBody;
exports.RangeLiteral = 'RangeLiteral';
function isRangeLiteral(item) {
    return exports.reflection.isInstance(item, exports.RangeLiteral);
}
exports.isRangeLiteral = isRangeLiteral;
exports.RegexLiteral = 'RegexLiteral';
function isRegexLiteral(item) {
    return exports.reflection.isInstance(item, exports.RegexLiteral);
}
exports.isRegexLiteral = isRegexLiteral;
exports.RowLiteral = 'RowLiteral';
function isRowLiteral(item) {
    return exports.reflection.isInstance(item, exports.RowLiteral);
}
exports.isRowLiteral = isRowLiteral;
exports.RuntimeParameterLiteral = 'RuntimeParameterLiteral';
function isRuntimeParameterLiteral(item) {
    return exports.reflection.isInstance(item, exports.RuntimeParameterLiteral);
}
exports.isRuntimeParameterLiteral = isRuntimeParameterLiteral;
exports.SinglePipeDefinition = 'SinglePipeDefinition';
function isSinglePipeDefinition(item) {
    return exports.reflection.isInstance(item, exports.SinglePipeDefinition);
}
exports.isSinglePipeDefinition = isSinglePipeDefinition;
exports.TextLiteral = 'TextLiteral';
function isTextLiteral(item) {
    return exports.reflection.isInstance(item, exports.TextLiteral);
}
exports.isTextLiteral = isTextLiteral;
exports.ValuetypeAssignment = 'ValuetypeAssignment';
function isValuetypeAssignment(item) {
    return exports.reflection.isInstance(item, exports.ValuetypeAssignment);
}
exports.isValuetypeAssignment = isValuetypeAssignment;
exports.ValuetypeAssignmentLiteral = 'ValuetypeAssignmentLiteral';
function isValuetypeAssignmentLiteral(item) {
    return exports.reflection.isInstance(item, exports.ValuetypeAssignmentLiteral);
}
exports.isValuetypeAssignmentLiteral = isValuetypeAssignmentLiteral;
exports.ValuetypeDefinition = 'ValuetypeDefinition';
function isValuetypeDefinition(item) {
    return exports.reflection.isInstance(item, exports.ValuetypeDefinition);
}
exports.isValuetypeDefinition = isValuetypeDefinition;
exports.ValuetypeDefinitionReference = 'ValuetypeDefinitionReference';
function isValuetypeDefinitionReference(item) {
    return exports.reflection.isInstance(item, exports.ValuetypeDefinitionReference);
}
exports.isValuetypeDefinitionReference = isValuetypeDefinitionReference;
class JayveeAstReflection extends langium_1.AbstractAstReflection {
    getAllTypes() {
        return ['AtomicLiteral', 'BlockDefinition', 'BlockTypeLiteral', 'BooleanLiteral', 'CellLiteral', 'CellRangeLiteral', 'ChainedPipeDefinition', 'CollectionLiteral', 'ColumnLiteral', 'ConstraintDefinition', 'ConstraintReferenceLiteral', 'ConstraintTypeLiteral', 'JayveeModel', 'NumericLiteral', 'PipeDefinition', 'PipelineDefinition', 'PrimitiveValuetypeKeywordLiteral', 'PropertyAssignment', 'PropertyBody', 'PropertyValueLiteral', 'RangeLiteral', 'RegexLiteral', 'RowLiteral', 'RuntimeParameterLiteral', 'SinglePipeDefinition', 'TextLiteral', 'ValuetypeAssignment', 'ValuetypeAssignmentLiteral', 'ValuetypeDefinition', 'ValuetypeDefinitionReference'];
    }
    computeIsSubtype(subtype, supertype) {
        switch (subtype) {
            case exports.BooleanLiteral:
            case exports.ConstraintReferenceLiteral:
            case exports.NumericLiteral:
            case exports.RegexLiteral:
            case exports.TextLiteral:
            case exports.ValuetypeAssignmentLiteral:
            case exports.CellRangeLiteral: {
                return this.isSubtype(exports.AtomicLiteral, supertype);
            }
            case exports.CellLiteral:
            case exports.ColumnLiteral:
            case exports.RangeLiteral:
            case exports.RowLiteral: {
                return this.isSubtype(exports.CellRangeLiteral, supertype);
            }
            case exports.ChainedPipeDefinition:
            case exports.SinglePipeDefinition: {
                return this.isSubtype(exports.PipeDefinition, supertype);
            }
            case exports.CollectionLiteral:
            case exports.AtomicLiteral: {
                return this.isSubtype(exports.PropertyValueLiteral, supertype);
            }
            default: {
                return false;
            }
        }
    }
    getReferenceType(refInfo) {
        const referenceId = `${refInfo.container.$type}:${refInfo.property}`;
        switch (referenceId) {
            case 'ChainedPipeDefinition:blocks':
            case 'SinglePipeDefinition:from':
            case 'SinglePipeDefinition:to': {
                return exports.BlockDefinition;
            }
            case 'ConstraintReferenceLiteral:value': {
                return exports.ConstraintDefinition;
            }
            case 'ValuetypeDefinitionReference:reference': {
                return exports.ValuetypeDefinition;
            }
            default: {
                throw new Error(`${referenceId} is not a valid reference id.`);
            }
        }
    }
    getTypeMetaData(type) {
        switch (type) {
            case 'BooleanLiteral': {
                return {
                    name: 'BooleanLiteral',
                    mandatory: [
                        { name: 'value', type: 'boolean' }
                    ]
                };
            }
            case 'ChainedPipeDefinition': {
                return {
                    name: 'ChainedPipeDefinition',
                    mandatory: [
                        { name: 'blocks', type: 'array' }
                    ]
                };
            }
            case 'CollectionLiteral': {
                return {
                    name: 'CollectionLiteral',
                    mandatory: [
                        { name: 'values', type: 'array' }
                    ]
                };
            }
            case 'JayveeModel': {
                return {
                    name: 'JayveeModel',
                    mandatory: [
                        { name: 'constraints', type: 'array' },
                        { name: 'pipelines', type: 'array' },
                        { name: 'valuetypes', type: 'array' }
                    ]
                };
            }
            case 'PipelineDefinition': {
                return {
                    name: 'PipelineDefinition',
                    mandatory: [
                        { name: 'blocks', type: 'array' },
                        { name: 'constraints', type: 'array' },
                        { name: 'pipes', type: 'array' },
                        { name: 'valuetypes', type: 'array' }
                    ]
                };
            }
            case 'PropertyBody': {
                return {
                    name: 'PropertyBody',
                    mandatory: [
                        { name: 'properties', type: 'array' }
                    ]
                };
            }
            default: {
                return {
                    name: type,
                    mandatory: []
                };
            }
        }
    }
}
exports.JayveeAstReflection = JayveeAstReflection;
exports.reflection = new JayveeAstReflection();


/***/ }),

/***/ "./libs/language-server/src/lib/ast/generated/grammar.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


/******************************************************************************
 * This file was generated by langium-cli 1.0.0.
 * DO NOT EDIT MANUALLY!
 ******************************************************************************/
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.JayveeGrammar = void 0;
const langium_1 = __webpack_require__("langium");
let loadedJayveeGrammar;
const JayveeGrammar = () => loadedJayveeGrammar !== null && loadedJayveeGrammar !== void 0 ? loadedJayveeGrammar : (loadedJayveeGrammar = (0, langium_1.loadGrammarFromJson)(`{
  "$type": "Grammar",
  "isDeclared": true,
  "name": "Jayvee",
  "imports": [],
  "rules": [
    {
      "$type": "ParserRule",
      "name": "JayveeModel",
      "entry": true,
      "definition": {
        "$type": "Alternatives",
        "elements": [
          {
            "$type": "Assignment",
            "feature": "pipelines",
            "operator": "+=",
            "terminal": {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@1"
              },
              "arguments": []
            }
          },
          {
            "$type": "Assignment",
            "feature": "valuetypes",
            "operator": "+=",
            "terminal": {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@27"
              },
              "arguments": []
            }
          },
          {
            "$type": "Assignment",
            "feature": "constraints",
            "operator": "+=",
            "terminal": {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@38"
              },
              "arguments": []
            }
          }
        ],
        "cardinality": "*"
      },
      "definesHiddenTokens": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "PipelineDefinition",
      "definition": {
        "$type": "Group",
        "elements": [
          {
            "$type": "Keyword",
            "value": "pipeline"
          },
          {
            "$type": "Assignment",
            "feature": "name",
            "operator": "=",
            "terminal": {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@10"
              },
              "arguments": []
            }
          },
          {
            "$type": "Keyword",
            "value": "{"
          },
          {
            "$type": "Alternatives",
            "elements": [
              {
                "$type": "Assignment",
                "feature": "blocks",
                "operator": "+=",
                "terminal": {
                  "$type": "RuleCall",
                  "rule": {
                    "$ref": "#/rules@2"
                  },
                  "arguments": []
                }
              },
              {
                "$type": "Assignment",
                "feature": "pipes",
                "operator": "+=",
                "terminal": {
                  "$type": "RuleCall",
                  "rule": {
                    "$ref": "#/rules@4"
                  },
                  "arguments": []
                }
              },
              {
                "$type": "Assignment",
                "feature": "valuetypes",
                "operator": "+=",
                "terminal": {
                  "$type": "RuleCall",
                  "rule": {
                    "$ref": "#/rules@27"
                  },
                  "arguments": []
                }
              },
              {
                "$type": "Assignment",
                "feature": "constraints",
                "operator": "+=",
                "terminal": {
                  "$type": "RuleCall",
                  "rule": {
                    "$ref": "#/rules@38"
                  },
                  "arguments": []
                }
              }
            ],
            "cardinality": "*"
          },
          {
            "$type": "Keyword",
            "value": "}"
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "BlockDefinition",
      "definition": {
        "$type": "Group",
        "elements": [
          {
            "$type": "Keyword",
            "value": "block"
          },
          {
            "$type": "Assignment",
            "feature": "name",
            "operator": "=",
            "terminal": {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@10"
              },
              "arguments": []
            }
          },
          {
            "$type": "Keyword",
            "value": "oftype"
          },
          {
            "$type": "Assignment",
            "feature": "type",
            "operator": "=",
            "terminal": {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@3"
              },
              "arguments": []
            }
          },
          {
            "$type": "Assignment",
            "feature": "body",
            "operator": "=",
            "terminal": {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@17"
              },
              "arguments": []
            }
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "BlockTypeLiteral",
      "definition": {
        "$type": "Assignment",
        "feature": "name",
        "operator": "=",
        "terminal": {
          "$type": "RuleCall",
          "rule": {
            "$ref": "#/rules@10"
          },
          "arguments": []
        }
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "PipeDefinition",
      "definition": {
        "$type": "Alternatives",
        "elements": [
          {
            "$type": "RuleCall",
            "rule": {
              "$ref": "#/rules@5"
            },
            "arguments": []
          },
          {
            "$type": "RuleCall",
            "rule": {
              "$ref": "#/rules@6"
            },
            "arguments": []
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "SinglePipeDefinition",
      "definition": {
        "$type": "Group",
        "elements": [
          {
            "$type": "Keyword",
            "value": "pipe"
          },
          {
            "$type": "Keyword",
            "value": "{"
          },
          {
            "$type": "Keyword",
            "value": "from"
          },
          {
            "$type": "Keyword",
            "value": ":"
          },
          {
            "$type": "Assignment",
            "feature": "from",
            "operator": "=",
            "terminal": {
              "$type": "CrossReference",
              "type": {
                "$ref": "#/rules@2"
              },
              "deprecatedSyntax": false
            }
          },
          {
            "$type": "Keyword",
            "value": ";"
          },
          {
            "$type": "Keyword",
            "value": "to"
          },
          {
            "$type": "Keyword",
            "value": ":"
          },
          {
            "$type": "Assignment",
            "feature": "to",
            "operator": "=",
            "terminal": {
              "$type": "CrossReference",
              "type": {
                "$ref": "#/rules@2"
              },
              "deprecatedSyntax": false
            }
          },
          {
            "$type": "Keyword",
            "value": ";"
          },
          {
            "$type": "Keyword",
            "value": "}"
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "ChainedPipeDefinition",
      "definition": {
        "$type": "Group",
        "elements": [
          {
            "$type": "Assignment",
            "feature": "blocks",
            "operator": "+=",
            "terminal": {
              "$type": "CrossReference",
              "type": {
                "$ref": "#/rules@2"
              },
              "deprecatedSyntax": false
            }
          },
          {
            "$type": "Group",
            "elements": [
              {
                "$type": "Keyword",
                "value": "->"
              },
              {
                "$type": "Assignment",
                "feature": "blocks",
                "operator": "+=",
                "terminal": {
                  "$type": "CrossReference",
                  "type": {
                    "$ref": "#/rules@2"
                  },
                  "deprecatedSyntax": false
                }
              }
            ],
            "cardinality": "+"
          },
          {
            "$type": "Keyword",
            "value": ";"
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "TerminalRule",
      "name": "TRUE",
      "type": {
        "$type": "ReturnType",
        "name": "boolean"
      },
      "definition": {
        "$type": "RegexToken",
        "regex": "true"
      },
      "fragment": false,
      "hidden": false
    },
    {
      "$type": "TerminalRule",
      "name": "FALSE",
      "type": {
        "$type": "ReturnType",
        "name": "boolean"
      },
      "definition": {
        "$type": "RegexToken",
        "regex": "false"
      },
      "fragment": false,
      "hidden": false
    },
    {
      "$type": "TerminalRule",
      "name": "CELL_REFERENCE",
      "definition": {
        "$type": "RegexToken",
        "regex": "([A-Z]+|\\\\*)([0-9]+|\\\\*)"
      },
      "fragment": false,
      "hidden": false
    },
    {
      "$type": "TerminalRule",
      "name": "ID",
      "definition": {
        "$type": "RegexToken",
        "regex": "[_a-zA-Z][\\\\w_]*"
      },
      "fragment": false,
      "hidden": false
    },
    {
      "$type": "TerminalRule",
      "name": "INTEGER",
      "type": {
        "$type": "ReturnType",
        "name": "number"
      },
      "definition": {
        "$type": "RegexToken",
        "regex": "[0-9]+"
      },
      "fragment": false,
      "hidden": false
    },
    {
      "$type": "TerminalRule",
      "name": "NUMBER",
      "type": {
        "$type": "ReturnType",
        "name": "number"
      },
      "definition": {
        "$type": "RegexToken",
        "regex": "[+-]?[0-9]+(\\\\.[0-9]+)?"
      },
      "fragment": false,
      "hidden": false
    },
    {
      "$type": "TerminalRule",
      "name": "STRING",
      "definition": {
        "$type": "RegexToken",
        "regex": "\\"[^\\"]*\\"|'[^']*'"
      },
      "fragment": false,
      "hidden": false
    },
    {
      "$type": "TerminalRule",
      "name": "REGEX",
      "definition": {
        "$type": "RegexToken",
        "regex": "\\\\/.+\\\\/"
      },
      "fragment": false,
      "hidden": false
    },
    {
      "$type": "TerminalRule",
      "hidden": true,
      "name": "WHITESPACE",
      "definition": {
        "$type": "RegexToken",
        "regex": "\\\\s+"
      },
      "fragment": false
    },
    {
      "$type": "TerminalRule",
      "hidden": true,
      "name": "SINGLE_LINE_COMMENT",
      "definition": {
        "$type": "RegexToken",
        "regex": "\\\\/\\\\/[^\\\\n\\\\r]*"
      },
      "fragment": false
    },
    {
      "$type": "ParserRule",
      "name": "PropertyBody",
      "definition": {
        "$type": "Group",
        "elements": [
          {
            "$type": "Keyword",
            "value": "{"
          },
          {
            "$type": "Assignment",
            "feature": "properties",
            "operator": "+=",
            "terminal": {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@18"
              },
              "arguments": []
            },
            "cardinality": "*"
          },
          {
            "$type": "Keyword",
            "value": "}"
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "PropertyAssignment",
      "definition": {
        "$type": "Group",
        "elements": [
          {
            "$type": "Assignment",
            "feature": "name",
            "operator": "=",
            "terminal": {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@10"
              },
              "arguments": []
            }
          },
          {
            "$type": "Keyword",
            "value": ":"
          },
          {
            "$type": "Assignment",
            "feature": "value",
            "operator": "=",
            "terminal": {
              "$type": "Alternatives",
              "elements": [
                {
                  "$type": "RuleCall",
                  "rule": {
                    "$ref": "#/rules@20"
                  },
                  "arguments": []
                },
                {
                  "$type": "RuleCall",
                  "rule": {
                    "$ref": "#/rules@19"
                  },
                  "arguments": []
                }
              ]
            }
          },
          {
            "$type": "Keyword",
            "value": ";"
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "RuntimeParameterLiteral",
      "definition": {
        "$type": "Group",
        "elements": [
          {
            "$type": "Keyword",
            "value": "requires"
          },
          {
            "$type": "Assignment",
            "feature": "name",
            "operator": "=",
            "terminal": {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@10"
              },
              "arguments": []
            }
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "PropertyValueLiteral",
      "definition": {
        "$type": "Alternatives",
        "elements": [
          {
            "$type": "RuleCall",
            "rule": {
              "$ref": "#/rules@21"
            },
            "arguments": []
          },
          {
            "$type": "RuleCall",
            "rule": {
              "$ref": "#/rules@22"
            },
            "arguments": []
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "AtomicLiteral",
      "definition": {
        "$type": "Alternatives",
        "elements": [
          {
            "$type": "RuleCall",
            "rule": {
              "$ref": "#/rules@23"
            },
            "arguments": []
          },
          {
            "$type": "RuleCall",
            "rule": {
              "$ref": "#/rules@24"
            },
            "arguments": []
          },
          {
            "$type": "RuleCall",
            "rule": {
              "$ref": "#/rules@25"
            },
            "arguments": []
          },
          {
            "$type": "RuleCall",
            "rule": {
              "$ref": "#/rules@33"
            },
            "arguments": []
          },
          {
            "$type": "RuleCall",
            "rule": {
              "$ref": "#/rules@26"
            },
            "arguments": []
          },
          {
            "$type": "RuleCall",
            "rule": {
              "$ref": "#/rules@28"
            },
            "arguments": []
          },
          {
            "$type": "RuleCall",
            "rule": {
              "$ref": "#/rules@40"
            },
            "arguments": []
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "CollectionLiteral",
      "definition": {
        "$type": "Group",
        "elements": [
          {
            "$type": "Keyword",
            "value": "["
          },
          {
            "$type": "Group",
            "elements": [
              {
                "$type": "Assignment",
                "feature": "values",
                "operator": "+=",
                "terminal": {
                  "$type": "RuleCall",
                  "rule": {
                    "$ref": "#/rules@21"
                  },
                  "arguments": []
                }
              },
              {
                "$type": "Group",
                "elements": [
                  {
                    "$type": "Keyword",
                    "value": ","
                  },
                  {
                    "$type": "Assignment",
                    "feature": "values",
                    "operator": "+=",
                    "terminal": {
                      "$type": "RuleCall",
                      "rule": {
                        "$ref": "#/rules@21"
                      },
                      "arguments": []
                    }
                  }
                ],
                "cardinality": "*"
              }
            ],
            "cardinality": "?"
          },
          {
            "$type": "Keyword",
            "value": ",",
            "cardinality": "?"
          },
          {
            "$type": "Keyword",
            "value": "]"
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "TextLiteral",
      "definition": {
        "$type": "Assignment",
        "feature": "value",
        "operator": "=",
        "terminal": {
          "$type": "RuleCall",
          "rule": {
            "$ref": "#/rules@13"
          },
          "arguments": []
        }
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "NumericLiteral",
      "definition": {
        "$type": "Assignment",
        "feature": "value",
        "operator": "=",
        "terminal": {
          "$type": "Alternatives",
          "elements": [
            {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@11"
              },
              "arguments": []
            },
            {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@12"
              },
              "arguments": []
            }
          ]
        }
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "BooleanLiteral",
      "definition": {
        "$type": "Assignment",
        "feature": "value",
        "operator": "=",
        "terminal": {
          "$type": "Alternatives",
          "elements": [
            {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@7"
              },
              "arguments": []
            },
            {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@8"
              },
              "arguments": []
            }
          ]
        }
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "RegexLiteral",
      "definition": {
        "$type": "Assignment",
        "feature": "value",
        "operator": "=",
        "terminal": {
          "$type": "RuleCall",
          "rule": {
            "$ref": "#/rules@14"
          },
          "arguments": []
        }
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "ValuetypeDefinition",
      "definition": {
        "$type": "Group",
        "elements": [
          {
            "$type": "Keyword",
            "value": "valuetype"
          },
          {
            "$type": "Assignment",
            "feature": "name",
            "operator": "=",
            "terminal": {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@10"
              },
              "arguments": []
            }
          },
          {
            "$type": "Keyword",
            "value": "oftype"
          },
          {
            "$type": "Assignment",
            "feature": "type",
            "operator": "=",
            "terminal": {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@31"
              },
              "arguments": []
            }
          },
          {
            "$type": "Keyword",
            "value": "{"
          },
          {
            "$type": "Keyword",
            "value": "constraints"
          },
          {
            "$type": "Keyword",
            "value": ":"
          },
          {
            "$type": "Assignment",
            "feature": "constraints",
            "operator": "=",
            "terminal": {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@22"
              },
              "arguments": []
            }
          },
          {
            "$type": "Keyword",
            "value": ";"
          },
          {
            "$type": "Keyword",
            "value": "}"
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "ValuetypeAssignmentLiteral",
      "definition": {
        "$type": "Assignment",
        "feature": "value",
        "operator": "=",
        "terminal": {
          "$type": "RuleCall",
          "rule": {
            "$ref": "#/rules@29"
          },
          "arguments": []
        }
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "ValuetypeAssignment",
      "definition": {
        "$type": "Group",
        "elements": [
          {
            "$type": "Assignment",
            "feature": "name",
            "operator": "=",
            "terminal": {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@13"
              },
              "arguments": []
            }
          },
          {
            "$type": "Keyword",
            "value": "oftype"
          },
          {
            "$type": "Assignment",
            "feature": "type",
            "operator": "=",
            "terminal": {
              "$type": "Alternatives",
              "elements": [
                {
                  "$type": "RuleCall",
                  "rule": {
                    "$ref": "#/rules@31"
                  },
                  "arguments": []
                },
                {
                  "$type": "RuleCall",
                  "rule": {
                    "$ref": "#/rules@30"
                  },
                  "arguments": []
                }
              ]
            }
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "ValuetypeDefinitionReference",
      "definition": {
        "$type": "Assignment",
        "feature": "reference",
        "operator": "=",
        "terminal": {
          "$type": "CrossReference",
          "type": {
            "$ref": "#/rules@27"
          },
          "deprecatedSyntax": false
        }
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "PrimitiveValuetypeKeywordLiteral",
      "definition": {
        "$type": "Assignment",
        "feature": "keyword",
        "operator": "=",
        "terminal": {
          "$type": "RuleCall",
          "rule": {
            "$ref": "#/rules@32"
          },
          "arguments": []
        }
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "PrimitiveValuetypeKeyword",
      "dataType": "string",
      "definition": {
        "$type": "Alternatives",
        "elements": [
          {
            "$type": "Keyword",
            "value": "text"
          },
          {
            "$type": "Keyword",
            "value": "decimal"
          },
          {
            "$type": "Keyword",
            "value": "integer"
          },
          {
            "$type": "Keyword",
            "value": "boolean"
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "CellRangeLiteral",
      "definition": {
        "$type": "Alternatives",
        "elements": [
          {
            "$type": "RuleCall",
            "rule": {
              "$ref": "#/rules@34"
            },
            "arguments": []
          },
          {
            "$type": "RuleCall",
            "rule": {
              "$ref": "#/rules@35"
            },
            "arguments": []
          },
          {
            "$type": "RuleCall",
            "rule": {
              "$ref": "#/rules@36"
            },
            "arguments": []
          },
          {
            "$type": "RuleCall",
            "rule": {
              "$ref": "#/rules@37"
            },
            "arguments": []
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "RangeLiteral",
      "definition": {
        "$type": "Group",
        "elements": [
          {
            "$type": "Keyword",
            "value": "range"
          },
          {
            "$type": "Assignment",
            "feature": "cellFrom",
            "operator": "=",
            "terminal": {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@9"
              },
              "arguments": []
            }
          },
          {
            "$type": "Keyword",
            "value": ":"
          },
          {
            "$type": "Assignment",
            "feature": "cellTo",
            "operator": "=",
            "terminal": {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@9"
              },
              "arguments": []
            }
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "ColumnLiteral",
      "definition": {
        "$type": "Group",
        "elements": [
          {
            "$type": "Keyword",
            "value": "column"
          },
          {
            "$type": "Assignment",
            "feature": "columnId",
            "operator": "=",
            "terminal": {
              "$type": "Alternatives",
              "elements": [
                {
                  "$type": "RuleCall",
                  "rule": {
                    "$ref": "#/rules@10"
                  },
                  "arguments": []
                },
                {
                  "$type": "Keyword",
                  "value": "*"
                }
              ]
            }
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "RowLiteral",
      "definition": {
        "$type": "Group",
        "elements": [
          {
            "$type": "Keyword",
            "value": "row"
          },
          {
            "$type": "Assignment",
            "feature": "rowId",
            "operator": "=",
            "terminal": {
              "$type": "Alternatives",
              "elements": [
                {
                  "$type": "RuleCall",
                  "rule": {
                    "$ref": "#/rules@11"
                  },
                  "arguments": []
                },
                {
                  "$type": "Keyword",
                  "value": "*"
                }
              ]
            }
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "CellLiteral",
      "definition": {
        "$type": "Group",
        "elements": [
          {
            "$type": "Keyword",
            "value": "cell"
          },
          {
            "$type": "Assignment",
            "feature": "cellId",
            "operator": "=",
            "terminal": {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@9"
              },
              "arguments": []
            }
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "ConstraintDefinition",
      "definition": {
        "$type": "Group",
        "elements": [
          {
            "$type": "Keyword",
            "value": "constraint"
          },
          {
            "$type": "Assignment",
            "feature": "name",
            "operator": "=",
            "terminal": {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@10"
              },
              "arguments": []
            }
          },
          {
            "$type": "Keyword",
            "value": "oftype"
          },
          {
            "$type": "Assignment",
            "feature": "type",
            "operator": "=",
            "terminal": {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@39"
              },
              "arguments": []
            }
          },
          {
            "$type": "Assignment",
            "feature": "body",
            "operator": "=",
            "terminal": {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@17"
              },
              "arguments": []
            }
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "ConstraintTypeLiteral",
      "definition": {
        "$type": "Assignment",
        "feature": "name",
        "operator": "=",
        "terminal": {
          "$type": "RuleCall",
          "rule": {
            "$ref": "#/rules@10"
          },
          "arguments": []
        }
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "ConstraintReferenceLiteral",
      "definition": {
        "$type": "Assignment",
        "feature": "value",
        "operator": "=",
        "terminal": {
          "$type": "CrossReference",
          "type": {
            "$ref": "#/rules@38"
          },
          "deprecatedSyntax": false
        }
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    }
  ],
  "definesHiddenTokens": false,
  "hiddenTokens": [],
  "interfaces": [],
  "types": [],
  "usedGrammars": []
}`));
exports.JayveeGrammar = JayveeGrammar;


/***/ }),

/***/ "./libs/language-server/src/lib/ast/generated/module.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


/******************************************************************************
 * This file was generated by langium-cli 1.0.0.
 * DO NOT EDIT MANUALLY!
 ******************************************************************************/
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.JayveeGeneratedModule = exports.JayveeGeneratedSharedModule = exports.JayveeLanguageMetaData = void 0;
const ast_1 = __webpack_require__("./libs/language-server/src/lib/ast/generated/ast.ts");
const grammar_1 = __webpack_require__("./libs/language-server/src/lib/ast/generated/grammar.ts");
exports.JayveeLanguageMetaData = {
    languageId: 'jayvee',
    fileExtensions: ['.jv'],
    caseInsensitive: false
};
exports.JayveeGeneratedSharedModule = {
    AstReflection: () => new ast_1.JayveeAstReflection()
};
exports.JayveeGeneratedModule = {
    Grammar: () => (0, grammar_1.JayveeGrammar)(),
    LanguageMetaData: () => exports.JayveeLanguageMetaData,
    parser: {}
};


/***/ }),

/***/ "./libs/language-server/src/lib/ast/index.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
const tslib_1 = __webpack_require__("tslib");
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/ast/generated/ast.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/ast/generated/grammar.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/ast/generated/module.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/ast/wrappers/index.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/ast/model-util.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/ast/collection-util.ts"), exports);


/***/ }),

/***/ "./libs/language-server/src/lib/ast/model-util.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.getValuetypeName = exports.inferTypesFromValue = exports.runtimeParameterAllowedForType = exports.PropertyValuetype = exports.IOType = exports.getBlocksInTopologicalSorting = exports.collectAllPipes = exports.collectIngoingPipes = exports.collectOutgoingPipes = exports.collectParents = exports.collectChildren = exports.collectStartingBlocks = void 0;
const assert_1 = __webpack_require__("assert");
const langium_1 = __webpack_require__("langium");
// eslint-disable-next-line import/no-cycle
const meta_inf_registry_1 = __webpack_require__("./libs/language-server/src/lib/meta-information/meta-inf-registry.ts");
const ast_1 = __webpack_require__("./libs/language-server/src/lib/ast/generated/ast.ts");
const pipe_wrapper_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/pipe-wrapper.ts");
function collectStartingBlocks(pipeline) {
    const result = [];
    for (const block of pipeline.blocks) {
        const blockMetaInf = (0, meta_inf_registry_1.getMetaInformation)(block.type);
        if (blockMetaInf === undefined) {
            continue;
        }
        if (!blockMetaInf.hasInput()) {
            result.push(block);
        }
    }
    return result;
}
exports.collectStartingBlocks = collectStartingBlocks;
function collectChildren(block) {
    const outgoingPipes = collectOutgoingPipes(block);
    return outgoingPipes.map((pipe) => pipe.to);
}
exports.collectChildren = collectChildren;
function collectParents(block) {
    const ingoingPipes = collectIngoingPipes(block);
    return ingoingPipes.map((pipe) => pipe.from);
}
exports.collectParents = collectParents;
function collectOutgoingPipes(block) {
    return collectPipes(block, 'outgoing');
}
exports.collectOutgoingPipes = collectOutgoingPipes;
function collectIngoingPipes(block) {
    return collectPipes(block, 'ingoing');
}
exports.collectIngoingPipes = collectIngoingPipes;
function collectPipes(block, kind) {
    const pipeline = block.$container;
    const allPipes = collectAllPipes(pipeline);
    return allPipes.filter((semanticPipe) => {
        switch (kind) {
            case 'outgoing':
                return semanticPipe.from === block;
            case 'ingoing':
                return semanticPipe.to === block;
        }
        return (0, langium_1.assertUnreachable)(kind);
    });
}
function collectAllPipes(pipeline) {
    const result = [];
    for (const pipe of pipeline.pipes) {
        result.push(...(0, pipe_wrapper_1.createSemanticPipes)(pipe));
    }
    return result;
}
exports.collectAllPipes = collectAllPipes;
/**
 * Returns blocks in a pipeline in topological order, based on
 * Kahn's algorithm.
 *
 * Considers a pipeline as a directed, acyclical graph where
 * blocks are nodes and pipes are edges. A list in topological
 * order has the property that parent nodes are always listed
 * before their children.
 *
 * "[...] a list in topological order is such that no element
 * appears in it until after all elements appearing on all paths
 * leading to the particular element have been listed."
 *
 * Kahn, A. B. (1962). Topological sorting of large networks. Communications of the ACM, 5(11), 558–562.
 */
function getBlocksInTopologicalSorting(pipeline) {
    const sortedNodes = [];
    const currentNodes = [...collectStartingBlocks(pipeline)];
    let unvisitedEdges = [...collectAllPipes(pipeline)];
    while (currentNodes.length > 0) {
        const node = currentNodes.pop();
        (0, assert_1.strict)(node !== undefined);
        sortedNodes.push(node);
        for (const childNode of collectChildren(node)) {
            // Mark edges between parent and child as visited
            collectIngoingPipes(childNode)
                .filter((e) => e.from === node)
                .forEach((e) => {
                unvisitedEdges = unvisitedEdges.filter((edge) => !edge.equals(e));
            });
            // If all edges to the child have been visited
            const notRemovedEdges = collectIngoingPipes(childNode).filter((e) => unvisitedEdges.some((edge) => edge.equals(e)));
            if (notRemovedEdges.length === 0) {
                // Insert it into currentBlocks
                currentNodes.push(childNode);
            }
        }
    }
    // If the graph still contains unvisited edges it is not a DAG
    (0, assert_1.strict)(unvisitedEdges.length === 0, `The pipeline ${pipeline.name} is expected to have no cycles`);
    return sortedNodes;
}
exports.getBlocksInTopologicalSorting = getBlocksInTopologicalSorting;
var IOType;
(function (IOType) {
    IOType["NONE"] = "None";
    IOType["FILE"] = "File";
    IOType["TEXT_FILE"] = "TextFile";
    IOType["FILE_SYSTEM"] = "FileSystem";
    IOType["SHEET"] = "Sheet";
    IOType["TABLE"] = "Table";
})(IOType = exports.IOType || (exports.IOType = {}));
var PropertyValuetype;
(function (PropertyValuetype) {
    PropertyValuetype["TEXT"] = "text";
    PropertyValuetype["INTEGER"] = "integer";
    PropertyValuetype["DECIMAL"] = "decimal";
    PropertyValuetype["BOOLEAN"] = "boolean";
    PropertyValuetype["CELL_RANGE"] = "cell-range";
    PropertyValuetype["REGEX"] = "regex";
    PropertyValuetype["COLLECTION"] = "collection";
    PropertyValuetype["VALUETYPE_ASSIGNMENT"] = "valuetype-assignment";
    PropertyValuetype["CONSTRAINT"] = "constraint";
})(PropertyValuetype = exports.PropertyValuetype || (exports.PropertyValuetype = {}));
function runtimeParameterAllowedForType(type) {
    switch (type) {
        case PropertyValuetype.CELL_RANGE:
        case PropertyValuetype.REGEX:
        case PropertyValuetype.VALUETYPE_ASSIGNMENT:
        case PropertyValuetype.COLLECTION:
        case PropertyValuetype.CONSTRAINT:
            return false;
        case PropertyValuetype.TEXT:
        case PropertyValuetype.INTEGER:
        case PropertyValuetype.DECIMAL:
        case PropertyValuetype.BOOLEAN:
            return true;
        default:
            (0, langium_1.assertUnreachable)(type);
    }
}
exports.runtimeParameterAllowedForType = runtimeParameterAllowedForType;
function inferTypesFromValue(value) {
    if ((0, ast_1.isTextLiteral)(value)) {
        return [PropertyValuetype.TEXT];
    }
    if ((0, ast_1.isNumericLiteral)(value)) {
        if (Number.isInteger(value.value)) {
            return [PropertyValuetype.INTEGER, PropertyValuetype.DECIMAL];
        }
        return [PropertyValuetype.DECIMAL];
    }
    if ((0, ast_1.isBooleanLiteral)(value)) {
        return [PropertyValuetype.BOOLEAN];
    }
    if ((0, ast_1.isCellRangeLiteral)(value)) {
        return [PropertyValuetype.CELL_RANGE];
    }
    if ((0, ast_1.isRegexLiteral)(value)) {
        return [PropertyValuetype.REGEX];
    }
    if ((0, ast_1.isValuetypeAssignmentLiteral)(value)) {
        return [PropertyValuetype.VALUETYPE_ASSIGNMENT];
    }
    if ((0, ast_1.isCollectionLiteral)(value)) {
        return [PropertyValuetype.COLLECTION];
    }
    if ((0, ast_1.isConstraintReferenceLiteral)(value)) {
        return [PropertyValuetype.CONSTRAINT];
    }
    (0, langium_1.assertUnreachable)(value);
}
exports.inferTypesFromValue = inferTypesFromValue;
function getValuetypeName(valuetype) {
    if ((0, ast_1.isValuetypeDefinitionReference)(valuetype)) {
        return valuetype.reference.$refText;
    }
    return valuetype.keyword;
}
exports.getValuetypeName = getValuetypeName;


/***/ }),

/***/ "./libs/language-server/src/lib/ast/wrappers/ast-node-wrapper.ts":
/***/ ((__unused_webpack_module, exports) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));


/***/ }),

/***/ "./libs/language-server/src/lib/ast/wrappers/cell-range-wrapper.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.getCellIndex = exports.isCellWrapper = exports.getRowIndex = exports.isRowWrapper = exports.getColumnIndex = exports.isColumnWrapper = exports.CellRangeWrapper = exports.rowIndexToString = exports.columnIndexToString = exports.CellIndex = exports.LAST_INDEX = void 0;
const assert_1 = __webpack_require__("assert");
const ast_1 = __webpack_require__("./libs/language-server/src/lib/ast/generated/ast.ts");
const column_id_util_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/util/column-id-util.ts");
exports.LAST_INDEX = Number.POSITIVE_INFINITY;
class CellIndex {
    constructor(columnIndex, rowIndex) {
        this.columnIndex = columnIndex;
        this.rowIndex = rowIndex;
    }
    toString() {
        return `${columnIndexToString(this.columnIndex)}${rowIndexToString(this.rowIndex)}`;
    }
    hasRelativeIndexes() {
        return this.columnIndex === exports.LAST_INDEX || this.rowIndex === exports.LAST_INDEX;
    }
    resolveRelativeIndexes(bounds) {
        let columnIndex = this.columnIndex;
        if (columnIndex === exports.LAST_INDEX) {
            columnIndex = bounds.lastColumnIndex;
        }
        let rowIndex = this.rowIndex;
        if (rowIndex === exports.LAST_INDEX) {
            rowIndex = bounds.lastRowIndex;
        }
        return new CellIndex(columnIndex, rowIndex);
    }
    isInBounds(bounds) {
        const columnInBounds = this.isIndexInBounds(this.columnIndex, bounds.lastColumnIndex);
        const rowInBounds = this.isIndexInBounds(this.rowIndex, bounds.lastRowIndex);
        return columnInBounds && rowInBounds;
    }
    isIndexInBounds(index, max) {
        if (max < 0) {
            return false;
        }
        return index === exports.LAST_INDEX || index <= max;
    }
}
exports.CellIndex = CellIndex;
function columnIndexToString(columnIndex) {
    if (columnIndex === exports.LAST_INDEX) {
        return '*';
    }
    return (0, column_id_util_1.columnIndexAsCharacters)(columnIndex);
}
exports.columnIndexToString = columnIndexToString;
function rowIndexToString(rowIndex) {
    if (rowIndex === exports.LAST_INDEX) {
        return '*';
    }
    return `${rowIndex + 1}`;
}
exports.rowIndexToString = rowIndexToString;
class CellRangeWrapper {
    constructor(cellRange, indexes) {
        this.astNode = cellRange;
        if (indexes !== undefined) {
            this.from = indexes.from;
            this.to = indexes.to;
        }
        else if ((0, ast_1.isCellLiteral)(cellRange)) {
            const cellIndex = parseCellId(cellRange.cellId);
            this.from = cellIndex;
            this.to = cellIndex;
        }
        else if ((0, ast_1.isColumnLiteral)(cellRange)) {
            const columnIndex = parseColumnId(cellRange.columnId);
            this.from = new CellIndex(columnIndex, 0);
            this.to = new CellIndex(columnIndex, exports.LAST_INDEX);
        }
        else if ((0, ast_1.isRowLiteral)(cellRange)) {
            const rowIndex = parseRowId(cellRange.rowId);
            this.from = new CellIndex(0, rowIndex);
            this.to = new CellIndex(exports.LAST_INDEX, rowIndex);
        }
        else {
            this.from = parseCellId(cellRange.cellFrom);
            this.to = parseCellId(cellRange.cellTo);
        }
    }
    static canBeWrapped(cellRange) {
        if ((0, ast_1.isCellLiteral)(cellRange)) {
            // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
            return cellRange.cellId !== undefined;
        }
        else if ((0, ast_1.isColumnLiteral)(cellRange)) {
            // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
            return cellRange.columnId !== undefined;
        }
        else if ((0, ast_1.isRowLiteral)(cellRange)) {
            // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
            return cellRange.rowId !== undefined;
        }
        // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
        return cellRange.cellFrom !== undefined && cellRange.cellTo !== undefined;
    }
    isInBounds(bounds) {
        return this.from.isInBounds(bounds) && this.to.isInBounds(bounds);
    }
    resolveRelativeIndexes(bounds) {
        const boundFrom = this.from.resolveRelativeIndexes(bounds);
        const boundTo = this.to.resolveRelativeIndexes(bounds);
        return new CellRangeWrapper(this.astNode, {
            from: boundFrom,
            to: boundTo,
        });
    }
    hasRelativeIndexes() {
        return this.from.hasRelativeIndexes() || this.to.hasRelativeIndexes();
    }
    toString() {
        return `${this.from.toString()}:${this.to.toString()}`;
    }
}
exports.CellRangeWrapper = CellRangeWrapper;
function isColumnWrapper(cellRange) {
    if ((0, ast_1.isColumnLiteral)(cellRange.astNode)) {
        return true;
    }
    if ((0, ast_1.isRangeLiteral)(cellRange.astNode)) {
        return (cellRange.from.columnIndex === cellRange.to.columnIndex &&
            cellRange.from.rowIndex === 0 &&
            cellRange.to.rowIndex === exports.LAST_INDEX);
    }
    return false;
}
exports.isColumnWrapper = isColumnWrapper;
function getColumnIndex(column) {
    (0, assert_1.strict)(isColumnWrapper(column));
    return column.from.columnIndex;
}
exports.getColumnIndex = getColumnIndex;
function isRowWrapper(cellRange) {
    if ((0, ast_1.isRowLiteral)(cellRange.astNode)) {
        return true;
    }
    if ((0, ast_1.isRangeLiteral)(cellRange.astNode)) {
        return (cellRange.from.rowIndex === cellRange.to.rowIndex &&
            cellRange.from.columnIndex === 0 &&
            cellRange.to.columnIndex === exports.LAST_INDEX);
    }
    return false;
}
exports.isRowWrapper = isRowWrapper;
function getRowIndex(row) {
    (0, assert_1.strict)(isRowWrapper(row));
    return row.from.rowIndex;
}
exports.getRowIndex = getRowIndex;
function isCellWrapper(cellRange) {
    if ((0, ast_1.isCellLiteral)(cellRange.astNode)) {
        return true;
    }
    if ((0, ast_1.isRangeLiteral)(cellRange.astNode)) {
        return (cellRange.from.columnIndex === cellRange.to.columnIndex &&
            cellRange.from.rowIndex === cellRange.to.rowIndex);
    }
    return false;
}
exports.isCellWrapper = isCellWrapper;
function getCellIndex(cell) {
    (0, assert_1.strict)(isCellWrapper(cell));
    return cell.from;
}
exports.getCellIndex = getCellIndex;
const CELL_INDEX_REGEX = /([A-Z]+|\*)([0-9]+|\*)/;
function parseCellId(cellId) {
    const matches = CELL_INDEX_REGEX.exec(cellId) || [];
    const columnId = matches[1];
    const rowId = matches[2];
    (0, assert_1.strict)(columnId !== undefined && rowId !== undefined, `Cell IDs are expected to match the regular expression ${CELL_INDEX_REGEX.toString()}`);
    const columnIndex = parseColumnId(columnId);
    const rowIndex = parseRowId(rowId);
    return new CellIndex(columnIndex, rowIndex);
}
function parseColumnId(columnId) {
    if (columnId === '*') {
        return exports.LAST_INDEX;
    }
    return (0, column_id_util_1.columnCharactersAsIndex)(columnId);
}
function parseRowId(rowId) {
    if (typeof rowId === 'string') {
        if (rowId === '*') {
            return exports.LAST_INDEX;
        }
        return Number.parseInt(rowId, 10) - 1;
    }
    return rowId - 1;
}


/***/ }),

/***/ "./libs/language-server/src/lib/ast/wrappers/index.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
const tslib_1 = __webpack_require__("tslib");
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/ast/wrappers/util/index.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/ast/wrappers/ast-node-wrapper.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/ast/wrappers/cell-range-wrapper.ts"), exports);


/***/ }),

/***/ "./libs/language-server/src/lib/ast/wrappers/pipe-wrapper.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.createSemanticPipes = exports.PipeWrapper = void 0;
const assert_1 = __webpack_require__("assert");
const ast_1 = __webpack_require__("./libs/language-server/src/lib/ast/generated/ast.ts");
class PipeWrapper {
    constructor(pipe, chainIndex) {
        var _a, _b, _c, _d;
        this.astNode = pipe;
        if ((0, ast_1.isSinglePipeDefinition)(pipe)) {
            // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
            (0, assert_1.strict)(((_a = pipe.from) === null || _a === void 0 ? void 0 : _a.ref) !== undefined);
            // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
            (0, assert_1.strict)(((_b = pipe.to) === null || _b === void 0 ? void 0 : _b.ref) !== undefined);
            this.from = pipe.from.ref;
            this.to = pipe.to.ref;
        }
        else {
            (0, assert_1.strict)(chainIndex !== undefined);
            (0, assert_1.strict)(0 <= chainIndex && chainIndex + 1 < pipe.blocks.length);
            (0, assert_1.strict)(((_c = pipe.blocks[chainIndex]) === null || _c === void 0 ? void 0 : _c.ref) !== undefined);
            (0, assert_1.strict)(((_d = pipe.blocks[chainIndex + 1]) === null || _d === void 0 ? void 0 : _d.ref) !== undefined);
            // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
            this.from = pipe.blocks[chainIndex].ref;
            // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
            this.to = pipe.blocks[chainIndex + 1].ref;
            this.chainIndex = chainIndex;
        }
    }
    getFromDiagnostic() {
        if ((0, ast_1.isSinglePipeDefinition)(this.astNode)) {
            const result = {
                node: this.astNode,
                property: 'from',
            };
            return result;
        }
        (0, assert_1.strict)(this.chainIndex !== undefined);
        const result = {
            node: this.astNode,
            property: 'blocks',
            index: this.chainIndex,
        };
        return result;
    }
    getToDiagnostic() {
        if ((0, ast_1.isSinglePipeDefinition)(this.astNode)) {
            const result = {
                node: this.astNode,
                property: 'to',
            };
            return result;
        }
        (0, assert_1.strict)(this.chainIndex !== undefined);
        const result = {
            node: this.astNode,
            property: 'blocks',
            index: this.chainIndex + 1,
        };
        return result;
    }
    equals(pipe) {
        return this.from === pipe.from && this.to === pipe.to;
    }
    static canBeWrapped(pipe, chainIndex) {
        var _a, _b, _c, _d;
        if ((0, ast_1.isSinglePipeDefinition)(pipe)) {
            // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
            return ((_a = pipe.from) === null || _a === void 0 ? void 0 : _a.ref) !== undefined && ((_b = pipe.to) === null || _b === void 0 ? void 0 : _b.ref) !== undefined;
        }
        return (chainIndex !== undefined &&
            0 <= chainIndex &&
            chainIndex + 1 < pipe.blocks.length &&
            ((_c = pipe.blocks[chainIndex]) === null || _c === void 0 ? void 0 : _c.ref) !== undefined &&
            ((_d = pipe.blocks[chainIndex + 1]) === null || _d === void 0 ? void 0 : _d.ref) !== undefined);
    }
}
exports.PipeWrapper = PipeWrapper;
function createSemanticPipes(pipe) {
    if ((0, ast_1.isSinglePipeDefinition)(pipe)) {
        return createFromSinglePipe(pipe);
    }
    return createFromChainedPipe(pipe);
}
exports.createSemanticPipes = createSemanticPipes;
function createFromSinglePipe(pipe) {
    if (PipeWrapper.canBeWrapped(pipe)) {
        return [new PipeWrapper(pipe)];
    }
    return [];
}
function createFromChainedPipe(pipe) {
    const result = [];
    for (let chainIndex = 0; chainIndex < pipe.blocks.length - 1; ++chainIndex) {
        if (!PipeWrapper.canBeWrapped(pipe, chainIndex)) {
            continue;
        }
        const semanticPipe = new PipeWrapper(pipe, chainIndex);
        result.push(semanticPipe);
    }
    return result;
}


/***/ }),

/***/ "./libs/language-server/src/lib/ast/wrappers/util/column-id-util.ts":
/***/ ((__unused_webpack_module, exports) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.columnCharactersAsIndex = exports.columnIndexAsCharacters = void 0;
const firstCharacter = 'A';
const lastCharacter = 'Z';
const characterOffset = firstCharacter.charCodeAt(0);
const numberOfCharacters = lastCharacter.charCodeAt(0) - characterOffset + 1;
function columnIndexAsCharacters(columnIndex) {
    let columnCharacters = '';
    let value = columnIndex + 1;
    do {
        const remainder = value % numberOfCharacters;
        let quotient = Math.floor(value / numberOfCharacters);
        if (remainder !== 0) {
            columnCharacters =
                String.fromCharCode(remainder + characterOffset - 1) + columnCharacters;
        }
        else {
            --quotient;
            columnCharacters = lastCharacter + columnCharacters;
        }
        value = quotient;
    } while (value > 0);
    return columnCharacters;
}
exports.columnIndexAsCharacters = columnIndexAsCharacters;
function columnCharactersAsIndex(columnCharacters) {
    let columnIndex = 0;
    for (let position = 0; position < columnCharacters.length; ++position) {
        const charCode = columnCharacters.charCodeAt(position);
        const characterIndex = charCode - characterOffset + 1;
        const factor = Math.pow(numberOfCharacters, columnCharacters.length - 1 - position);
        columnIndex += characterIndex * factor;
    }
    return columnIndex - 1;
}
exports.columnCharactersAsIndex = columnCharactersAsIndex;


/***/ }),

/***/ "./libs/language-server/src/lib/ast/wrappers/util/index.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
const tslib_1 = __webpack_require__("tslib");
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/ast/wrappers/util/column-id-util.ts"), exports);


/***/ }),

/***/ "./libs/language-server/src/lib/completion/jayvee-completion-provider.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.JayveeCompletionProvider = void 0;
const assert_1 = __webpack_require__("assert");
const langium_1 = __webpack_require__("langium");
const vscode_languageserver_1 = __webpack_require__("vscode-languageserver");
const ast_1 = __webpack_require__("./libs/language-server/src/lib/ast/generated/ast.ts");
const lsp_doc_generator_1 = __webpack_require__("./libs/language-server/src/lib/docs/lsp-doc-generator.ts");
const meta_inf_registry_1 = __webpack_require__("./libs/language-server/src/lib/meta-information/meta-inf-registry.ts");
const RIGHT_ARROW_SYMBOL = '\u{2192}';
class JayveeCompletionProvider extends langium_1.DefaultCompletionProvider {
    completionFor(context, next, acceptor) {
        const astNode = context.node;
        if (astNode !== undefined) {
            const isBlockTypeCompletion = ((0, ast_1.isBlockDefinition)(astNode) || (0, ast_1.isBlockTypeLiteral)(astNode)) &&
                next.type === ast_1.BlockTypeLiteral;
            if (isBlockTypeCompletion) {
                return this.completionForBlockType(acceptor);
            }
            const isConstraintTypeCompletion = ((0, ast_1.isConstraintDefinition)(astNode) || (0, ast_1.isConstraintTypeLiteral)(astNode)) &&
                next.type === ast_1.ConstraintTypeLiteral;
            if (isConstraintTypeCompletion) {
                return this.completionForConstraintType(acceptor);
            }
            const isFirstPropertyCompletion = (0, ast_1.isPropertyBody)(astNode) && next.type === ast_1.PropertyAssignment;
            const isOtherPropertyCompletion = (0, ast_1.isPropertyAssignment)(astNode) && next.type === ast_1.PropertyAssignment;
            if (isFirstPropertyCompletion || isOtherPropertyCompletion) {
                return this.completionForPropertyName(astNode, acceptor);
            }
        }
        return super.completionFor(context, next, acceptor);
    }
    completionForBlockType(acceptor) {
        (0, meta_inf_registry_1.getRegisteredBlockMetaInformation)().forEach((metaInf) => {
            const lspDocBuilder = new lsp_doc_generator_1.LspDocGenerator();
            const markdownDoc = lspDocBuilder.generateBlockTypeDoc(metaInf);
            acceptor({
                label: metaInf.type,
                labelDetails: {
                    detail: ` ${metaInf.inputType} ${RIGHT_ARROW_SYMBOL} ${metaInf.outputType}`,
                },
                kind: vscode_languageserver_1.CompletionItemKind.Class,
                detail: `(block type)`,
                documentation: {
                    kind: 'markdown',
                    value: markdownDoc,
                },
            });
        });
    }
    completionForConstraintType(acceptor) {
        (0, meta_inf_registry_1.getRegisteredConstraintMetaInformation)().forEach((metaInf) => {
            acceptor({
                label: metaInf.type,
                labelDetails: {
                    detail: ` ${metaInf.compatiblePrimitiveValuetypes.join(',')}`,
                },
                kind: vscode_languageserver_1.CompletionItemKind.Class,
                detail: `(constraint type)`,
            });
        });
    }
    completionForPropertyName(astNode, acceptor) {
        let container;
        if ((0, ast_1.isPropertyBody)(astNode)) {
            container = astNode.$container;
        }
        else {
            container = astNode.$container.$container;
        }
        const metaInf = (0, meta_inf_registry_1.getMetaInformation)(container.type);
        if (metaInf === undefined) {
            return;
        }
        const presentPropertyNames = container.body.properties.map((attr) => attr.name);
        const propertyKinds = [
            'required',
            'optional',
        ];
        for (const propertyKind of propertyKinds) {
            const propertyNames = metaInf.getPropertyNames(propertyKind, presentPropertyNames);
            this.constructPropertyCompletionValueItems(metaInf, propertyNames, propertyKind).forEach(acceptor);
        }
    }
    constructPropertyCompletionValueItems(metaInf, propertyNames, kind) {
        return propertyNames.map((propertyName) => {
            const propertySpec = metaInf.getPropertySpecification(propertyName);
            (0, assert_1.strict)(propertySpec !== undefined);
            const completionValueItem = {
                label: propertyName,
                labelDetails: {
                    detail: ` ${propertySpec.type}`,
                },
                kind: vscode_languageserver_1.CompletionItemKind.Field,
                detail: `(${kind} property)`,
                sortText: kind === 'required' ? '1' : '2',
            };
            if (propertySpec.defaultValue !== undefined) {
                const defaultValueString = JSON.stringify(propertySpec.defaultValue);
                // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                completionValueItem.labelDetails.detail += ` = ${defaultValueString}`;
            }
            const lspDocBuilder = new lsp_doc_generator_1.LspDocGenerator();
            const markdownDoc = lspDocBuilder.generatePropertyDoc(metaInf, propertyName);
            if (markdownDoc !== undefined) {
                completionValueItem.documentation = {
                    kind: 'markdown',
                    value: markdownDoc,
                };
            }
            return completionValueItem;
        });
    }
}
exports.JayveeCompletionProvider = JayveeCompletionProvider;


/***/ }),

/***/ "./libs/language-server/src/lib/constraint/allowlist-constraint-meta-inf.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.AllowlistConstraintMetaInformation = void 0;
const collection_util_1 = __webpack_require__("./libs/language-server/src/lib/ast/collection-util.ts");
const ast_1 = __webpack_require__("./libs/language-server/src/lib/ast/generated/ast.ts");
const model_util_1 = __webpack_require__("./libs/language-server/src/lib/ast/model-util.ts");
const constraint_meta_inf_1 = __webpack_require__("./libs/language-server/src/lib/meta-information/constraint-meta-inf.ts");
class AllowlistConstraintMetaInformation extends constraint_meta_inf_1.ConstraintMetaInformation {
    constructor() {
        super('AllowlistConstraint', {
            allowlist: {
                type: model_util_1.PropertyValuetype.COLLECTION,
                validation: (property, accept) => {
                    const propertyValue = property.value;
                    if (!(0, ast_1.isCollectionLiteral)(propertyValue)) {
                        return;
                    }
                    const { invalidItems } = (0, collection_util_1.validateTypedCollection)(propertyValue, ast_1.isTextLiteral);
                    invalidItems.forEach((invalidValue) => accept('error', 'Only text values are allowed in this collection', {
                        node: invalidValue,
                    }));
                },
            },
        }, ['text']);
    }
}
exports.AllowlistConstraintMetaInformation = AllowlistConstraintMetaInformation;


/***/ }),

/***/ "./libs/language-server/src/lib/constraint/constraint-registry.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.registerConstraints = void 0;
const meta_inf_registry_1 = __webpack_require__("./libs/language-server/src/lib/meta-information/meta-inf-registry.ts");
const allowlist_constraint_meta_inf_1 = __webpack_require__("./libs/language-server/src/lib/constraint/allowlist-constraint-meta-inf.ts");
const denylist_constraint_meta_inf_1 = __webpack_require__("./libs/language-server/src/lib/constraint/denylist-constraint-meta-inf.ts");
const length_constraint_meta_inf_1 = __webpack_require__("./libs/language-server/src/lib/constraint/length-constraint-meta-inf.ts");
const range_constraint_meta_inf_1 = __webpack_require__("./libs/language-server/src/lib/constraint/range-constraint-meta-inf.ts");
const regex_constraint_meta_inf_1 = __webpack_require__("./libs/language-server/src/lib/constraint/regex-constraint-meta-inf.ts");
function registerConstraints() {
    (0, meta_inf_registry_1.registerMetaInformation)(allowlist_constraint_meta_inf_1.AllowlistConstraintMetaInformation);
    (0, meta_inf_registry_1.registerMetaInformation)(denylist_constraint_meta_inf_1.DenylistConstraintMetaInformation);
    (0, meta_inf_registry_1.registerMetaInformation)(regex_constraint_meta_inf_1.RegexConstraintMetaInformation);
    (0, meta_inf_registry_1.registerMetaInformation)(length_constraint_meta_inf_1.LengthConstraintMetaInformation);
    (0, meta_inf_registry_1.registerMetaInformation)(range_constraint_meta_inf_1.RangeConstraintMetaInformation);
}
exports.registerConstraints = registerConstraints;


/***/ }),

/***/ "./libs/language-server/src/lib/constraint/denylist-constraint-meta-inf.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.DenylistConstraintMetaInformation = void 0;
const collection_util_1 = __webpack_require__("./libs/language-server/src/lib/ast/collection-util.ts");
const ast_1 = __webpack_require__("./libs/language-server/src/lib/ast/generated/ast.ts");
const model_util_1 = __webpack_require__("./libs/language-server/src/lib/ast/model-util.ts");
const constraint_meta_inf_1 = __webpack_require__("./libs/language-server/src/lib/meta-information/constraint-meta-inf.ts");
class DenylistConstraintMetaInformation extends constraint_meta_inf_1.ConstraintMetaInformation {
    constructor() {
        super('DenylistConstraint', {
            denylist: {
                type: model_util_1.PropertyValuetype.COLLECTION,
                validation: (property, accept) => {
                    const propertyValue = property.value;
                    if (!(0, ast_1.isCollectionLiteral)(propertyValue)) {
                        return;
                    }
                    const { invalidItems } = (0, collection_util_1.validateTypedCollection)(propertyValue, ast_1.isTextLiteral);
                    invalidItems.forEach((invalidValue) => accept('error', 'Only text values are allowed in this collection', {
                        node: invalidValue,
                    }));
                },
            },
        }, ['text']);
    }
}
exports.DenylistConstraintMetaInformation = DenylistConstraintMetaInformation;


/***/ }),

/***/ "./libs/language-server/src/lib/constraint/index.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
const tslib_1 = __webpack_require__("tslib");
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/constraint/denylist-constraint-meta-inf.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/constraint/constraint-registry.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/constraint/length-constraint-meta-inf.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/constraint/range-constraint-meta-inf.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/constraint/regex-constraint-meta-inf.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/constraint/allowlist-constraint-meta-inf.ts"), exports);


/***/ }),

/***/ "./libs/language-server/src/lib/constraint/length-constraint-meta-inf.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.LengthConstraintMetaInformation = void 0;
const assert_1 = __webpack_require__("assert");
const ast_1 = __webpack_require__("./libs/language-server/src/lib/ast/index.ts");
const model_util_1 = __webpack_require__("./libs/language-server/src/lib/ast/model-util.ts");
const constraint_meta_inf_1 = __webpack_require__("./libs/language-server/src/lib/meta-information/constraint-meta-inf.ts");
class LengthConstraintMetaInformation extends constraint_meta_inf_1.ConstraintMetaInformation {
    constructor() {
        super('LengthConstraint', {
            minLength: {
                type: model_util_1.PropertyValuetype.INTEGER,
                defaultValue: 0,
                validation: nonNegativeValidation,
            },
            maxLength: {
                type: model_util_1.PropertyValuetype.INTEGER,
                defaultValue: Number.POSITIVE_INFINITY,
                validation: nonNegativeValidation,
            },
        }, ['text'], (propertyBody, accept) => {
            const minLengthProperty = propertyBody.properties.find((p) => p.name === 'minLength');
            const maxLengthProperty = propertyBody.properties.find((p) => p.name === 'maxLength');
            if (minLengthProperty === undefined ||
                maxLengthProperty === undefined) {
                return;
            }
            (0, assert_1.strict)((0, ast_1.isNumericLiteral)(minLengthProperty.value));
            (0, assert_1.strict)((0, ast_1.isNumericLiteral)(maxLengthProperty.value));
            const minLength = minLengthProperty.value.value;
            const maxLength = maxLengthProperty.value.value;
            if (minLength > maxLength) {
                [minLengthProperty, maxLengthProperty].forEach((property) => {
                    accept('error', 'The minimum length needs to be smaller or equal to the maximum length', { node: property.value });
                });
            }
        });
    }
}
exports.LengthConstraintMetaInformation = LengthConstraintMetaInformation;
function nonNegativeValidation(property, accept) {
    const propertyValue = property.value;
    (0, assert_1.strict)((0, ast_1.isNumericLiteral)(propertyValue));
    if (propertyValue.value < 0) {
        accept('error', `Bounds for length need to be equal or greater than zero`, {
            node: propertyValue,
        });
    }
}


/***/ }),

/***/ "./libs/language-server/src/lib/constraint/range-constraint-meta-inf.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.RangeConstraintMetaInformation = void 0;
const assert_1 = __webpack_require__("assert");
const ast_1 = __webpack_require__("./libs/language-server/src/lib/ast/index.ts");
const model_util_1 = __webpack_require__("./libs/language-server/src/lib/ast/model-util.ts");
const constraint_meta_inf_1 = __webpack_require__("./libs/language-server/src/lib/meta-information/constraint-meta-inf.ts");
class RangeConstraintMetaInformation extends constraint_meta_inf_1.ConstraintMetaInformation {
    constructor() {
        super('RangeConstraint', {
            lowerBound: {
                type: model_util_1.PropertyValuetype.DECIMAL,
                defaultValue: Number.NEGATIVE_INFINITY,
            },
            lowerBoundInclusive: {
                type: model_util_1.PropertyValuetype.BOOLEAN,
                defaultValue: true,
            },
            upperBound: {
                type: model_util_1.PropertyValuetype.DECIMAL,
                defaultValue: Number.POSITIVE_INFINITY,
            },
            upperBoundInclusive: {
                type: model_util_1.PropertyValuetype.BOOLEAN,
                defaultValue: true,
            },
        }, ['integer', 'decimal'], (propertyBody, accept) => {
            const lowerBoundProperty = propertyBody.properties.find((p) => p.name === 'lowerBound');
            const upperBoundProperty = propertyBody.properties.find((p) => p.name === 'upperBound');
            if (lowerBoundProperty === undefined ||
                upperBoundProperty === undefined) {
                return;
            }
            (0, assert_1.strict)((0, ast_1.isNumericLiteral)(lowerBoundProperty.value));
            (0, assert_1.strict)((0, ast_1.isNumericLiteral)(upperBoundProperty.value));
            const lowerBound = lowerBoundProperty.value.value;
            const upperBound = upperBoundProperty.value.value;
            if (lowerBound > upperBound) {
                [lowerBoundProperty, upperBoundProperty].forEach((property) => {
                    accept('error', 'The lower bound needs to be smaller or equal to the upper bound', { node: property.value });
                });
            }
            else if (lowerBound === upperBound) {
                const lowerBoundInclusiveProperty = propertyBody.properties.find((p) => p.name === 'lowerBoundInclusive');
                let lowerBoundInclusive = true;
                if (lowerBoundInclusiveProperty !== undefined) {
                    (0, assert_1.strict)((0, ast_1.isBooleanLiteral)(lowerBoundInclusiveProperty.value));
                    lowerBoundInclusive = lowerBoundInclusiveProperty.value.value;
                }
                const upperBoundInclusiveProperty = propertyBody.properties.find((p) => p.name === 'upperBoundInclusive');
                let upperBoundInclusive = true;
                if (upperBoundInclusiveProperty !== undefined) {
                    (0, assert_1.strict)((0, ast_1.isBooleanLiteral)(upperBoundInclusiveProperty.value));
                    upperBoundInclusive = upperBoundInclusiveProperty.value.value;
                }
                const errorMessage = 'Lower and upper bounds need to be inclusive if they are identical';
                if (!lowerBoundInclusive) {
                    accept('error', errorMessage, {
                        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                        node: lowerBoundInclusiveProperty.value,
                    });
                }
                if (!upperBoundInclusive) {
                    accept('error', errorMessage, {
                        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                        node: upperBoundInclusiveProperty.value,
                    });
                }
            }
        });
    }
}
exports.RangeConstraintMetaInformation = RangeConstraintMetaInformation;


/***/ }),

/***/ "./libs/language-server/src/lib/constraint/regex-constraint-meta-inf.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.RegexConstraintMetaInformation = void 0;
const model_util_1 = __webpack_require__("./libs/language-server/src/lib/ast/model-util.ts");
const constraint_meta_inf_1 = __webpack_require__("./libs/language-server/src/lib/meta-information/constraint-meta-inf.ts");
class RegexConstraintMetaInformation extends constraint_meta_inf_1.ConstraintMetaInformation {
    constructor() {
        super('RegexConstraint', {
            regex: {
                type: model_util_1.PropertyValuetype.REGEX,
            },
        }, ['text']);
    }
}
exports.RegexConstraintMetaInformation = RegexConstraintMetaInformation;


/***/ }),

/***/ "./libs/language-server/src/lib/docs/index.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
const tslib_1 = __webpack_require__("tslib");
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/docs/jayvee-doc-generator.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/docs/lsp-doc-generator.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/docs/markdown-builder.ts"), exports);


/***/ }),

/***/ "./libs/language-server/src/lib/docs/jayvee-doc-generator.ts":
/***/ ((__unused_webpack_module, exports) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));


/***/ }),

/***/ "./libs/language-server/src/lib/docs/lsp-doc-generator.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.LspDocGenerator = void 0;
const markdown_builder_1 = __webpack_require__("./libs/language-server/src/lib/docs/markdown-builder.ts");
class LspDocGenerator {
    generateBlockTypeDoc(metaInf) {
        const markdownBuilder = new markdown_builder_1.MarkdownBuilder();
        return markdownBuilder.line(metaInf.docs.description).build();
    }
    generatePropertyDoc(metaInf, propertyName) {
        var _a, _b;
        const markdownBuilder = new markdown_builder_1.MarkdownBuilder();
        const propertySpec = metaInf.getPropertySpecification(propertyName);
        if (propertySpec === undefined) {
            return undefined;
        }
        return markdownBuilder
            .line((_a = propertySpec.docs) === null || _a === void 0 ? void 0 : _a.description)
            .newLine()
            .line((_b = propertySpec.docs) === null || _b === void 0 ? void 0 : _b.validation)
            .build();
    }
}
exports.LspDocGenerator = LspDocGenerator;


/***/ }),

/***/ "./libs/language-server/src/lib/docs/markdown-builder.ts":
/***/ ((__unused_webpack_module, exports) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.MarkdownBuilder = void 0;
class MarkdownBuilder {
    constructor() {
        this.markdownTextLines = [];
    }
    getHeaderPrefix(depth) {
        return '#'.repeat(depth);
    }
    heading(title, depth) {
        this.markdownTextLines.push(`${this.getHeaderPrefix(depth)} ${title}`);
        this.newLine();
        return this;
    }
    line(text) {
        if (text !== undefined) {
            this.markdownTextLines.push(text);
        }
        return this;
    }
    code(code, language) {
        return this.line('```' + (language !== null && language !== void 0 ? language : ''))
            .line(code)
            .line('```')
            .newLine();
    }
    newLine() {
        this.markdownTextLines.push('');
        return this;
    }
    comment(comment) {
        this.markdownTextLines.push(`<!-- ${comment} -->`);
        return this;
    }
    build() {
        return this.markdownTextLines.join('\n');
    }
}
exports.MarkdownBuilder = MarkdownBuilder;


/***/ }),

/***/ "./libs/language-server/src/lib/extension.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.useExtension = void 0;
const meta_inf_registry_1 = __webpack_require__("./libs/language-server/src/lib/meta-information/meta-inf-registry.ts");
function useExtension(extension) {
    extension.getBlockMetaInf().forEach(meta_inf_registry_1.registerMetaInformation);
}
exports.useExtension = useExtension;


/***/ }),

/***/ "./libs/language-server/src/lib/hover/jayvee-hover-provider.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.JayveeHoverProvider = void 0;
const langium_1 = __webpack_require__("langium");
const ast_1 = __webpack_require__("./libs/language-server/src/lib/ast/index.ts");
const lsp_doc_generator_1 = __webpack_require__("./libs/language-server/src/lib/docs/lsp-doc-generator.ts");
const meta_information_1 = __webpack_require__("./libs/language-server/src/lib/meta-information/index.ts");
class JayveeHoverProvider extends langium_1.AstNodeHoverProvider {
    getAstNodeHoverContent(astNode) {
        let doc = undefined;
        if ((0, ast_1.isBlockTypeLiteral)(astNode)) {
            doc = this.getBlockTypeMarkdownDoc(astNode);
        }
        if ((0, ast_1.isPropertyAssignment)(astNode)) {
            doc = this.getPropertyMarkdownDoc(astNode);
        }
        if (doc === undefined) {
            return undefined;
        }
        const hover = {
            contents: {
                kind: 'markdown',
                value: doc,
            },
        };
        return hover;
    }
    getBlockTypeMarkdownDoc(blockType) {
        const blockMetaInf = (0, meta_information_1.getMetaInformation)(blockType);
        if (blockMetaInf === undefined) {
            return;
        }
        const lspDocBuilder = new lsp_doc_generator_1.LspDocGenerator();
        return lspDocBuilder.generateBlockTypeDoc(blockMetaInf);
    }
    getPropertyMarkdownDoc(property) {
        const block = property.$container.$container;
        const metaInf = (0, meta_information_1.getMetaInformation)(block.type);
        if (metaInf === undefined) {
            return;
        }
        const lspDocBuilder = new lsp_doc_generator_1.LspDocGenerator();
        return lspDocBuilder.generatePropertyDoc(metaInf, property.name);
    }
}
exports.JayveeHoverProvider = JayveeHoverProvider;


/***/ }),

/***/ "./libs/language-server/src/lib/index.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
const tslib_1 = __webpack_require__("tslib");
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/ast/index.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/constraint/index.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/docs/index.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/meta-information/index.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/util/index.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/validation/index.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/jayvee-module.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/extension.ts"), exports);


/***/ }),

/***/ "./libs/language-server/src/lib/jayvee-module.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.createJayveeServices = exports.JayveeModule = void 0;
const langium_1 = __webpack_require__("langium");
const module_1 = __webpack_require__("./libs/language-server/src/lib/ast/generated/module.ts");
const jayvee_completion_provider_1 = __webpack_require__("./libs/language-server/src/lib/completion/jayvee-completion-provider.ts");
const constraint_registry_1 = __webpack_require__("./libs/language-server/src/lib/constraint/constraint-registry.ts");
const jayvee_hover_provider_1 = __webpack_require__("./libs/language-server/src/lib/hover/jayvee-hover-provider.ts");
const jayvee_value_converter_1 = __webpack_require__("./libs/language-server/src/lib/jayvee-value-converter.ts");
const validation_registry_1 = __webpack_require__("./libs/language-server/src/lib/validation/validation-registry.ts");
const block_validator_1 = __webpack_require__("./libs/language-server/src/lib/validation/validators/block-validator.ts");
const cell_range_selection_validator_1 = __webpack_require__("./libs/language-server/src/lib/validation/validators/cell-range-selection-validator.ts");
const column_expression_validator_1 = __webpack_require__("./libs/language-server/src/lib/validation/validators/column-expression-validator.ts");
const constraint_validator_1 = __webpack_require__("./libs/language-server/src/lib/validation/validators/constraint-validator.ts");
const model_validator_1 = __webpack_require__("./libs/language-server/src/lib/validation/validators/model-validator.ts");
const pipe_validator_1 = __webpack_require__("./libs/language-server/src/lib/validation/validators/pipe-validator.ts");
const pipeline_validator_1 = __webpack_require__("./libs/language-server/src/lib/validation/validators/pipeline-validator.ts");
const property_body_validator_1 = __webpack_require__("./libs/language-server/src/lib/validation/validators/property-body-validator.ts");
const regex_value_validator_1 = __webpack_require__("./libs/language-server/src/lib/validation/validators/regex-value-validator.ts");
const valuetype_validator_1 = __webpack_require__("./libs/language-server/src/lib/validation/validators/valuetype-validator.ts");
/**
 * Dependency injection module that overrides Langium default services and contributes the
 * declared custom services. The Langium defaults can be partially specified to override only
 * selected services, while the custom services must be fully specified.
 */
exports.JayveeModule = {
    parser: {
        ValueConverter: () => new jayvee_value_converter_1.JayveeValueConverter(),
    },
    validation: {
        ValidationRegistry: (services) => new validation_registry_1.JayveeValidationRegistry(services),
        JayveeModelValidator: () => new model_validator_1.JayveeModelValidator(),
        PipelineDefinitionValidator: () => new pipeline_validator_1.PipelineDefinitionValidator(),
        PipeValidator: () => new pipe_validator_1.PipeValidator(),
        BlockValidator: () => new block_validator_1.BlockValidator(),
        PropertyBodyValidator: () => new property_body_validator_1.PropertyBodyValidator(),
        CellRangeSelectionValidator: () => new cell_range_selection_validator_1.CellRangeSelectionValidator(),
        ColumnExpressionValidator: () => new column_expression_validator_1.ColumnExpressionValidator(),
        ConstraintValidator: () => new constraint_validator_1.ConstraintValidator(),
        RegexValueValidator: () => new regex_value_validator_1.RegexValueValidator(),
        ValuetypeValidator: () => new valuetype_validator_1.ValuetypeValidator(),
    },
    lsp: {
        CompletionProvider: (services) => new jayvee_completion_provider_1.JayveeCompletionProvider(services),
        HoverProvider: (services) => new jayvee_hover_provider_1.JayveeHoverProvider(services),
    },
};
/**
 * Create the full set of services required by Langium.
 *
 * First inject the shared services by merging two modules:
 *  - Langium default shared services
 *  - Services generated by langium-cli
 *
 * Then inject the language-specific services by merging three modules:
 *  - Langium default language-specific services
 *  - Services generated by langium-cli
 *  - Services specified in this file
 *
 * @param context Optional module context with the LSP connection
 * @returns An object wrapping the shared services and the language-specific services
 */
function createJayveeServices(context) {
    const shared = (0, langium_1.inject)((0, langium_1.createDefaultSharedModule)(context), module_1.JayveeGeneratedSharedModule);
    const Jayvee = (0, langium_1.inject)((0, langium_1.createDefaultModule)({ shared }), module_1.JayveeGeneratedModule, exports.JayveeModule);
    shared.ServiceRegistry.register(Jayvee);
    (0, constraint_registry_1.registerConstraints)();
    return { shared, Jayvee };
}
exports.createJayveeServices = createJayveeServices;


/***/ }),

/***/ "./libs/language-server/src/lib/jayvee-value-converter.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.JayveeValueConverter = void 0;
const langium_1 = __webpack_require__("langium");
class JayveeValueConverter extends langium_1.DefaultValueConverter {
    runConverter(rule, input, cstNode) {
        if (rule.name === 'REGEX') {
            // Trim leading and trailing '/' character:
            return input.substring(1, input.length - 1);
        }
        return super.runConverter(rule, input, cstNode);
    }
}
exports.JayveeValueConverter = JayveeValueConverter;


/***/ }),

/***/ "./libs/language-server/src/lib/meta-information/block-meta-inf.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.BlockMetaInformation = void 0;
// eslint-disable-next-line import/no-cycle
const model_util_1 = __webpack_require__("./libs/language-server/src/lib/ast/model-util.ts");
// eslint-disable-next-line import/no-cycle
const meta_inf_1 = __webpack_require__("./libs/language-server/src/lib/meta-information/meta-inf.ts");
class BlockMetaInformation extends meta_inf_1.MetaInformation {
    constructor(blockType, properties, inputType, outputType, validation) {
        super(blockType, properties, validation);
        this.inputType = inputType;
        this.outputType = outputType;
        this.docs = {};
    }
    canBeConnectedTo(blockAfter) {
        return this.outputType === blockAfter.inputType;
    }
    hasInput() {
        return this.inputType !== model_util_1.IOType.NONE;
    }
    hasOutput() {
        return this.outputType !== model_util_1.IOType.NONE;
    }
}
exports.BlockMetaInformation = BlockMetaInformation;


/***/ }),

/***/ "./libs/language-server/src/lib/meta-information/constraint-meta-inf.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.ConstraintMetaInformation = void 0;
// eslint-disable-next-line import/no-cycle
const meta_inf_1 = __webpack_require__("./libs/language-server/src/lib/meta-information/meta-inf.ts");
class ConstraintMetaInformation extends meta_inf_1.MetaInformation {
    constructor(constraintType, properties, compatiblePrimitiveValuetypes, validation) {
        super(constraintType, properties, validation);
        this.compatiblePrimitiveValuetypes = compatiblePrimitiveValuetypes;
    }
}
exports.ConstraintMetaInformation = ConstraintMetaInformation;


/***/ }),

/***/ "./libs/language-server/src/lib/meta-information/index.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
const tslib_1 = __webpack_require__("tslib");
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/meta-information/block-meta-inf.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/meta-information/meta-inf.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/meta-information/meta-inf-registry.ts"), exports);


/***/ }),

/***/ "./libs/language-server/src/lib/meta-information/meta-inf-registry.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.getOrFailMetaInformation = exports.getRegisteredConstraintMetaInformation = exports.getRegisteredBlockMetaInformation = exports.getMetaInformation = exports.registerMetaInformation = exports.metaInformationRegistry = void 0;
const assert_1 = __webpack_require__("assert");
const errors_1 = __webpack_require__("langium/lib/utils/errors");
const ast_1 = __webpack_require__("./libs/language-server/src/lib/ast/generated/ast.ts");
const registry_1 = __webpack_require__("./libs/language-server/src/lib/util/registry.ts");
// eslint-disable-next-line import/no-cycle
const block_meta_inf_1 = __webpack_require__("./libs/language-server/src/lib/meta-information/block-meta-inf.ts");
// eslint-disable-next-line import/no-cycle
const constraint_meta_inf_1 = __webpack_require__("./libs/language-server/src/lib/meta-information/constraint-meta-inf.ts");
exports.metaInformationRegistry = new registry_1.Registry();
function registerMetaInformation(metaInfClass) {
    const metaInf = new metaInfClass();
    exports.metaInformationRegistry.register(metaInf.type, metaInf);
}
exports.registerMetaInformation = registerMetaInformation;
function getMetaInformation(type) {
    const typeString = type === null || type === void 0 ? void 0 : type.name;
    if (typeString === undefined) {
        return undefined;
    }
    (0, assert_1.strict)(type !== undefined);
    const metaInf = exports.metaInformationRegistry.get(typeString);
    if (metaInf === undefined) {
        return undefined;
    }
    if ((0, ast_1.isBlockTypeLiteral)(type)) {
        (0, assert_1.strict)(metaInf instanceof block_meta_inf_1.BlockMetaInformation);
        return metaInf;
    }
    if ((0, ast_1.isConstraintTypeLiteral)(type)) {
        (0, assert_1.strict)(metaInf instanceof constraint_meta_inf_1.ConstraintMetaInformation);
        return metaInf;
    }
    (0, errors_1.assertUnreachable)(type);
}
exports.getMetaInformation = getMetaInformation;
function getRegisteredBlockMetaInformation() {
    return exports.metaInformationRegistry
        .getAll()
        .filter((metaInf) => metaInf instanceof block_meta_inf_1.BlockMetaInformation);
}
exports.getRegisteredBlockMetaInformation = getRegisteredBlockMetaInformation;
function getRegisteredConstraintMetaInformation() {
    return exports.metaInformationRegistry
        .getAll()
        .filter((metaInf) => metaInf instanceof constraint_meta_inf_1.ConstraintMetaInformation);
}
exports.getRegisteredConstraintMetaInformation = getRegisteredConstraintMetaInformation;
function getOrFailMetaInformation(type) {
    const result = getMetaInformation(type);
    (0, assert_1.strict)(result !== undefined, `Meta information for type ${type.name} was expected to be present, got undefined instead`);
    return result;
}
exports.getOrFailMetaInformation = getOrFailMetaInformation;


/***/ }),

/***/ "./libs/language-server/src/lib/meta-information/meta-inf.ts":
/***/ ((__unused_webpack_module, exports) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.MetaInformation = void 0;
class MetaInformation {
    constructor(type, properties, validation) {
        this.type = type;
        this.properties = properties;
        this.validation = validation;
    }
    validate(propertyBody, accept) {
        for (const property of propertyBody.properties) {
            const propertySpecification = this.getPropertySpecification(property.name);
            const propertyValidationFn = propertySpecification === null || propertySpecification === void 0 ? void 0 : propertySpecification.validation;
            if (propertyValidationFn === undefined) {
                continue;
            }
            propertyValidationFn(property, accept);
        }
        if (this.validation !== undefined) {
            this.validation(propertyBody, accept);
        }
    }
    getPropertySpecification(name) {
        return this.properties[name];
    }
    getPropertySpecifications() {
        return this.properties;
    }
    hasPropertySpecification(name) {
        return this.getPropertySpecification(name) !== undefined;
    }
    getPropertyNames(kind = undefined, excludeNames = []) {
        const resultingPropertyNames = [];
        for (const [name, spec] of Object.entries(this.properties)) {
            if (kind === 'optional' && spec.defaultValue === undefined) {
                continue;
            }
            if (kind === 'required' && spec.defaultValue !== undefined) {
                continue;
            }
            if (excludeNames.includes(name)) {
                continue;
            }
            resultingPropertyNames.push(name);
        }
        return resultingPropertyNames;
    }
}
exports.MetaInformation = MetaInformation;


/***/ }),

/***/ "./libs/language-server/src/lib/util/constructor-class.ts":
/***/ ((__unused_webpack_module, exports) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));


/***/ }),

/***/ "./libs/language-server/src/lib/util/index.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
const tslib_1 = __webpack_require__("tslib");
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/util/constructor-class.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/util/registry.ts"), exports);


/***/ }),

/***/ "./libs/language-server/src/lib/util/registry.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.Registry = void 0;
const assert_1 = __webpack_require__("assert");
class Registry {
    constructor() {
        this.registry = new Map();
    }
    register(key, classToRegister) {
        (0, assert_1.strict)(!this.registry.has(key), `Multiple keys "${key}" were registered, expected at most one register call per key`);
        this.registry.set(key, classToRegister);
    }
    getAll() {
        return [...this.registry.values()];
    }
    get(key) {
        return this.registry.get(key);
    }
}
exports.Registry = Registry;


/***/ }),

/***/ "./libs/language-server/src/lib/validation/index.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
const tslib_1 = __webpack_require__("tslib");
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/validation/validation-registry.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/validation/validation-util.ts"), exports);


/***/ }),

/***/ "./libs/language-server/src/lib/validation/validation-registry.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.JayveeValidationRegistry = void 0;
const langium_1 = __webpack_require__("langium");
/**
 * Registry for validation checks.
 */
class JayveeValidationRegistry extends langium_1.ValidationRegistry {
    constructor(services) {
        super(services);
        const modelValidator = services.validation.JayveeModelValidator;
        this.register(modelValidator.checks, modelValidator);
        const pipelineValidator = services.validation.PipelineDefinitionValidator;
        this.register(pipelineValidator.checks, pipelineValidator);
        const pipeValidator = services.validation.PipeValidator;
        this.register(pipeValidator.checks, pipeValidator);
        const blockValidator = services.validation.BlockValidator;
        this.register(blockValidator.checks, blockValidator);
        const propertyBodyValidator = services.validation.PropertyBodyValidator;
        this.register(propertyBodyValidator.checks, propertyBodyValidator);
        const cellRangeSelectionValidator = services.validation.CellRangeSelectionValidator;
        this.register(cellRangeSelectionValidator.checks, cellRangeSelectionValidator);
        const columnExpressionValidator = services.validation.ColumnExpressionValidator;
        this.register(columnExpressionValidator.checks, columnExpressionValidator);
        const regexValueValidator = services.validation.RegexValueValidator;
        this.register(regexValueValidator.checks, regexValueValidator);
        const constraintValidator = services.validation.ConstraintValidator;
        this.register(constraintValidator.checks, constraintValidator);
        const valuetypeValidator = services.validation.ValuetypeValidator;
        this.register(valuetypeValidator.checks, valuetypeValidator);
    }
}
exports.JayveeValidationRegistry = JayveeValidationRegistry;


/***/ }),

/***/ "./libs/language-server/src/lib/validation/validation-util.ts":
/***/ ((__unused_webpack_module, exports) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.getNodesWithNonUniqueNames = exports.generateNonUniqueNameErrorMessage = void 0;
function generateNonUniqueNameErrorMessage(node) {
    return `The ${node.$type.toLowerCase()} name "${node.name}" needs to be unique.`;
}
exports.generateNonUniqueNameErrorMessage = generateNonUniqueNameErrorMessage;
function getNodesWithNonUniqueNames(nodes) {
    const nodesByName = getNodesByName(nodes);
    const resultingNodes = [];
    for (const nodesWithSameName of Object.values(nodesByName)) {
        if (nodesWithSameName.length > 1) {
            resultingNodes.push(...nodesWithSameName);
        }
    }
    return resultingNodes;
}
exports.getNodesWithNonUniqueNames = getNodesWithNonUniqueNames;
function getNodesByName(nodes) {
    return groupBy(nodes, (node) => node.name);
}
function groupBy(elements, keyFn) {
    const initialValue = {};
    return elements.reduce((result, element) => {
        const key = keyFn(element);
        if (key === undefined) {
            return result;
        }
        const array = result[key];
        // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
        if (array === undefined) {
            result[key] = [];
        }
        result[key].push(element);
        return result;
    }, initialValue);
}


/***/ }),

/***/ "./libs/language-server/src/lib/validation/validators/block-validator.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.BlockValidator = void 0;
/**
 * See the FAQ section of README.md for an explanation why the following ESLint rule is disabled for this file.
 */
/* eslint-disable @typescript-eslint/no-unnecessary-condition */
const langium_1 = __webpack_require__("langium");
const ast_1 = __webpack_require__("./libs/language-server/src/lib/ast/index.ts");
const meta_inf_registry_1 = __webpack_require__("./libs/language-server/src/lib/meta-information/meta-inf-registry.ts");
class BlockValidator {
    get checks() {
        return {
            BlockDefinition: [
                this.checkIngoingPipes,
                this.checkOutgoingPipes,
                this.checkBlockType,
            ],
        };
    }
    checkIngoingPipes(block, accept) {
        BlockValidator.checkPipesOfBlock(block, 'input', accept);
    }
    checkOutgoingPipes(block, accept) {
        BlockValidator.checkPipesOfBlock(block, 'output', accept);
    }
    static checkPipesOfBlock(block, whatToCheck, accept) {
        const blockMetaInf = (0, meta_inf_registry_1.getMetaInformation)(block.type);
        if (blockMetaInf === undefined) {
            return;
        }
        let pipes;
        switch (whatToCheck) {
            case 'input': {
                pipes = (0, ast_1.collectIngoingPipes)(block);
                break;
            }
            case 'output': {
                pipes = (0, ast_1.collectOutgoingPipes)(block);
                break;
            }
            default: {
                (0, langium_1.assertUnreachable)(whatToCheck);
            }
        }
        if ((whatToCheck === 'input' && !blockMetaInf.hasInput()) ||
            (whatToCheck === 'output' && !blockMetaInf.hasOutput())) {
            for (const pipe of pipes) {
                accept('error', `Blocks of type ${blockMetaInf.type} do not have an ${whatToCheck}`, whatToCheck === 'input'
                    ? pipe.getToDiagnostic()
                    : pipe.getFromDiagnostic());
            }
        }
        else if (pipes.length > 1 && whatToCheck === 'input') {
            for (const pipe of pipes) {
                accept('error', `At most one pipe can be connected to the ${whatToCheck} of a ${blockMetaInf.type}`, pipe.getToDiagnostic());
            }
        }
        else if (pipes.length === 0) {
            accept('warning', `A pipe should be connected to the ${whatToCheck} of this block`, {
                node: block,
                property: 'name',
            });
        }
    }
    checkBlockType(block, accept) {
        var _a, _b;
        if (block.type === undefined) {
            return;
        }
        const metaInf = (0, meta_inf_registry_1.getMetaInformation)(block.type);
        if (metaInf === undefined) {
            accept('error', `Unknown block type '${(_b = (_a = block === null || block === void 0 ? void 0 : block.type) === null || _a === void 0 ? void 0 : _a.name) !== null && _b !== void 0 ? _b : ''}'`, {
                node: block,
                property: 'type',
            });
        }
    }
}
exports.BlockValidator = BlockValidator;


/***/ }),

/***/ "./libs/language-server/src/lib/validation/validators/cell-range-selection-validator.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.CellRangeSelectionValidator = void 0;
const cell_range_wrapper_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/cell-range-wrapper.ts");
class CellRangeSelectionValidator {
    get checks() {
        return {
            RangeLiteral: [this.checkRangeLimits],
        };
    }
    checkRangeLimits(range, accept) {
        if (range.cellFrom === undefined || range.cellTo === undefined) {
            return;
        }
        const semanticCellRange = new cell_range_wrapper_1.CellRangeWrapper(range);
        if (semanticCellRange.from.columnIndex > semanticCellRange.to.columnIndex ||
            semanticCellRange.from.rowIndex > semanticCellRange.to.rowIndex) {
            accept('error', `Cell ranges need to be spanned from top-left to bottom-right`, {
                node: semanticCellRange.astNode,
            });
        }
    }
}
exports.CellRangeSelectionValidator = CellRangeSelectionValidator;


/***/ }),

/***/ "./libs/language-server/src/lib/validation/validators/column-expression-validator.ts":
/***/ ((__unused_webpack_module, exports) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.ColumnExpressionValidator = void 0;
class ColumnExpressionValidator {
    get checks() {
        return {
            ColumnLiteral: [this.checkColumnIdSyntax],
        };
    }
    checkColumnIdSyntax(columnExpression, accept) {
        if (columnExpression.columnId === undefined) {
            return;
        }
        const columnIdRegex = /^([A-Z]+|\*)$/;
        if (!columnIdRegex.test(columnExpression.columnId)) {
            accept('error', 'Columns need to be denoted via capital letters or the * character', {
                node: columnExpression,
                property: 'columnId',
            });
        }
    }
}
exports.ColumnExpressionValidator = ColumnExpressionValidator;


/***/ }),

/***/ "./libs/language-server/src/lib/validation/validators/constraint-validator.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.ConstraintValidator = void 0;
const meta_inf_registry_1 = __webpack_require__("./libs/language-server/src/lib/meta-information/meta-inf-registry.ts");
class ConstraintValidator {
    get checks() {
        return {
            ConstraintDefinition: [this.checkConstraintType],
        };
    }
    checkConstraintType(constraint, accept) {
        var _a;
        const metaInf = (0, meta_inf_registry_1.getMetaInformation)(constraint.type);
        if (metaInf === undefined) {
            accept('error', `Unknown constraint type '${(_a = constraint.type.name) !== null && _a !== void 0 ? _a : ''}'`, {
                node: constraint,
                property: 'type',
            });
        }
    }
}
exports.ConstraintValidator = ConstraintValidator;


/***/ }),

/***/ "./libs/language-server/src/lib/validation/validators/model-validator.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.JayveeModelValidator = void 0;
const validation_util_1 = __webpack_require__("./libs/language-server/src/lib/validation/validation-util.ts");
class JayveeModelValidator {
    get checks() {
        return {
            JayveeModel: [this.checkUniquePipelineDefinitionNames],
        };
    }
    checkUniquePipelineDefinitionNames(model, accept) {
        (0, validation_util_1.getNodesWithNonUniqueNames)(model.pipelines).forEach((pipeline) => {
            accept('error', (0, validation_util_1.generateNonUniqueNameErrorMessage)(pipeline), {
                node: pipeline,
                property: 'name',
            });
        });
    }
}
exports.JayveeModelValidator = JayveeModelValidator;


/***/ }),

/***/ "./libs/language-server/src/lib/validation/validators/pipe-validator.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.PipeValidator = void 0;
const pipe_wrapper_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/pipe-wrapper.ts");
const meta_inf_registry_1 = __webpack_require__("./libs/language-server/src/lib/meta-information/meta-inf-registry.ts");
class PipeValidator {
    get checks() {
        return {
            PipeDefinition: this.checkBlockCompatibility,
        };
    }
    checkBlockCompatibility(pipe, accept) {
        const semanticPipes = (0, pipe_wrapper_1.createSemanticPipes)(pipe);
        for (const semanticPipe of semanticPipes) {
            const fromBlockType = semanticPipe.from.type;
            const toBlockType = semanticPipe.to.type;
            const fromBlockMetaInf = (0, meta_inf_registry_1.getMetaInformation)(fromBlockType);
            const toBlockMetaInf = (0, meta_inf_registry_1.getMetaInformation)(toBlockType);
            if (fromBlockMetaInf === undefined || toBlockMetaInf === undefined) {
                continue;
            }
            if (fromBlockMetaInf.hasOutput() && toBlockMetaInf.hasInput()) {
                if (!fromBlockMetaInf.canBeConnectedTo(toBlockMetaInf)) {
                    const errorMessage = `The output type "${fromBlockMetaInf.outputType}" of ${fromBlockMetaInf.type} is incompatible with the input type "${toBlockMetaInf.inputType}" of ${toBlockMetaInf.type}`;
                    accept('error', errorMessage, semanticPipe.getFromDiagnostic());
                    accept('error', errorMessage, semanticPipe.getToDiagnostic());
                }
            }
        }
    }
}
exports.PipeValidator = PipeValidator;


/***/ }),

/***/ "./libs/language-server/src/lib/validation/validators/pipeline-validator.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.PipelineDefinitionValidator = void 0;
const model_util_1 = __webpack_require__("./libs/language-server/src/lib/ast/model-util.ts");
const validation_util_1 = __webpack_require__("./libs/language-server/src/lib/validation/validation-util.ts");
class PipelineDefinitionValidator {
    get checks() {
        return {
            PipelineDefinition: [
                this.checkStartingBlocks,
                this.checkUniqueBlockNames,
            ],
        };
    }
    checkStartingBlocks(pipeline, accept) {
        const startingBlocks = (0, model_util_1.collectStartingBlocks)(pipeline);
        if (startingBlocks.length === 0) {
            accept('error', `An extractor block is required for this pipeline`, {
                node: pipeline,
                property: 'name',
            });
        }
    }
    checkUniqueBlockNames(pipeline, accept) {
        (0, validation_util_1.getNodesWithNonUniqueNames)(pipeline.blocks).forEach((block) => {
            accept('error', (0, validation_util_1.generateNonUniqueNameErrorMessage)(block), {
                node: block,
                property: 'name',
            });
        });
    }
}
exports.PipelineDefinitionValidator = PipelineDefinitionValidator;


/***/ }),

/***/ "./libs/language-server/src/lib/validation/validators/property-body-validator.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.PropertyBodyValidator = void 0;
const ast_1 = __webpack_require__("./libs/language-server/src/lib/ast/index.ts");
const meta_inf_registry_1 = __webpack_require__("./libs/language-server/src/lib/meta-information/meta-inf-registry.ts");
const validation_util_1 = __webpack_require__("./libs/language-server/src/lib/validation/validation-util.ts");
class PropertyBodyValidator {
    get checks() {
        return {
            PropertyBody: [
                this.checkPropertyNames,
                this.checkUniquePropertyNames,
                this.checkPropertyTyping,
                this.checkPropertyCompleteness,
                this.checkCustomPropertyValidation,
            ],
        };
    }
    checkPropertyNames(propertyBody, accept) {
        const metaInf = inferMetaInformation(propertyBody);
        if (metaInf === undefined) {
            return;
        }
        for (const property of propertyBody.properties) {
            if (!metaInf.hasPropertySpecification(property.name)) {
                accept('error', `Invalid property name "${property.name}".`, {
                    node: property,
                    property: 'name',
                });
            }
        }
    }
    checkUniquePropertyNames(propertyBody, accept) {
        (0, validation_util_1.getNodesWithNonUniqueNames)(propertyBody.properties).forEach((property) => {
            accept('error', (0, validation_util_1.generateNonUniqueNameErrorMessage)(property), {
                node: property,
                property: 'name',
            });
        });
    }
    checkPropertyTyping(propertyBody, accept) {
        const metaInf = inferMetaInformation(propertyBody);
        if (metaInf === undefined) {
            return;
        }
        for (const property of propertyBody.properties) {
            const propertySpec = metaInf.getPropertySpecification(property.name);
            if (propertySpec === undefined) {
                continue;
            }
            const propertyType = propertySpec.type;
            if (property.value === undefined) {
                continue;
            }
            const propertyValue = property.value;
            if ((0, ast_1.isRuntimeParameterLiteral)(propertyValue)) {
                if (!(0, ast_1.runtimeParameterAllowedForType)(propertyType)) {
                    accept('error', `Runtime parameters are not allowed for properties of type ${propertyType}`, {
                        node: property,
                        property: 'name',
                    });
                }
            }
            else {
                const matchingPropertyTypes = (0, ast_1.inferTypesFromValue)(propertyValue);
                if (!matchingPropertyTypes.includes(propertyType)) {
                    accept('error', `The value needs to be of type ${propertyType}`, {
                        node: property,
                        property: 'value',
                    });
                }
            }
        }
    }
    checkPropertyCompleteness(propertyBody, accept) {
        const metaInf = inferMetaInformation(propertyBody);
        if (metaInf === undefined) {
            return;
        }
        const presentPropertyNames = propertyBody.properties.map((property) => property.name);
        const missingRequiredPropertyNames = metaInf.getPropertyNames('required', presentPropertyNames);
        if (missingRequiredPropertyNames.length > 0) {
            accept('error', `The following required properties are missing: ${missingRequiredPropertyNames
                .map((name) => `"${name}"`)
                .join(', ')}`, {
                node: propertyBody.$container,
                property: 'type',
            });
        }
    }
    checkCustomPropertyValidation(propertyBody, accept) {
        const metaInf = inferMetaInformation(propertyBody);
        if (metaInf === undefined) {
            return;
        }
        metaInf.validate(propertyBody, accept);
    }
}
exports.PropertyBodyValidator = PropertyBodyValidator;
function inferMetaInformation(propertyBody) {
    const type = propertyBody.$container.type;
    return (0, meta_inf_registry_1.getMetaInformation)(type);
}


/***/ }),

/***/ "./libs/language-server/src/lib/validation/validators/regex-value-validator.ts":
/***/ ((__unused_webpack_module, exports) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.RegexValueValidator = void 0;
class RegexValueValidator {
    get checks() {
        return {
            RegexLiteral: [this.checkRegexParsability],
        };
    }
    checkRegexParsability(regex, accept) {
        try {
            new RegExp(regex.value);
        }
        catch (error) {
            if (error instanceof SyntaxError) {
                accept('error', `A parsing error occurred: ${error.message}`, {
                    node: regex,
                });
            }
            else {
                accept('error', `An unknown parsing error occurred: ${JSON.stringify(error)}.`, {
                    node: regex,
                });
            }
        }
    }
}
exports.RegexValueValidator = RegexValueValidator;


/***/ }),

/***/ "./libs/language-server/src/lib/validation/validators/valuetype-validator.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.ValuetypeValidator = void 0;
const ast_1 = __webpack_require__("./libs/language-server/src/lib/ast/index.ts");
const meta_inf_registry_1 = __webpack_require__("./libs/language-server/src/lib/meta-information/meta-inf-registry.ts");
class ValuetypeValidator {
    get checks() {
        return {
            ValuetypeDefinition: [
                this.checkConstraintsCollectionValues,
                this.checkConstraintsMatchPrimitiveValuetype,
            ],
        };
    }
    checkConstraintsCollectionValues(valuetype, accept) {
        const constraints = valuetype.constraints;
        constraints.values.forEach((collectionValue) => {
            const types = (0, ast_1.inferTypesFromValue)(collectionValue);
            if (!types.includes(ast_1.PropertyValuetype.CONSTRAINT)) {
                accept('error', 'Only constraints are allowed in this collection', {
                    node: collectionValue,
                });
            }
        });
    }
    checkConstraintsMatchPrimitiveValuetype(valuetype, accept) {
        var _a;
        if (valuetype.type === undefined) {
            return;
        }
        const constraintReferences = (_a = valuetype === null || valuetype === void 0 ? void 0 : valuetype.constraints) === null || _a === void 0 ? void 0 : _a.values.filter(ast_1.isConstraintReferenceLiteral);
        for (const constraintReference of constraintReferences) {
            const constraint = constraintReference === null || constraintReference === void 0 ? void 0 : constraintReference.value.ref;
            const constraintType = constraint === null || constraint === void 0 ? void 0 : constraint.type;
            if (constraintType === undefined) {
                continue;
            }
            const metaInf = (0, meta_inf_registry_1.getMetaInformation)(constraintType);
            if (metaInf === undefined) {
                continue;
            }
            if (!metaInf.compatiblePrimitiveValuetypes.includes(valuetype.type.keyword)) {
                accept('error', `Only constraints for type "${valuetype.type.keyword}" are allowed in this collection`, {
                    node: constraintReference,
                });
            }
        }
    }
}
exports.ValuetypeValidator = ValuetypeValidator;


/***/ }),

/***/ "@fast-csv/parse":
/***/ ((module) => {

module.exports = require("@fast-csv/parse");

/***/ }),

/***/ "assert":
/***/ ((module) => {

module.exports = require("assert");

/***/ }),

/***/ "chalk":
/***/ ((module) => {

module.exports = require("chalk");

/***/ }),

/***/ "commander":
/***/ ((module) => {

module.exports = require("commander");

/***/ }),

/***/ "fp-ts/lib/Either":
/***/ ((module) => {

module.exports = require("fp-ts/lib/Either");

/***/ }),

/***/ "jszip":
/***/ ((module) => {

module.exports = require("jszip");

/***/ }),

/***/ "langium":
/***/ ((module) => {

module.exports = require("langium");

/***/ }),

/***/ "langium/lib/utils/errors":
/***/ ((module) => {

module.exports = require("langium/lib/utils/errors");

/***/ }),

/***/ "langium/node":
/***/ ((module) => {

module.exports = require("langium/node");

/***/ }),

/***/ "mime-types":
/***/ ((module) => {

module.exports = require("mime-types");

/***/ }),

/***/ "pg":
/***/ ((module) => {

module.exports = require("pg");

/***/ }),

/***/ "sqlite3":
/***/ ((module) => {

module.exports = require("sqlite3");

/***/ }),

/***/ "tslib":
/***/ ((module) => {

module.exports = require("tslib");

/***/ }),

/***/ "util":
/***/ ((module) => {

module.exports = require("util");

/***/ }),

/***/ "vscode-languageserver":
/***/ ((module) => {

module.exports = require("vscode-languageserver");

/***/ }),

/***/ "vscode-languageserver-protocol":
/***/ ((module) => {

module.exports = require("vscode-languageserver-protocol");

/***/ }),

/***/ "vscode-languageserver-types":
/***/ ((module) => {

module.exports = require("vscode-languageserver-types");

/***/ }),

/***/ "vscode-uri":
/***/ ((module) => {

module.exports = require("vscode-uri");

/***/ }),

/***/ "fs":
/***/ ((module) => {

module.exports = require("fs");

/***/ }),

/***/ "https":
/***/ ((module) => {

module.exports = require("https");

/***/ }),

/***/ "path":
/***/ ((module) => {

module.exports = require("path");

/***/ }),

/***/ "./apps/interpreter/package.json":
/***/ ((module) => {

module.exports = JSON.parse('{"name":"@jvalue/jayvee-interpreter","version":"0.0.12","description":"Interpreter for .jv files","main":"main.js","bin":{"jv":"main.js","jayvee":"main.js"},"bugs":{"url":"https://github.com/jvalue/jayvee/issues"},"repository":{"type":"git","url":"https://github.com/jvalue/jayvee.git"},"homepage":"https://github.com/jvalue/jayvee","publishConfig":{"access":"public","registry":"https://registry.npmjs.org"}}');

/***/ })

/******/ 	});
/************************************************************************/
/******/ 	// The module cache
/******/ 	var __webpack_module_cache__ = {};
/******/ 	
/******/ 	// The require function
/******/ 	function __webpack_require__(moduleId) {
/******/ 		// Check if module is in cache
/******/ 		var cachedModule = __webpack_module_cache__[moduleId];
/******/ 		if (cachedModule !== undefined) {
/******/ 			return cachedModule.exports;
/******/ 		}
/******/ 		// Create a new module (and put it into the cache)
/******/ 		var module = __webpack_module_cache__[moduleId] = {
/******/ 			// no module.id needed
/******/ 			// no module.loaded needed
/******/ 			exports: {}
/******/ 		};
/******/ 	
/******/ 		// Execute the module function
/******/ 		__webpack_modules__[moduleId](module, module.exports, __webpack_require__);
/******/ 	
/******/ 		// Return the exports of the module
/******/ 		return module.exports;
/******/ 	}
/******/ 	
/************************************************************************/
var __webpack_exports__ = {};
// This entry need to be wrapped in an IIFE because it need to be isolated against other modules in the chunk.
(() => {
var exports = __webpack_exports__;

// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
const commander_1 = __webpack_require__("commander");
const package_json_1 = __webpack_require__("./apps/interpreter/package.json");
const interpreter_1 = __webpack_require__("./apps/interpreter/src/interpreter.ts");
const runtimeParameterRegex = /^([_a-zA-Z][\w_]*)=(.*)$/;
function collectRuntimeParameters(optionValue, previous) {
    const regexMatch = optionValue.match(runtimeParameterRegex);
    if (regexMatch == null) {
        throw new Error(`Encountered runtime parameter with invalid syntax: ${optionValue}`);
    }
    // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
    const parameter = regexMatch[1];
    // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
    const value = regexMatch[2];
    previous.set(parameter, value);
    return previous;
}
const program = new commander_1.Command();
const version = package_json_1.version;
program.version(version);
program
    .argument('<file>', `path to the .jv source file`)
    .option('-e, --env <parameter>=<value>', 'provide a runtime parameters', collectRuntimeParameters, new Map())
    .option('-d, --debug', 'enable debug logging', false)
    .description('Run a Jayvee file')
    .action(interpreter_1.runAction);
program.showHelpAfterError();
program.parse(process.argv);

})();

var __webpack_export_target__ = exports;
for(var i in __webpack_exports__) __webpack_export_target__[i] = __webpack_exports__[i];
if(__webpack_exports__.__esModule) Object.defineProperty(__webpack_export_target__, "__esModule", { value: true });
/******/ })()
;
//# sourceMappingURL=main.js.map